/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CAccountTemplatesFile.h

  Description :	

  Last Modified $Date: 2008/09/29 04:22:27 $

  $Revision: 1.4 $

*******************************************************************************/

#ifndef CACCOUNTTEMPLATESFILE_H_
#define CACCOUNTTEMPLATESFILE_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CString.h"
#include "CTypes.h"
#include "tinyxml.h"
#include <stdio.h>

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************
class CAccountTemplate;
class CAccountTemplateParam;

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************

//******************************************************************************
//******************************************************************************
//
//	Account Templates file reader class
//
//******************************************************************************
//******************************************************************************
class CAccountTemplatesFile : public CBase
{
	public:
		static bool						Open();
		static void						Close();

	public:
		typedef std::vector< CAccountTemplate * >	CAccountTemplateList;

	public:
		static CAccountTemplateList *s_pAccountTemplates;

};

class CAccountTemplate : public CBase
{
	public:

		CAccountTemplate()
		{
			p_paramList = new CAccountTemplateParamList();
		}
		
		CAccountTemplate(CAccountTemplate* account);
		virtual ~CAccountTemplate();

	public:
		typedef std::vector< CAccountTemplateParam * >	CAccountTemplateParamList;

	public:
		// Replaces any params in the string with the values from the parameter list
		CString 	replaceWithParams(CString str);

	public:
		CString		template_name;
		CString		display_name;
		
		CString		email_address;
		CString		account_type;
		CString		incoming_server;
		CString		incoming_server_port;
		CString		incoming_server_username;
		CString		incoming_server_password;
		CString		incoming_server_tls;
		CString		outgoing_server;
		CString		outgoing_server_port;
		CString		outgoing_server_username;
		CString		outgoing_server_password;
		CString		outgoing_server_tls;
		CString		leave_messages_on_server;
		CString		reply_to;
		
		CAccountTemplateParamList *p_paramList;
};

class CAccountTemplateParam : public CBase
{
	public:
		CAccountTemplateParam() 
		{
			
		}
		
		~CAccountTemplateParam()
		{
			//destroy each item in list
			//destroy list
		}
		
		CAccountTemplateParam(CAccountTemplateParam* param)
		{
			name = param->name;
			prompt = param->prompt;
			value = param->value;
		}
	
	public:
		CString		name;
		CString		prompt;
		CString		value;	
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CCONFIGFILE_H_ */
