/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CBase.h

  Description :	

  Last Modified $Date: 2008/10/06 04:50:49 $

  $Revision: 1.2 $

*******************************************************************************/

#ifndef CBASE_H_
#define CBASE_H_

#ifdef INSTRUMENTED_ALLOCS

#include <psptypes.h>
#include <pspkerneltypes.h>

#include <new>

/* forward declaration for the friend statement */
class AllocHeaderList;
class CBase;

class AllocHeader
{
	friend class AllocHeaderList;
	friend class CBase;
	
	public:
		int magic;
		int blockSize;
		int allocCount;
		
		AllocHeader(int aBlockSize, int aAllocCount);
		void poison();
	
	private:
		struct AllocHeader* m_next;
		struct AllocHeader* m_prev;
};

class AllocHeaderList
{
	friend class CBase;
	
	public:
		AllocHeaderList();
		void append(AllocHeader* allocHeader);
		void remove(AllocHeader* allocHeader);
		
	private:
		AllocHeader* m_head;
		AllocHeader* m_tail;
};

class CBase
{
	public:
		static bool Open();
		static void Close();
		
		static void* operator new(size_t size); 
		static void operator delete(void *p);
		
		static void printLiveMemory();
		
	private:
		static void lockList();
		static void unlockList();
		
		static AllocHeaderList s_allocHeaderList;
		static SceUID s_listSema;
		static u32 s_nextAllocCount;
		
};

#else /* INSTRUMENTED_ALLOCS */

class CBase
{
	public:
		static bool Open() { return true; }
		static void Close() {};
};

#endif /* INSTRUMENTED_ALLOCS */

#endif /* CBASE_H_ */
