/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CCachedBase.h

  Description :	

  Last Modified $Date: 2007/01/03 05:31:14 $

  $Revision: 1.1 $

*******************************************************************************/
#ifndef CCACHEDBASE_H_
#define CCACHEDBASE_H_

#include "CTypes.h"
#include "CString.h"

#define CACHE_FILE_VERSION 1

class CFile;

class CCachedBase
{
	public: /* Constructors & destructors */
	
		CCachedBase(CString resourcePath, CString cachedPath);
		virtual ~CCachedBase();
		
	public: /* New methods */
	
		virtual bool isDirty();
		virtual bool isCacheValid();
		virtual bool isResourceValid();
		virtual u32 getMagicNumber() = 0; /* Magic number for storing the cached texture */
		virtual CString getCacheExtension() = 0; /* Extension to append to the resource name when caching */
		virtual bool initFromSource() = 0; /* Reads the texture from the source */
		virtual bool initFromCache(); /* Reads the texture from the cache */
		virtual bool cache();
		
	protected: /* New methods */
	
		bool isValidDateTime(ScePspDateTime* dateTime);
		void fetchDates();
		bool readFromCache(void* dest, const CFile* p_file, u32 size);
		bool writeToCache(void* src, const CFile* p_file, u32 size);
		
	protected: /* Data */
	
		CString m_resourcePath;
		CString m_cachedPath;
		ScePspDateTime m_resourceDate;
		ScePspDateTime m_cachedDate;
		
	protected: /* Temporary data for keeping the state */
	
		CFile* m_pFile; /* Used to keep a handle of the opened cached file during initialization and on clean up */
};

#endif /*CCACHEDBASE_H_*/
