/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CCachedFont.cpp

  Description :	

  Last Modified $Date: 2007/01/03 05:31:14 $

  $Revision: 1.1 $

*******************************************************************************/
#include "CCachedFont.h"
#include "CFileSystem.h"
#include <malloc.h> // For memalign

typedef struct FontCacheFormat {
	u32 width;
	u32 height;
	u32 canvasWidth;
	u32 canvasHeight;
	bool swizzled;
	u32 fontSize;
	u32 lineHeight;
	u32 metricTableSize;
} FontCacheFormat;

//******************************************************************************
//	
//******************************************************************************
CCachedFont::CCachedFont(CString resourcePath, u32 size) :
	CFont(size),
	CCachedBase(resourcePath, resourcePath + "." + getCacheExtension())
{
}

//******************************************************************************
//	
//******************************************************************************
CCachedFont::~CCachedFont()
{
	if (isDirty() && isResourceValid())
		cache();
}

//******************************************************************************
//	
//******************************************************************************
CString CCachedFont::getCacheExtension()
{
	CString ext;
	ext.Printf("fcache%d", m_fontSize);
	return ext;
}

//******************************************************************************
//	
//******************************************************************************
bool CCachedFont::initFromSource()
{
	return (CTextureManager::CreateFont(m_resourcePath, m_fontSize, this) != NULL);
}

//******************************************************************************
//	
//******************************************************************************
bool CCachedFont::initFromCache()
{
	/* Try to open the cached file */
	m_pFile = CFileSystem::Open(m_cachedPath, "rb");
	if (m_pFile == NULL)
		return false;
		
	/* With the cache file open, call the base class' initFromCache */
	bool toRet = false;
	if (CCachedBase::initFromCache())
	{
		/* Read the font info */
		FontCacheFormat readBuf;
		toRet = readFromCache(&readBuf, m_pFile, sizeof(FontCacheFormat));

		if (toRet)
		{
			Init(readBuf.width, readBuf.height);
			m_swizzled = readBuf.swizzled;
			m_lineHeight = readBuf.lineHeight;
			m_metricTableSize = readBuf.metricTableSize;
			
			/* Read the metrics table */
			CFont::Metrics* metrics = (CFont::Metrics*)memalign(16, m_metricTableSize * sizeof(CFont::Metrics));
			toRet = readFromCache(metrics, m_pFile, m_metricTableSize * sizeof(CFont::Metrics));
			if (toRet)
			{
				SetMetrics(metrics, m_metricTableSize);
			
				/* Read the actual font texture */
				toRet = readFromCache(m_pBuffer, m_pFile, (m_nCanvasWidth * m_nCanvasHeight * m_bpp));
			}
			else
			{
				free(metrics);
				metrics = NULL;
			}
		}
	}

	/* Cleanup */
	CFileSystem::Close(m_pFile);
	m_pFile = NULL;
	return toRet;
}

//******************************************************************************
//	
//******************************************************************************
bool CCachedFont::cache()
{
	/* Try to open the cache file */
	m_pFile = CFileSystem::Open(m_cachedPath, "wb");
	if (m_pFile == NULL)
		return false;
	
	/* With the cache file open, call the base class' cache */
	bool toRet = false;
	if (CCachedBase::cache())
	{
		/* Write the font info */
		FontCacheFormat writeBuf;
		writeBuf.width = m_nWidth;
		writeBuf.height = m_nHeight;
		writeBuf.canvasWidth = m_nCanvasWidth;
		writeBuf.canvasHeight = m_nCanvasHeight;
		writeBuf.swizzled = m_swizzled;
		writeBuf.fontSize = m_fontSize;
		writeBuf.lineHeight = m_lineHeight;
		writeBuf.metricTableSize = m_metricTableSize;
		toRet = writeToCache(&writeBuf, m_pFile, sizeof(FontCacheFormat));
		
		if (toRet)
		{
			/* Write the metrics table */
			toRet = writeToCache(m_fontMetrics, m_pFile, m_metricTableSize * sizeof(CFont::Metrics));
		
			if (toRet)
			{
				/* Write the actual font texture */
				writeToCache(m_pBuffer, m_pFile, (m_nCanvasWidth * m_nCanvasHeight * m_bpp));
			}
		}
	}
	
	/* Cleanup */
	CFileSystem::Close(m_pFile);
	m_pFile = NULL;
	return toRet;
}
