/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CCachedTexture.cpp

  Description :	

  Last Modified $Date: 2007/01/03 05:31:14 $

  $Revision: 1.1 $

*******************************************************************************/
#include "CCachedTexture.h"
#include "CFileSystem.h"

typedef struct TextureCacheFormat {
	u32 width;
	u32 height;
	u32 canvasWidth;
	u32 canvasHeight;
	bool swizzled;
} TextureCacheFormat;

//******************************************************************************
//	
//******************************************************************************
CCachedTexture::CCachedTexture(CString resourcePath, CTexture::Format format) :
	CTexture(format),
	CCachedBase(resourcePath, resourcePath + "." + getCacheExtension())
{
}

//******************************************************************************
//	
//******************************************************************************
CCachedTexture::~CCachedTexture()
{
	if (isDirty() && isResourceValid())
		cache();
}

//******************************************************************************
//	
//******************************************************************************
bool CCachedTexture::initFromSource()
{
	return (CTextureManager::Create(m_resourcePath, false, this) != NULL);
}

//******************************************************************************
//	
//******************************************************************************
bool CCachedTexture::initFromCache()
{
	/* Try to open the cached file */
	m_pFile = CFileSystem::Open(m_cachedPath, "rb");
	if (m_pFile == NULL)
		return false;
		
	/* With the cache file open, call the base class' initFromCache */
	bool toRet = false;
	if (CCachedBase::initFromCache())
	{
		/* Read the texture info */
		TextureCacheFormat readBuf;
		toRet = readFromCache(&readBuf, m_pFile, sizeof(TextureCacheFormat));

		if (toRet)
		{
			Init(readBuf.width, readBuf.height);
			m_swizzled = readBuf.swizzled;
			
			/* Read the actual texture */
			toRet = readFromCache(m_pBuffer, m_pFile, (m_nCanvasWidth * m_nCanvasHeight * m_bpp));
		}
	}

	/* Cleanup */
	CFileSystem::Close(m_pFile);
	m_pFile = NULL;
	return toRet;
}

//******************************************************************************
//	
//******************************************************************************
bool CCachedTexture::cache()
{
	/* Try to open the cache file */
	m_pFile = CFileSystem::Open(m_cachedPath, "wb");
	if (m_pFile == NULL)
		return false;
	
	/* With the cache file open, call the base class' cache */
	bool toRet = false;
	if (CCachedBase::cache())
	{
		/* Write the texture info */
		TextureCacheFormat writeBuf;
		writeBuf.width = m_nWidth;
		writeBuf.height = m_nHeight;
		writeBuf.canvasWidth = m_nCanvasWidth;
		writeBuf.canvasHeight = m_nCanvasHeight;
		writeBuf.swizzled = m_swizzled;
		toRet = writeToCache(&writeBuf, m_pFile, sizeof(TextureCacheFormat));
		
		if (toRet)
		{
			/* Write the actual texture */
			toRet = writeToCache(m_pBuffer, m_pFile, (m_nCanvasWidth * m_nCanvasHeight * m_bpp));
		}
	}
	
	/* Cleanup */
	CFileSystem::Close(m_pFile);
	m_pFile = NULL;
	return toRet;
}
