#ifndef CCERTIFICATEMANAGER_H_
#define CCERTIFICATEMANAGER_H_

#include "VMimeTypes.h"
#include "CCertificateStoreTemp.h"
#include "CCertificateStoreMemstick.h"
#include "CCertificateStoreFlash.h"
#include "CCertHashSet.h"
#include <vmime/vmime.hpp>
#include <pspthreadman.h>

/* This class overrides the certificate_verification_exception to add the
 * certificate in question. */
class CCertificateException : public vmime::exceptions::certificate_verification_exception, public CBase
{
	public: /* Constructors & destructors */
	
		CCertificateException(VX509CertificateRef aCert, const vmime::string& what) :
			vmime::exceptions::certificate_verification_exception(what, NO_EXCEPTION),
			m_cert(aCert)
		{};
		~CCertificateException() throw ();
		
	public: /* New methods */
	
		VX509CertificateRef cert() { return m_cert; };
	
	private:
	
		VX509CertificateRef m_cert;
};

class CCertificateManager : public vmime::security::cert::defaultCertificateVerifier, public CBase
{
	public: /* Constructors & destructors */
	
		CCertificateManager(CString aPermPath, CString aFlashPath);
		virtual ~CCertificateManager();
		
	public: /* New methods */
		
		bool addTempCert(VX509CertificateRef aCert);
		bool addPermCert(VX509CertificateRef aCert);
		
		void verify(VCertificateChainRef aCertChain);
		
	private: /* New methods */
	
		void lock();
		void unlock();
		
	private: /* Data */
	
		CCertificateStoreTemp m_tempStore;
		CCertificateStoreMemstick m_permStore;
		CCertificateStoreFlash m_flashStore;
		CCertHashSet m_certSet;
		SceUID m_sema;
		
};

#endif /*CCERTIFICATEMANAGER_H_*/
