/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CComposeWindow.h

  Description :	

  Last Modified $Date: 2006/11/25 01:27:01 $

  $Revision: 1.2 $

*******************************************************************************/

#ifndef CCOMPOSEWINDOW_H_
#define CCOMPOSEWINDOW_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CWindow.h"
#include "CFileSystem.h"
#include "CString.h"
#include "CGfx.h"
#include "CControlTextBox.h"
#include "CControlSelect.h"
#include "CControlButton.h"
#include "CControlAddress.h"
#include "CControlMsgBox.h"
#include "CDialogBoxInput.h"
#include "CContextMenu.h"
#include "CControlManager.h"
#include "CContacts.h"
#include "CMailbox.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************
enum eSendingState
{
	eSendingState_Inactive,
	eSendingState_Start,
	eSendingState_AddressApproval,
	eSendingState_SubjectApproval,
	eSendingState_BodyApproval,
	eSendingState_Approved,
	eSendingState_Active,
	eSendingState_Failed,
	eSendingState_FailedAwaitingConfirmation,
	eSendingState_Complete,
	eSendingState_CompleteAwaitingConfirmation
};
//******************************************************************************
//   Constants
//******************************************************************************


//******************************************************************************
//   Class definitions
//******************************************************************************
class CComposeWindow : public CWindow, public MMailboxEventListener
{
	public: 		
		CComposeWindow(CMailbox* mailbox);
		CComposeWindow(CMailbox* mailbox, CContactEntry* contact);
		CComposeWindow(	CMailbox* mailbox,
						CControlAddress::AddressEntryList* to,
						CControlAddress::AddressEntryList* cc = NULL,
						CControlAddress::AddressEntryList* bcc = NULL,
						CText* subject = NULL,
						CText* body = NULL);

		~CComposeWindow();

		virtual void Render();
		virtual void Process();
		virtual bool ProcessInput();
		int	getExitValue();

	public:
		enum eExitCode {
				OK
			};

	public: // Inherited from MMailboxEventListener
		void sendDone(CMailbox* aMailbox, u32 aTotal);
		void sendFailed(CMailbox* aMailbox, CString aErrorText);

	private:
		/* Initializes the controls */
		void init();

		/* Handles HUH */
		void handleHUH();

		/* Handles Context Menu */
		void handleContextMenu();

		/* Handles the sending state machine */
		void handleSendingStateMachine();
		
		/* Handles sending the message */
		void sendMessage();

	/* Network */
	private:
		CMailbox* m_pMailbox;

	/* Sub windows */
	private:
		CDialogBoxInput* m_pMsgDialog;
		CContextMenu* m_pContextMenu;

	/* Controls */
	private:
		CControlManager* m_pControlManager;
		CControlAddress* m_pToField;
		CControlAddress* m_pCcField;
		CControlAddress* m_pBccField;
		CControlTextBox* m_pSubjectField;
		CControlMsgBox* m_pMessageField;
		
	/* Instances */
	private:
		bool m_sendEmpty;
		int m_exitValue;
		eSendingState m_sending;
		CString m_sendError;

		enum eMenuItems {
			HIDE,
			SEND,
			CLOSE
		};
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CCOMPOSEWINDOW_H_ */
