/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CContactsWindow.h

  Description :	

  Last Modified $Date: 2007/01/01 21:55:47 $

  $Revision: 1.2 $

*******************************************************************************/

#ifndef CContactsWindow_H_
#define CContactsWindow_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CWindow.h"
#include "CFileSystem.h"
#include "CString.h"
#include "CGfx.h"
#include "CControlManager.h"
#include "CControlScrollBar.h"
#include "CContextMenu.h"
#include "CContacts.h"
#include "CComposeWindow.h"
#include "CContactsEditWindow.h"
#include "CMailbox.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************

//******************************************************************************
//   Constants
//******************************************************************************
#define DISPLAYED_CONTACT_COUNT 15

//******************************************************************************
//   Class definitions
//******************************************************************************
class CContactsWindow : public CWindow
{
	public:

		CContactsWindow(CMailbox* mailbox);
		~CContactsWindow();

		virtual void			Render();
		virtual void			Process();
		virtual bool			ProcessInput();

	private:

		void DeleteContact(CContactEntry* contact);
					
		void BuildDisplayListFromScratch();
		void UpdateDisplayListText();
		void UpdateScrollBar();
		
		void MoveUp();
		void MoveDown();
		void MovePageUp();
		void MovePageDown();

	private:	
		CComposeWindow* m_pComposeWindow;
		CContactsEditWindow* m_pContactsEditWindow;
				
		CContextMenu* m_pContextMenu;
		CControlLabel* m_pContactsLabels[DISPLAYED_CONTACT_COUNT];
		CControlManager* m_pControlManager;
		CControlScrollBar* m_pScrollBar;
		
		CContacts::CContactList* m_pDisplayedContacts;
		CContactEntry* m_pSelectedContact;
		int m_selectedContactNum;
		CContactEntry* m_pFirstContact;
		int m_firstContactNum;
		bool m_firstDraw;
		
		bool m_saveChanges;
	
		CMailbox* m_pMailbox;
		
		enum eMenuItems {
			HIDE,
			ADD,
			DELETE,
			EDIT,
			SEND,
			CANCEL,
			CLOSE
		};
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CContactsWindow_H_ */
