/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControl.cpp

  Description :	

  Last Modified $Date: 2007/01/27 18:42:53 $

  $Revision: 1.4 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CControl.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//******************************************************************************
//
//	Main Window implementation
//
//******************************************************************************
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CControl::CControl() :
	m_pUp(NULL),
	m_pDown(NULL),
	m_pLeft(NULL),
	m_pRight(NULL),
	m_selected(false),
	m_active(false),
	m_hidden(false),
	m_enabled(true)
{
	
}

//******************************************************************************
//	
//******************************************************************************
CControl::~CControl()
{
	
}

//******************************************************************************
//	
//******************************************************************************
void CControl::activate()
{
	m_active = true;
}

//******************************************************************************
//	
//******************************************************************************
void CControl::deActivate()
{
	m_active = false;
}

//******************************************************************************
//	
//******************************************************************************
void CControl::select()
{
	m_selected = true;
	
	std::list<MControlSelectionListener*>::iterator it = m_selectionListeners.begin();

	while (it != m_selectionListeners.end())
	{
		(*it)->selectionEvent(this);
		it++;
	}
}

//******************************************************************************
//	
//******************************************************************************
void CControl::deSelect()
{
	m_selected = false;
	
	std::list<MControlSelectionListener*>::iterator it = m_selectionListeners.begin();

	while (it != m_selectionListeners.end())
	{
		(*it)->deselectionEvent(this);
		it++;
	}
}

//******************************************************************************
//	
//******************************************************************************
void CControl::hide()
{
	m_hidden = true;
}

//******************************************************************************
//	
//******************************************************************************
void CControl::show()
{
	m_hidden = false;
}

//******************************************************************************
//	
//******************************************************************************
void CControl::enable()
{
	/* Note that these should only be called if the control has no control manager,
	 * otherwise, call CControlManager::enableControl/disableControl to ensure proper
	 * handling of disabling a selected control. */
	m_enabled = true;
}

//******************************************************************************
//	
//******************************************************************************
void CControl::disable()
{
	/* Note that these should only be called if the control has no control manager,
	 * otherwise, call CControlManager::enableControl/disableControl to ensure proper
	 * handling of disabling a selected control. */
	m_enabled = false;
}

//******************************************************************************
//	
//******************************************************************************
bool CControl::isActive()
{
	return m_active;
}


//******************************************************************************
//	
//******************************************************************************
bool CControl::isSelected()
{
	return m_selected;
}

//******************************************************************************
//	
//******************************************************************************
bool CControl::isHidden()
{
	return m_hidden;
}

//******************************************************************************
//	
//******************************************************************************
bool CControl::isEnabled()
{
	return m_enabled;
}

//******************************************************************************
//	
//******************************************************************************
void CControl::addSelectionListener(MControlSelectionListener* listener)
{
	m_selectionListeners.push_back(listener);
}

//******************************************************************************
//	
//******************************************************************************
CControl* CControl::findFirstEnabledControlLeft()
{
	CControl* curControl(this);
	while (curControl->m_pLeft != NULL) {
		if (curControl->m_pLeft->isEnabled()) {
			return curControl->m_pLeft;
		}
		curControl = curControl->m_pLeft;
	}
	
	return NULL;
}

//******************************************************************************
//	
//******************************************************************************
CControl* CControl::findFirstEnabledControlRight()
{
	CControl* curControl(this);
	while (curControl->m_pRight != NULL) {
		if (curControl->m_pRight->isEnabled()) {
			return curControl->m_pRight;
		}
		curControl = curControl->m_pRight;
	}
	
	return NULL;
}

//******************************************************************************
//	
//******************************************************************************
CControl* CControl::findFirstEnabledControlDown()
{
	CControl* curControl(this);
	while (curControl->m_pDown != NULL) {
		if (curControl->m_pDown->isEnabled()) {
			return curControl->m_pDown;
		}
		curControl = curControl->m_pDown;
	}
	
	return NULL;
}

//******************************************************************************
//	
//******************************************************************************
CControl* CControl::findFirstEnabledControlUp()
{
	CControl* curControl(this);
	while (curControl->m_pUp != NULL) {
		if (curControl->m_pUp->isEnabled()) {
			return curControl->m_pUp;
		}
		curControl = curControl->m_pUp;
	}
	
	return NULL;
}

//*******************************  END OF FILE  ********************************
