/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CComposeWindow.h

  Description :	

  Last Modified $Date: 2008/09/29 04:22:27 $

  $Revision: 1.6 $

*******************************************************************************/

#ifndef CControl_H_
#define CControl_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CString.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************
class MControlSelectionListener;

//******************************************************************************
//   Constants
//******************************************************************************


//******************************************************************************
//   Class definitions
//******************************************************************************
class CControl : public CBase
{
	friend class CControlManager;
	
	protected:
		CControl();
		virtual ~CControl();

	public:
		virtual void activate();
		void deActivate();
		void select();
		void deSelect();
		void hide();
		void show();
		
		/* Note that these should only be called if the control has no control manager,
		 * otherwise, call CControlManager::enableControl/disableControl to ensure proper
		 * handling of disabling a selected control. */
		void enable();
		void disable();
		
		bool isActive();
		bool isSelected();
		bool isHidden();
		bool isEnabled();

	public:
		virtual void ProcessControlInput() = 0;
		virtual void RenderControl() = 0;
		virtual void ProcessControl() = 0;

	public:
		void addSelectionListener(MControlSelectionListener* listener);
		
	protected:
		CControl* findFirstEnabledControlLeft();
		CControl* findFirstEnabledControlRight();
		CControl* findFirstEnabledControlDown();
		CControl* findFirstEnabledControlUp();

	public:
		CControl* m_pUp;
		CControl* m_pDown;
		CControl* m_pLeft;
		CControl* m_pRight;

	protected:
		bool m_selected;
		bool m_active;
		bool m_hidden;
		bool m_enabled;

	private:
		std::list<MControlSelectionListener*> m_selectionListeners;

};

class MControlSelectionListener
{
	public: // Constructors & Destructors
		virtual ~MControlSelectionListener() {};
	
	public: 
		virtual void selectionEvent(CControl* control) {};
		virtual void deselectionEvent(CControl* control) {};
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CCOMPOSEWINDOW_H_ */
