/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControlGroupBox.cpp

  Description :	

  Last Modified $Date: 2007/01/20 23:20:03 $

  $Revision: 1.3 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CControlGroupBox.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CGfx.h"
#include "CSkinManager.h"
#include "CFrameWork.h"
#include "CControl.h"
#include "CHUHelp.h"
#include "CConnectionManager.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const float HEADER_PADDING( 5.f ); // Padding between border and menu item (all sides)
static const float LINE_LENGTH_BEFORE_TEXT( 15.f );

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CControlGroupBox::CControlGroupBox(V2 const size, V2 const position, CString text, int seperator):
	m_position(position),
	m_size(size),
	m_text(text),
	m_selected(false),
	m_seperator(seperator)
{
	
}

//******************************************************************************
//	
//******************************************************************************
CControlGroupBox::~CControlGroupBox()
{

}	

//******************************************************************************
//	
//******************************************************************************
void	CControlGroupBox::RenderControl()
{
	ARGB color;
	
	if (m_selected) {
		color = CSkinManager::GetColor( "group_box", "selected_border_color", 0xFF000000);
	} else {
		color = CSkinManager::GetColor( "group_box", "selected_border_color", 0xFF000000);
	}
	
	V2 text_size(CFont::GetActiveUiFont()->GetStringSize(m_text));
	
	// Draw left side line
	CGfx::DrawLine( V2(m_position.x, m_position.y + 0.5 * text_size.y), V2(m_position.x, m_position.y + m_size.y), color);
	
	// Draw bottom line
	CGfx::DrawLine( V2(m_position.x, m_position.y + m_size.y), V2(m_position.x + m_size.x, m_position.y + m_size.y), color);
	
	// Draw right side line
	CGfx::DrawLine( V2(m_position.x + m_size.x, m_position.y + m_size.y), V2(m_position.x + m_size.x, m_position.y + 0.5 * text_size.y), color);
	
	// Draw top right line
	CGfx::DrawLine( V2(m_position.x + m_size.x, m_position.y + 0.5 * text_size.y), V2(m_position.x + 2.0 *HEADER_PADDING + LINE_LENGTH_BEFORE_TEXT + text_size.x, m_position.y + 0.5 * text_size.y), color);
	
	// Draw top left line
	CGfx::DrawLine (V2(m_position.x, m_position.y + 0.5 * text_size.y), V2(m_position.x + LINE_LENGTH_BEFORE_TEXT, m_position.y + 0.5 * text_size.y), color);
	
	// Draw the seperator
	if (m_seperator > 0 && m_seperator < m_size.y)
	{
		CGfx::DrawLine( V2(m_position.x, m_position.y + m_seperator), V2(m_position.x + m_size.x, m_position.y + m_seperator), color);
	}
	
	// Draw text
	CFont::GetActiveUiFont()->Print( m_text, V2( m_position.x + HEADER_PADDING + LINE_LENGTH_BEFORE_TEXT, m_position.y), color);
}

//******************************************************************************
//
//******************************************************************************
void	CControlGroupBox::ProcessControlInput()
{

}

//******************************************************************************
// 
//******************************************************************************
void	CControlGroupBox::ProcessControl() {

}

//******************************************************************************
// 
//******************************************************************************
void 	CControlGroupBox::selectionEvent(CControl* control) {
	m_selected = true;	
}

//******************************************************************************
// 
//******************************************************************************
void	CControlGroupBox::deselectionEvent(CControl* control) {
	m_selected = false;
}


//*******************************  END OF FILE  ********************************
