/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControlManager.cpp

  Description :	

  Last Modified $Date: 2007/01/27 18:42:53 $

  $Revision: 1.5 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CControlManager.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CInput.h"
#include "CGfx.h"
#include "CSkinManager.h"
#include "CFrameWork.h"
#include "CControlManager.h"
#include "CMath.h"
#include "CHUHelp.h"
#include "irda_KeyboardLayouts.h"
#include "CKeyboardNavWrapper.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CControlManager::CControlManager() :
	m_pControlList(NULL),
	m_pSelectedControl(NULL)
{
	m_pControlList = new CControlList();
}

//******************************************************************************
//	
//******************************************************************************
CControlManager::~CControlManager()
{
	// Delete of the controls in the list
	CControlList::iterator it = m_pControlList->begin();
	while( it !=  m_pControlList->end() ) {
		SAFE_DELETE(*it);
		it++;
	}

	// Remove the control list
	SAFE_DELETE(m_pControlList);
}	

//******************************************************************************
//	
//******************************************************************************
void	CControlManager::RenderControl()
{
	if (m_pControlList->size() > 0) {
		//Render all items in the list
		CControlList::iterator it = m_pControlList->begin();
		while(it !=  m_pControlList->end()) {
			if ((*it) != m_pSelectedControl) {
				(*it)->RenderControl();
			}
			it++;
		}
		if (m_pSelectedControl != NULL) {
			m_pSelectedControl->RenderControl();
		}
	}
}

//******************************************************************************
//	
//******************************************************************************
void	CControlManager::ProcessControl()
{
	//Process all items in the list
	CControlList::iterator it = m_pControlList->begin();
	while(it !=  m_pControlList->end()) {
		if((*it)->isActive()) {
			(*it)->ProcessControl();
		}
		it++;
	}
}

//******************************************************************************
// Return true if no element is currently active (eg process keys if this returns true)
//******************************************************************************
bool	CControlManager::ProcessControlInput()
{
	if (m_pControlList->size() > 0 && m_pSelectedControl != NULL) {
		if (m_pSelectedControl->isActive()) {
			m_pSelectedControl->ProcessControlInput();
			return false;
		} else {
			CControl* newSelectedControl(NULL);
			
			if (CKeyboardNavWrapper::ActivateControl())
			{
				CInput::GetKeyboardCharKey(); // "consume" the key
				m_pSelectedControl->activate();
			}
			else if (CKeyboardNavWrapper::Down())
			{
				CInput::GetKeyboardCharKey(); // "consume" the key
				if ((newSelectedControl = m_pSelectedControl->findFirstEnabledControlDown()) != NULL)
					selectControl(newSelectedControl);
			}
			else if (CKeyboardNavWrapper::Up())
			{
				CInput::GetKeyboardCharKey(); // "consume" the key
				if ((newSelectedControl = m_pSelectedControl->findFirstEnabledControlUp()) != NULL)
					selectControl(newSelectedControl);
			}
			else if (CKeyboardNavWrapper::Left())
			{
				CInput::GetKeyboardCharKey(); // "consume" the key
				if ((newSelectedControl = m_pSelectedControl->findFirstEnabledControlLeft()) != NULL)
					selectControl(newSelectedControl);
			}
			else if (CKeyboardNavWrapper::Right())
			{
				CInput::GetKeyboardCharKey(); // "consume" the key
				if ((newSelectedControl = m_pSelectedControl->findFirstEnabledControlRight()) != NULL)
					selectControl(newSelectedControl);
			}
		}
	}
	return true;
}

//******************************************************************************
//	
//******************************************************************************
void CControlManager::addControl(CControl* control) {
	// Add control to the list
	if (control != NULL) {
		m_pControlList->push_back(control);
	}
}

//******************************************************************************
//	
//******************************************************************************
void CControlManager::selectControl(CControl* control) {
	if (ownsControl(control))
	{
		if (m_pSelectedControl)
			m_pSelectedControl->deSelect();
			
		m_pSelectedControl = control;
		m_pSelectedControl->select();
	}
}

//******************************************************************************
//	
//******************************************************************************
void CControlManager::enableControl(CControl* control) {
	if (ownsControl(control))
	{
		control->enable();
	}
}

//******************************************************************************
//	
//******************************************************************************
void CControlManager::disableControl(CControl* control) {
	if (ownsControl(control))
	{
		if (control == m_pSelectedControl)
		{
			// We're trying to disable the selected control, so let's find the
			// nearest control to enable.
			CControl* newSelectedControl(control->findFirstEnabledControlLeft());
			if (newSelectedControl == NULL)
			{
				newSelectedControl = control->findFirstEnabledControlRight();
				if (newSelectedControl == NULL)
				{
					newSelectedControl = control->findFirstEnabledControlUp();
					if (newSelectedControl == NULL)
					{
						newSelectedControl = control->findFirstEnabledControlDown();
					}
				}
			}
			
			if (newSelectedControl == NULL)
				m_pSelectedControl = NULL;
			else
				selectControl(newSelectedControl);
		}
		
		control->disable();
	}
}

//******************************************************************************
//	
//******************************************************************************
bool CControlManager::hasActive() {
	if (m_pSelectedControl != NULL) {
		return m_pSelectedControl->isActive();
	}
	return false;
}

//******************************************************************************
//	
//******************************************************************************
bool CControlManager::ownsControl(CControl* control)
{
	CControlList::iterator it(m_pControlList->begin());
	while (it != m_pControlList->end())
	{
		if (*it == control)
			return true;
		it++;
	}
	
	return false;
}

//*******************************  END OF FILE  ********************************
