/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControlMsgBox.cpp

  Description :	

  Last Modified $Date: 2008/09/28 01:02:03 $

  $Revision: 1.6 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CControlMsgBox.h"
#include "CControlTextBox.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CInput.h"
#include "CGfx.h"
#include "CSkinManager.h"
#include "CFrameWork.h"
#include "CControl.h"
#include "CHUHelp.h"
#include "CConfigFile.h"
#include "irda_KeyboardLayouts.h"
#include "CKeyboardNavWrapper.h"
#include <CScrollableText.h>

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const float	WINDOW_BORDER( 1.f ); // Border around context menu (all sides)
static const float  WINDOW_PADDING( 1.f ); // Padding between border and menu item (all sides)
static const float  CURSOR_WIDTH( 1.f ); // Width of the text cursor
//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CControlMsgBox::CControlMsgBox(V2 const size, V2 const position, CText* text) :
	m_size(size),
	m_position(position),
	m_text(NULL),
	m_scrollText(NULL),
	m_cursorTimer(NULL)
{
	m_cursorTimer = new CCursorTimer();
	setText(text);
}

//******************************************************************************
//	
//******************************************************************************
CControlMsgBox::~CControlMsgBox()
{
	SAFE_DELETE(m_scrollText);
	SAFE_DELETE(m_text);
	SAFE_DELETE(m_cursorTimer);
}

//******************************************************************************
//	
//******************************************************************************
void	CControlMsgBox::RenderControl()
{
	bool showCursor = false;
	
	if(isHidden()) {
		return;
	}

	// Draw the borders
    // Sort of cheating here, couldn't send bgcolor to none opaque, if we want that functionality
	// need to draw 4 seperate quads
	if (isSelected()) {
		CGfx::DrawQuad( V2( m_position.x, m_position.y ),
						V2( m_size.x, m_size.y),
						CSkinManager::GetColor( "text_box", "selected_border_color", 0xFFAAFFAA));
	} else {
		CGfx::DrawQuad( V2( m_position.x, m_position.y ),
						V2( m_size.x, m_size.y),
						CSkinManager::GetColor( "text_box", "border_color", 0xFFAAFFAA));
	}

	// Draw menu background (overlay over the menu border quad)
	CGfx::DrawQuad( V2( m_position.x + WINDOW_BORDER, m_position.y + WINDOW_BORDER),
					V2( m_size.x - WINDOW_BORDER * 2, m_size.y - WINDOW_BORDER * 2),
					CSkinManager::GetColor( "text_box", "background_color", 0xFF00FF00));

	// Draw text
	//V2 text_size( CFont::GetDefaultFont()->GetStringSize( m_text ));
	V2 text_position( m_position.x + WINDOW_BORDER + WINDOW_PADDING, m_position.y + WINDOW_BORDER + WINDOW_PADDING );
    //V2 text_position( m_position.x + WINDOW_BORDER + WINDOW_PADDING, m_position.y + WINDOW_BORDER + WINDOW_PADDING );
    V2 text_area(m_size.x - WINDOW_BORDER * 2 - WINDOW_PADDING * 2, m_size.y - WINDOW_BORDER * 2 - WINDOW_PADDING * 2);
	
	if (isActive())
	{
		showCursor = m_cursorTimer->isCursorVisible();
	}
	
	CGfx::SetClipRegion( text_position, text_area );
	//CFont::GetDefaultFont()->Print( m_text, text_position, CSkinManager::GetColor( "text_box", "text_color", 0xFFFFFFFF) );
	//CFont::GetDefaultFont()->Print(m_text, 0, text_area, text_position, CSkinManager::GetColor( "text_box", "text_color", 0xFFFFFFFF), m_show_cursor);
	m_scrollText->render(showCursor);
	CGfx::DisableClipRegions();

	if (isActive()) {
		// Set up HUD
		CHUHelp::Clear();
		CHUHelp::SetButton(CHUHelp::BUTTON_RTRIGGER, "Toggle Cursor/Entry Mode");
		CHUHelp::SetButton(CHUHelp::BUTTON_LTRIGGER, "Exit input");
	}
}

//******************************************************************************
//
//******************************************************************************
void	CControlMsgBox::ProcessControlInput()
{
	if (isActive()) {
		CInput::EnableKeyboard();
		if (CInput::IsKeyboardPressed()) {
			char chr = CInput::GetKeyboardCharKey();
			u16 ctrl = CInput::GetKeyboardControlKey();
			m_cursorTimer->Show();
			
			if (ctrl == CKeyboard::BACKSPACE)
			{
	   	  		m_scrollText->backspace();
			}
			else if (ctrl == CKeyboard::LEFT)
			{
				m_scrollText->decCursor();
			}
			else if (ctrl == CKeyboard::RIGHT)
			{
				m_scrollText->incCursor();
			}
			else if (ctrl == CKeyboard::UP)
			{
				m_scrollText->upCursor();
			}
			else if (ctrl == CKeyboard::DOWN)
			{
				m_scrollText->downCursor();
			}
			else if (ctrl == CKeyboard::ENTER)
			{
				m_scrollText->insert("\n");
			}
			else if (ctrl == CKeyboard::ESC)
			{
				deActivate();
			}
			else if (ctrl == CKeyboard::PAGE_UP)
			{
				m_scrollText->pageUp();
			}
			else if (ctrl == CKeyboard::PAGE_UP)
			{
				m_scrollText->pageDown();
			}
			else if (ctrl == CKeyboard::HOME)
			{
				m_scrollText->home();
			}
			else if (ctrl == CKeyboard::END)
			{
				m_scrollText->end();
			}
			else if (chr >= ' ' && chr <= '~')
			{
				m_scrollText->insert(chr);
			}
		}

		if (CInput::ShouldDisableKeyboard()) {
			// Means the last keyboard process detected that the keyboard should be disabled, so let's deactivate the control
			CInput::DisableKeyboard();
			deActivate();
		}
	}
}

//******************************************************************************
//
//******************************************************************************
CText* CControlMsgBox::getText() {
	return m_text;
}



//******************************************************************************
// 
//******************************************************************************
void CControlMsgBox::ProcessControl() {

}

//******************************************************************************
//	
//******************************************************************************
void CControlMsgBox::activate()
{
	CInput::ResetKeyboard();
	m_active = true;
}

//******************************************************************************
//	
//******************************************************************************
void CControlMsgBox::deActivate()
{
	m_active = false;
}

//******************************************************************************
//	
//******************************************************************************
void CControlMsgBox::setText(CText* text)
{
	SAFE_DELETE(m_scrollText);
	SAFE_DELETE(m_text);
	m_text = text;

	m_scrollText = new CScrollableText(m_text, V2(m_size.x - WINDOW_BORDER * 2 - WINDOW_PADDING * 2,
	                                              m_size.y - WINDOW_BORDER * 2 - WINDOW_PADDING * 2),
	                                       V2(m_position.x + WINDOW_BORDER + WINDOW_PADDING,
	                                          m_position.y + WINDOW_BORDER + WINDOW_PADDING),
	                                       CSkinManager::GetColor( "dialog_box", "text_color", 0xFFFFFFFF),
	                                       true);
}

//******************************************************************************
//	
//******************************************************************************
void CControlMsgBox::dumpDebug(const char* aSnapshotFile)
{
	m_scrollText->dumpDebug();
	m_text->snapshot(aSnapshotFile);
}

//*******************************  END OF FILE  ********************************
