/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControlScrollBar.cpp

  Description :	

  Last Modified $Date: 2006/11/25 16:41:26 $

  $Revision: 1.3 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CControlScrollBar.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CGfx.h"
#include "CSkinManager.h"
#include "CFrameWork.h"
#include "CControl.h"
#include "CControlLabel.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const float	WINDOW_BORDER( 1.f ); // Border around progress bar

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CControlScrollBar::CControlScrollBar(V2 const size, V2 const position, u32 maxDisplayedItems):
	m_size(size),
	m_position(position),
	m_maxDisplayedItems(maxDisplayedItems),
	m_firstDisplayedItem(0),
	m_totalItems(0)
{
					
}

//******************************************************************************
//	
//******************************************************************************
CControlScrollBar::~CControlScrollBar()
{

}	

//******************************************************************************
//	
//******************************************************************************
void	CControlScrollBar::RenderControl()
{
	// Don't render if the # of items displayed in the list is less than the list size
	if (m_totalItems < m_maxDisplayedItems) {
		return;
	}

	float itemSize = (m_size.y - WINDOW_BORDER*2) / m_totalItems;

	// Draw the border
	// Sort of cheating here, couldn't send bgcolor to none opaque, if we want that functionality
	// need to draw 4 seperate quads
	CGfx::DrawQuad( V2(m_position.x, m_position.y),
				    V2(m_size.x, m_size.y),
					CSkinManager::GetColor("scroll_bar", "border_color", 0xFFD4D4D4));

	// Draw menu background (overlay over the menu border quad)
	CGfx::DrawQuad( V2(m_position.x + WINDOW_BORDER, m_position.y + WINDOW_BORDER),
					V2(m_size.x - WINDOW_BORDER * 2, m_size.y - WINDOW_BORDER * 2),
					CSkinManager::GetColor( "scroll_bar", "background_color", 0xFFFFFFFF));

	// Draw the bar
	CGfx::DrawQuad( V2(m_position.x + WINDOW_BORDER, m_position.y + WINDOW_BORDER + itemSize*(m_firstDisplayedItem)),
					V2(m_size.x - WINDOW_BORDER*2, itemSize*m_maxDisplayedItems),
					CSkinManager::GetColor( "scroll_bar", "bar_color", 0xFFFFFFFF));
}

//******************************************************************************
//
//******************************************************************************
void CControlScrollBar::ProcessControlInput()
{

}

//******************************************************************************
// 
//******************************************************************************
void CControlScrollBar::ProcessControl() {

}

//******************************************************************************
// 
//******************************************************************************
void CControlScrollBar::update(u32 firstDisplayedItem, u32 totalItems) {
	m_totalItems = totalItems;
	m_firstDisplayedItem = firstDisplayedItem;
}

//******************************************************************************
// 
//******************************************************************************
void CControlScrollBar::setMaxDisplayedItems(u32 maxDisplayedItems) {
	m_maxDisplayedItems = maxDisplayedItems;
}

//*******************************  END OF FILE  ********************************
