/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControlSelect.h

  Description :	

  Last Modified $Date: 2008/09/29 04:22:27 $

  $Revision: 1.4 $

*******************************************************************************/

#ifndef CControlSelect_H_
#define CControlSelect_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CWindow.h"
#include "CControl.h"
#include "CGfx.h"
#include "CString.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************
enum eEvents {
	CHANGE = 0
};

class CControlSelectItem;
class MControlSelectEventListener;

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************
class CControlSelect : public CControl
{
	public:
		typedef std::vector<CControlSelectItem *> CControlSelectItemList;

	public:
		CControlSelect(V2 const size, V2 const position, int maxDisplayedItems, CControlSelectItemList * const itemList, CControlSelectItem * selectedItem);

		virtual ~CControlSelect();
		
		virtual void RenderControl();
		virtual void ProcessControlInput();
		virtual void ProcessControl();
		virtual void activate();
		u32	getSelectedValue();
		CString	getSelectedText();
		void subscribe(MControlSelectEventListener* listener);
		void rebuildList(CControlSelectItemList* const itemList, CControlSelectItem* selectedItem);
		
	private:
		void initList(CControlSelectItemList* const itemList, CControlSelectItem* selectedItem);
		void destroyList();
		void buildDisplayList();
		void updateFirstDisplayed();
		void notifyEvent(eEvents aEvent);

	private:
		V2 m_size;
		V2 m_position;
		int m_maxDisplayedItems;
		CControlSelectItemList* m_pItemList;
		CControlSelectItemList* m_pDisplayedItemList;
		CControlSelectItem* m_pSelectedItem;
		CControlSelectItem* m_pDisplayedItem;
		CControlSelectItem* m_pHistoricItem; //used to remember the original selected item		
		std::list<MControlSelectEventListener*> m_listeners;

};


class CControlSelectItem : public CBase
{
	public:
		CControlSelectItem(CString text, int value) {
			m_text = text;
			m_value = value;
		}

	public:
		CString		m_text;
		u32			m_value;
};

class MControlSelectEventListener
{
	public: // Constructors & Destructors
		virtual ~MControlSelectEventListener() {};
	
	public: 
		virtual void onChange(CControlSelect* control) {};
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CControlSelect_H_ */
