/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) T Swann, Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CFont.h

  Description :	

  Last Modified $Date: 2007/01/03 05:31:14 $

  $Revision: 1.3 $

*******************************************************************************/

#ifndef CFONT_H_
#define CFONT_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CVector.h"
#include "CString.h"
#include "CTextureManager.h"
#include <CText.h>
#include <CTextPtr.h>

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************
class CTextAsciiRenderTable;
class CCachedFont;

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************
class CFont : public CTexture
{
	friend class CTextureManager;
	friend class CCachedFont;

	public: /* Types */
	
		typedef struct Metrics {
			short x, y, width, height, xoffset, yoffset, xadvance;
		} Metrics;

	public: /* Constructors & destructors */
	
		virtual ~CFont();

	public: /* Static methods */

		static bool Open();
		static void Close();
		static CFont* GetActiveUiFont();
		static CFont* GetActiveTextFont();

	public: /* New methods */

		void Print(const CString & string, const V2 & pos, ARGB color);
		bool Print(const CText*, const int aLineNum, const V2 & size, const V2 & pos, ARGB color, const bool aShowCursor = false);
		void Print(CTextPtr aStart, size_t aCount, const V2 & pos, ARGB color);
		
		V2 GetStringSize(const CString & string) const;
		CString GetWrappedString(const CString & string, float width) const;
		
		u32 GetHeight() const { return m_lineHeight; }
		CTextAsciiRenderTable* GetRenderTable() { return m_ctextRenderTable; }

	protected: /* Used by TTFReader and CCachedFont */
	
		CFont(u32 fontSize);
		void SetMetrics(Metrics* metrics, u32 metricTableSize);
		void SetHeight(u32 lineHeight);

	protected:

		void InitCTextRenderTable();

	protected: /* Data */

		u32 m_fontSize;
		CTextAsciiRenderTable* m_ctextRenderTable; /* For CText rendering purposes */
		Metrics* m_fontMetrics;
		u32 m_metricTableSize;
		u32 m_lineHeight;
		static CFont* s_pActiveUiFont;
		static CFont* s_pActiveTextFont;

};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CFONT_H_ */
