/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CHUHelp.cpp

  Description :	

  Last Modified $Date: 2007/01/01 21:55:47 $

  $Revision: 1.4 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CHUHelp.h"
#include "CFrameWork.h"
#include "CRenderable.h"
#include "CGfx.h"
#include "CFont.h"
#include "CSkinManager.h"
#include "CTextureManager.h"
#include "CUSBManager.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const float	WINDOW_BORDER(2.f); // Border around context menu (all sides)
static const float  WINDOW_PADDING(8.f); // Padding between border and menu item (all sides)
static const float	WINDOW_WIDTH(360.f);
static const float	ITEM_PADDING(4.f);
static const V2		ICON_SIZE(16.f, 16.f);
static const float  TEXT_WIDTH(WINDOW_WIDTH - WINDOW_BORDER*2 - WINDOW_PADDING*2 - ICON_SIZE.x - ITEM_PADDING);
static const V2		EMPTY_SIZE(WINDOW_WIDTH, WINDOW_BORDER*2 + WINDOW_PADDING*2);

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************
bool								CHUHelp::s_bVisible( true );
CHUHelp *							CHUHelp::s_pInstance( NULL );
CString								CHUHelp::s_szButtonText[ MAX_BUTTONS ] =	{ "", "", "", "", "", "", "", "", "", "", "", "" };
V2									CHUHelp::m_size( 0, 0 );
CSkinManager::eSkinComponent		CHUHelp::s_buttonTexture[ MAX_BUTTONS ] = {
										CSkinManager::SC_BUTTON_CROSS,
										CSkinManager::SC_BUTTON_CIRCLE,
										CSkinManager::SC_BUTTON_SQUARE,
										CSkinManager::SC_BUTTON_TRIANGLE,
										CSkinManager::SC_BUTTON_LTRIGGER,
										CSkinManager::SC_BUTTON_RTRIGGER,
										CSkinManager::SC_BUTTON_START,
										CSkinManager::SC_BUTTON_SELECT,
										CSkinManager::SC_BUTTON_UP,
										CSkinManager::SC_BUTTON_DOWN,
										CSkinManager::SC_BUTTON_RIGHT,
										CSkinManager::SC_BUTTON_LEFT };
		
//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
void	CHUHelp::Create()
{
	ASSERT( s_pInstance == NULL, "HUHelp has already been created!" );

	s_pInstance = new CHUHelp();
}

//******************************************************************************
//	
//******************************************************************************
void	CHUHelp::Destroy()
{
	ASSERT( s_pInstance != NULL, "HUHelp hasn't been created yet!" );

	SAFE_DELETE( s_pInstance );
}

//******************************************************************************
//	
//******************************************************************************
CHUHelp::CHUHelp()
{
	CRenderable::Register( CRenderable::RO_HUD, Render );
}

//******************************************************************************
//	
//******************************************************************************
CHUHelp::~CHUHelp()
{
	CRenderable::UnRegister( CRenderable::RO_HUD, Render );
}

//******************************************************************************
//	
//******************************************************************************
void	CHUHelp::Render()
{
	if ( s_pInstance != NULL )
	{
		if ( s_bVisible == true )
		{
			s_pInstance->RenderInternal();
		}
	}
}

//******************************************************************************
//	
//******************************************************************************
void	CHUHelp::RenderInternal()
{
	// Don't render if the HUHelp isn't setup
	if (m_size.x != EMPTY_SIZE.x || m_size.y != EMPTY_SIZE.y) {
		V2 m_position( CGfx::s_ScreenWidth / 2 - m_size.x/2, CGfx::s_ScreenHeight / 2 - m_size.y/2);

		// Draw overlay
		CGfx::DrawQuad( V2( 0, 0 ), V2( CGfx::s_ScreenWidth, CGfx::s_ScreenHeight), CSkinManager::GetColor( "huh", "overlay_color", 0x8F000000));

		// Draw the borders
		// Sort of cheating here, couldn't send bgcolor to none opaque, if we want that functionality
		// need to draw 4 seperate quads
		CGfx::DrawQuad( V2( m_position.x, m_position.y ),
						V2( m_size.x, m_size.y),
						CSkinManager::GetColor( "huh", "border_color", 0xFF00FF00));

		// Draw menu background (overlay over the menu border quad)
		CGfx::DrawQuad( V2( WINDOW_BORDER + m_position.x, WINDOW_BORDER + m_position.y),
						V2( m_size.x - WINDOW_BORDER * 2, m_size.y - WINDOW_BORDER * 2),
						CSkinManager::GetColor( "huh", "background_color", 0xFF000000));

		V2 item_position( m_position.x + WINDOW_BORDER + WINDOW_PADDING, m_position.y + WINDOW_BORDER + WINDOW_PADDING);

		for (int x=0; x< MAX_BUTTONS; x++) {
			if (!s_szButtonText[x].IsEmpty()) {
				// print graphic
				CTexture * const	p_button_texture( CSkinManager::GetComponent( s_buttonTexture[x] )->GetTexture() );
				CGfx::DrawQuad( p_button_texture, item_position, ICON_SIZE, 0xffffffff );
		
				// print text
				CString		button_text(CFont::GetActiveUiFont()->GetWrappedString( s_szButtonText[x], TEXT_WIDTH ));
				const V2	text_size( CFont::GetActiveUiFont()->GetStringSize( button_text ));
				CFont::GetActiveUiFont()->Print( button_text, item_position + V2( ICON_SIZE.x + ITEM_PADDING, 0), CSkinManager::GetColor( "huh", "text_color", 0xFF000000) );
				
				// increment the item counter
				item_position.y += ( ICON_SIZE.y > text_size.y ) ? ICON_SIZE.y : text_size.y;
				if (x != MAX_BUTTONS) {
					item_position.y += ITEM_PADDING;
				}
			}
		}
	}
}

//******************************************************************************
//	
//******************************************************************************
void	CHUHelp::SetButton( eButton button, const CString & text )
{
	s_szButtonText[ button ] = text;
	BuildSize();
}

//******************************************************************************
//	
//******************************************************************************
void	CHUHelp::Clear()
{
	for (int x=0; x< MAX_BUTTONS; x++ ) {
		s_szButtonText[ x ].Clear();
	}

	BuildSize();
}

//******************************************************************************
//	
//******************************************************************************
void	CHUHelp::Show( bool show )
{
	if (show) {
		BuildSize();
	}
	s_bVisible = show;
}

//******************************************************************************
//	
//******************************************************************************
bool	CHUHelp::IsVisible()
{
	return s_bVisible;
}

//******************************************************************************
//	
//******************************************************************************
void	CHUHelp::BuildSize() {
	m_size = V2(WINDOW_WIDTH, 0);

	m_size.y += WINDOW_BORDER + WINDOW_PADDING;

	for (int x=0; x< MAX_BUTTONS; x++) {
		if (!s_szButtonText[x].IsEmpty()) {
			// print text
			CString		button_text(CFont::GetActiveUiFont()->GetWrappedString( s_szButtonText[x], TEXT_WIDTH ));
			const V2	text_size( CFont::GetActiveUiFont()->GetStringSize( button_text ));

			// increment the item counter
			m_size.y += ( ICON_SIZE.y > text_size.y ) ? ICON_SIZE.y : text_size.y;
			if (x != MAX_BUTTONS) {
				m_size.y += ITEM_PADDING;
			}
		}
	}

	m_size.y += WINDOW_BORDER + WINDOW_PADDING;
}

//*******************************  END OF FILE  ********************************
