/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CKeyboardNav.h

  Description :	

  Last Modified $Date: 2008/09/29 04:22:27 $

  $Revision: 1.5 $

*******************************************************************************/

#ifndef CKEYBOARDNAV_H_
#define CKEYBOARDNAV_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CKeyboard.h"
#include "CPspPadRepeat.h"
#include "CInput.h"

//******************************************************************************
//   Macros
//******************************************************************************
#define VALIDATE_KEYBOARD() if (!m_keyboard) return false

//******************************************************************************
//   Types
//******************************************************************************

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************
/* The CKeyboardNav class defines the interface PSPoste is to use when wanting
 * to navigate the interface, it uses the CKeyboard class to get key info. 
 */
class CKeyboardNav : public CBase
{
	public: /* Constructors & Destructors */
	
		CKeyboardNav(CKeyboard* aKeyboard) :
			m_keyboard(aKeyboard),
			m_pspPadRepeat(PSP_CTRL_SELECT | PSP_CTRL_START | PSP_CTRL_UP | PSP_CTRL_RIGHT | PSP_CTRL_DOWN | PSP_CTRL_LEFT | PSP_CTRL_LTRIGGER | PSP_CTRL_RTRIGGER | PSP_CTRL_TRIANGLE | PSP_CTRL_CIRCLE | PSP_CTRL_CROSS | PSP_CTRL_SQUARE)
		{};
		CKeyboardNav() :
			m_keyboard(NULL),
			m_pspPadRepeat(PSP_CTRL_SELECT | PSP_CTRL_START | PSP_CTRL_UP | PSP_CTRL_RIGHT | PSP_CTRL_DOWN | PSP_CTRL_LEFT | PSP_CTRL_LTRIGGER | PSP_CTRL_RTRIGGER | PSP_CTRL_TRIANGLE | PSP_CTRL_CIRCLE | PSP_CTRL_CROSS | PSP_CTRL_SQUARE)
		{};
		virtual ~CKeyboardNav();
		
	public: /* Getters & Setters */
	
		void setKeyboard(CKeyboard* aKeyboard) { m_keyboard = aKeyboard; };
		
	public:
	
		virtual void process(SceCtrlData *aPadData) { m_pspPadRepeat.process(aPadData); };
	
	public: /* Events */
		
		virtual bool up() { return (m_pspPadRepeat.getRepeatButtons() & PSP_CTRL_UP); };
		virtual bool down() { return (m_pspPadRepeat.getRepeatButtons() & PSP_CTRL_DOWN); };
		virtual bool left() { return (m_pspPadRepeat.getRepeatButtons() & PSP_CTRL_LEFT); };
		virtual bool right() { return (m_pspPadRepeat.getRepeatButtons() & PSP_CTRL_RIGHT); };
		virtual bool pageUp() { return (m_pspPadRepeat.getRepeatButtons() & PSP_CTRL_LTRIGGER); };
		virtual bool pageDown() { return (m_pspPadRepeat.getRepeatButtons() & PSP_CTRL_RTRIGGER); };
		virtual bool activateControl() { return (CInput::IsButtonReleased(CInput::CROSS)); };
		virtual bool deactivateControl() { return (CInput::IsButtonReleased(CInput::START)); };
		virtual bool openContextMenu() { return (CInput::IsButtonClicked(CInput::TRIANGLE)); };
		virtual bool closeContextMenu() { return (CInput::IsButtonClicked(CInput::TRIANGLE) || CInput::IsButtonClicked(CInput::LTRIGGER)); };
		virtual bool expandSubContextMenu() { return (CInput::IsButtonClicked(CInput::LEFT) || activateControl()); };
		virtual bool collapseSubContextMenu() { return (CInput::IsButtonClicked(CInput::RIGHT) || CInput::IsButtonClicked(CInput::LTRIGGER) || closeContextMenu()); };
		virtual bool markMessageToggle() { return (m_pspPadRepeat.getRepeatButtons() & PSP_CTRL_SQUARE); };
		virtual bool openMessage() { return (CInput::IsButtonClicked(CInput::CROSS)); };
		virtual bool closeMessage() { return (CInput::IsButtonClicked(CInput::CROSS) || CInput::IsButtonClicked(CInput::CIRCLE) || CInput::IsButtonClicked(CInput::SQUARE)); };
		virtual bool closeInputWindow() { return (CInput::IsButtonClicked(CInput::CIRCLE) || CInput::IsButtonClicked(CInput::SQUARE)); };
		virtual bool showHUH() { return (CInput::IsButtonDown(CInput::START)) && m_keyboard->allowHUH(); };
		virtual bool hideHUH() { return (!showHUH()); };
		virtual bool nextTab() { return (m_pspPadRepeat.getRepeatButtons() & PSP_CTRL_RTRIGGER); };
		virtual bool prevTab() { return (m_pspPadRepeat.getRepeatButtons() & PSP_CTRL_LTRIGGER); };
		virtual bool saveComboBoxSelection() { return (CInput::IsButtonClicked(CInput::CROSS)); };
		virtual bool cancelComboBoxSelection() { return (CInput::IsButtonClicked(CInput::TRIANGLE) || CInput::IsButtonClicked(CInput::LTRIGGER) || closeInputWindow()); };
		
	protected:
	
		CKeyboard* m_keyboard;
		CPspPadRepeat m_pspPadRepeat;
		
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CKEYBOARDNAV_H_ */
