/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CKeyboardNavWrapper.h

  Description :	

  Last Modified $Date: 2006/11/16 03:44:59 $

  $Revision: 1.2 $

*******************************************************************************/

#ifndef CKEYBOARDNAVWRAPPER_H_
#define CKEYBOARDNAVWRAPPER_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CKeyboardNav.h"

//******************************************************************************
//   Macros
//******************************************************************************
#define VALIDATE_ACTIVE_NAV() if (!s_activeNav) return false

//******************************************************************************
//   Types
//******************************************************************************

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************
/* The CKeyboardNavWrapper class defines the interface PSPoste is to use when wanting
 * to navigate the interface, it uses the CKeyboard class to get key info. 
 */
class CKeyboardNavWrapper
{
	public: /* Events */
		
		static void SetActiveNav(CKeyboardNav* aNav)
		{
			s_activeNav = aNav;
		};
		
		static bool Up()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->up();
		};
		
		static bool Down()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->down();
		};
		
		static bool Left()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->left();
		};
		
		static bool Right()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->right();
		};
		
		static bool PageUp()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->pageUp();
		};
		
		static bool PageDown()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->pageDown();
		};
		
		static bool ActivateControl()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->activateControl();
		};
		
		static bool DeactivateControl() /* Note that deactivate control is done implicitly for text controls by the CKeyboard::Process */
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->deactivateControl();
		};
		
		static bool OpenContextMenu()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->openContextMenu();
		};
		
		static bool CloseContextMenu()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->closeContextMenu();
		};
		
		static bool ExpandSubContextMenu()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->expandSubContextMenu();
		};
		
		static bool CollapseSubContextMenu()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->collapseSubContextMenu();
		};
		
		static bool MarkMessageToggle()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->markMessageToggle();
		};
		
		static bool OpenMessage()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->openMessage();
		};
		
		static bool CloseMessage()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->closeMessage();
		};
		
		static bool CloseInputWindow()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->closeInputWindow();
		};
		
		static bool ShowHUH()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->showHUH();
		};
		
		static bool HideHUH()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->hideHUH();
		};
		
		static bool NextTab()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->nextTab();
		};
		
		static bool PrevTab()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->prevTab();
		};
		
		static bool SaveComboBoxSelection()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->saveComboBoxSelection();
		};
		
		static bool CancelComboBoxSelection()
		{
			VALIDATE_ACTIVE_NAV();
			return s_activeNav->cancelComboBoxSelection();
		};
		
	private:
	
		static CKeyboardNav* s_activeNav;
		
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CKEYBOARDNAVWRAPPER_H_ */
