/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CLog.h

  Description :	

  Last Modified $Date: 2006/10/09 16:55:45 $

  $Revision: 1.1 $

*******************************************************************************/

#ifndef _CLOG_H
#define _CLOG_H

//******************************************************************************
//   Include Files
//******************************************************************************
#include <stdio.h>
#include <string>
#include <stdarg.h>
#include "pspthreadman.h"

#ifndef CLOG_PORTABLE_BUILD
#include "CString.h"
#endif

//******************************************************************************
//   Class definitions
//******************************************************************************
class CLog
{
	public:
		// Open the log files
		static int Open(int aOverwrite = 1);
		
		// Close the log files (includes a call to Flush)
		static void Close();
		
		// Flush any outstanding data to the log files
		static void Flush();
		
		// Are the log files opened?
		static int IsOpen();
		
		// Print an informative message to the log file for stdout
		static int Message(const char* aMsg, ...);
		static int MessageRaw(const char* aMsg, int len);
		
		// Convenient overloads for Message() and MessageRaw()
		inline static int MessageRaw(const char* aMsg) {return MessageRaw(aMsg, strlen(aMsg));}
#ifndef CLOG_PORTABLE_BUILD
		inline static int Message(const CString& aMsg) {return MessageRaw(aMsg.GetPtr());}
		inline static int Message(const CString* aMsg) {return MessageRaw(aMsg->GetPtr());}
#endif
		inline static int Message(const std::string& aMsg) {return MessageRaw(aMsg.c_str());}
		inline static int Message(const std::string* aMsg) {return MessageRaw(aMsg->c_str());}
		
		// Print a debug message to the log file for stdout, the differences
		// between Message() and Debug() are:
		//
		// 1) Debug messages will only be printed out when _DEBUG is defined
		// 2) Debug messages are prepended with "[Debug]: "
		static int Debug(const char* aMsg, ...);
		static int DebugRaw(const char* aMsg, int len);
		
		// Convenient overloads for Debug() and DebugRaw()
		inline static int DebugRaw(const char* aMsg) {return DebugRaw(aMsg, strlen(aMsg));}
#ifndef CLOG_PORTABLE_BUILD
		inline static int Debug(const CString& aMsg) {return DebugRaw(aMsg.GetPtr());}
		inline static int Debug(const CString* aMsg) {return DebugRaw(aMsg->GetPtr());}
#endif
		inline static int Debug(const std::string& aMsg) {return DebugRaw(aMsg.c_str());}
		inline static int Debug(const std::string* aMsg) {return DebugRaw(aMsg->c_str());}
		
		// Print an error message to the log file fo stderr
		static int Error(const char* aErr, ...);
		static int ErrorRaw(const char* aErr, int len);
		
		// Convenient overloads for Error() and ErrorRaw()
		inline static int ErrorRaw(const char* aErr) {return ErrorRaw(aErr, strlen(aErr));}
#ifndef CLOG_PORTABLE_BUILD
		inline static int Error(const CString& aErr) {return ErrorRaw(aErr.GetPtr());}
		inline static int Error(const CString* aErr) {return ErrorRaw(aErr->GetPtr());}
#endif
		inline static int Error(const std::string& aErr) {return ErrorRaw(aErr.c_str());}
		inline static int Error(const std::string* aErr) {return ErrorRaw(aErr->c_str());}
		
		// Print a critical error message to the log file for stderr, the
		// differences between Error() and Critical() are:
		//
		// 1) Critical errors should be used when they terminate the app
		// 2) Critical errors are prepended with "[Critical]: "
		static int Critical(const char* aErr, ...);
		static int CriticalRaw(const char* aErr, int len);
		
		// Convenient overloads for Critical() and CriticalRaw
		inline static int CriticalRaw(const char* aErr) {return CriticalRaw(aErr, strlen(aErr));}
#ifndef CLOG_PORTABLE_BUILD
		inline static int Critical(const CString& aErr) {return CriticalRaw(aErr.GetPtr());}
		inline static int Critical(const CString* aErr) {return CriticalRaw(aErr->GetPtr());}
#endif
		inline static int Critical(const std::string& aErr) {return CriticalRaw(aErr.c_str());}
		inline static int Critical(const std::string* aErr) {return CriticalRaw(aErr->c_str());}
		
	private:
		static int SysOpen(int aOverwrite);
		static void SysClose();
		static int SysIsOpen();
		static void SysTimestamp(FILE* f);
		static void lock();
		static void unlock();
		
	private:
		static FILE* s_out;
		static FILE* s_err;
		static SceUID m_sema;
};

#endif
