/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CMaildirStore.h

  Description :	

*******************************************************************************/

#ifndef CMAILDIRSTORE_H_
#define CMAILDIRSTORE_H_

#include "VMimeTypes.h"
#include "CString.h"
#include "CMailbox.h"

#include <pspthreadman.h>

// Note that CMaildirStore subclasses from progressListener while it actually
// doesn't use the vmime progress listener mechanism. This is because it
// actually acts in a aimilar manner to the progress listener.
class CMaildirStore : public vmime::utility::progressListener, public CBase
{
	public: // Constructors & Destructors
	
		CMaildirStore(const CString& aPath, CMailbox* aMailbox, SceUID aMailboxMbx);
		~CMaildirStore();
		
	public: // Inherited from progressListener
		
		const bool cancel() const {return false;};
		void start(const int predictedTotal);
		void progress(const int current, const int currentTotal);
		void stop(const int total);
		
	public: // New methods
	
		void getMessages(CMailbox::TMessageList* aMsgList, bool sort = true, int from = 1, int to = -1);
		u32 getMessageCount();
		void addMessages(VMessageRefList aMsgs);
		void filterAlreadyRetrievedMessages(VMessageRefList& aMsgs,
		                                    bool prune = false);
		int deleteMessage(const int num);
		int deleteMessages(std::vector<int> nums);
		
		CString getBody(CMessageHeader* aMsg);
		void markRead(CMessageHeader* aMsg, bool aIsRead);
		void sortMessageList(CMailbox::TMessageList* aMsgList);
		
	private: // New methods
	
		void connect();
		void disconnect();
		void open(const int aMode); // V_MODE_READ_ONLY or V_MODE_READ_WRITE
		void close();
		
		void failed(CString aErrorText);
		
		void lock();
		void unlock();
		
		void sysGetMessages(CMailbox::TMessageList* aMsgList, bool sort = true, int from = 1, int to = -1);
		u32 sysGetMessageCount();
		void sysAddMessages(VMessageRefList aMsgs);
		void sysAddMessage(VMessageRef aMsg);
		
		void saveUids(); // Write to file
		void restoreUids(); // Read from file
		
		CMessageHeader* getMessageHeader(VMessageRef aMsg);
		time_t convertTime(vmime::ref<const vmime::datetime> aDatetime);
		
	private: // Data members
	
		SceUID             m_sema;
		SceUID             m_mailbox_mbx;
		CString            m_path;
		CMailbox*          m_mailbox;
		VMessageRefList    m_list;
		VSessionRef        m_sess_ref;
		VStoreRef          m_store_ref;
		VFolderRef         m_folder_ref;
		
		// Corresponds to UIDs of messages that have been retrieved but may
		// still be on the server
		std::vector<std::string> m_uids;
};

#endif /*CMAILDIRSTORE_H_*/
