/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CMessageViewWindow.cpp

  Description :	

  Last Modified $Date: 2008/09/29 04:22:27 $

  $Revision: 1.5 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CMessageViewWindow.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CInput.h"
#include "CGfx.h"
#include "CSkinManager.h"
#include "CProcess.h"
#include "CFrameWork.h"
#include "CControlTextBox.h"
#include "CHUHelp.h"
#include "CMailbox.h"
#include "irda_KeyboardLayouts.h"
#include "CKeyboardNavWrapper.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const float	WINDOW_BORDER(2.f); // Border around context menu (all sides)
static const float  WINDOW_PADDING(8.f); // Padding between border and menu item (all sides)
static const float	ITEM_PADDING(4.f);

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CMessageViewWindow::CMessageViewWindow(CMessageHeader* message, CMailbox* aMailbox):
	m_pComposeWindow(NULL),
	m_pMessage(message),
	m_pMailbox(aMailbox),
	m_size(0,0),
	m_exitValue(-1),
	m_pContextMenu(NULL),
	m_nPageRepeatCount(0)
{
	m_text = new CText("--- Retrieving ---");
	
	m_text->reset();
	m_text->insert(m_pMailbox->getBody(message).GetStr());
	m_scrollableText = new CScrollableText(m_text, V2(CGfx::s_ScreenWidth - 10, CGfx::s_ScreenHeight - 42), V2(5, 42), CSkinManager::GetColor( "dialog_box", "text_color", 0xFFFFFFFF));
	if (message->unread)
		m_pMailbox->markRead(message, true);
	
	V2 text_size(0, 0);
	m_size.x = 470;
	/* BEN NOTE: Pre CText code
	// If text is provided, determine text_size
	if (!m_text.IsEmpty()) {
		m_text = CFont::GetDefaultFont()->GetWrappedString(m_text, m_size.x);
		text_size = CFont::GetDefaultFont()->GetStringSize(m_text);
	}
	*/
	
    SetFocus(true);
}

//******************************************************************************
//	
//******************************************************************************
CMessageViewWindow::~CMessageViewWindow()
{
	SAFE_DELETE(m_scrollableText);
	SAFE_DELETE(m_text);
}	

//******************************************************************************
//	
//******************************************************************************
void	CMessageViewWindow::Render()
{
	// Draw background
	CGfx::DrawQuad( V2( 0, 0 ), V2( CGfx::s_ScreenWidth, CGfx::s_ScreenHeight), CSkinManager::GetColor( "main_window", "background_color", 0xFFFFFFFF));
	
	// Draw the text
	//int textYLocation = 38;
	//CFont::GetDefaultFont()->Print(m_text, V2(5, textYLocation), CSkinManager::GetColor( "dialog_box", "text_color", 0xFFFFFFFF));
	//CFont::GetDefaultFont()->Print(m_text, m_text->begin(), V2(CGfx::s_ScreenWidth - 10, CGfx::s_ScreenHeight - 38), V2(5, textYLocation), CSkinManager::GetColor( "dialog_box", "text_color", 0xFFFFFFFF));
	
	m_scrollableText->render();
	
	// Draw Header background
	CGfx::DrawQuad( V2( 0, 0 ), V2( CGfx::s_ScreenWidth, 35), CSkinManager::GetColor( "main_window", "background_color", 0xFFFFFFFF));
	CString m_MessageFrom("From: ");
	m_MessageFrom += m_pMessage->from->getEmail();
	CFont::GetActiveTextFont()->Print(m_MessageFrom, V2(5, 5), 0xff000000);
	
	V2 date_size(0, 0);
	struct tm ltm;
	struct tm* msgtm;
	char msgDate[64];
	CString msgDateStr;
	msgtm = localtime(&m_pMessage->date);
	if (ltm.tm_year == msgtm->tm_year && ltm.tm_mon == msgtm->tm_mon &&
	    ltm.tm_mday == msgtm->tm_mday)
	{
		strftime(msgDate, 64, "%H:%M", msgtm);
	}
	else
	{
		strftime(msgDate, 64, "%d/%m/%y %H:%M", msgtm);
	}
	msgDateStr = msgDate;
	date_size = CFont::GetActiveTextFont()->GetStringSize(msgDateStr);
	CFont::GetActiveTextFont()->Print(msgDateStr, V2(480-date_size.x, 5), 0xff000000);
	
	CString m_MessageSubject("Subject: ");
	m_MessageSubject += m_pMessage->subject;
	CFont::GetActiveTextFont()->Print(m_MessageSubject, V2(5, 22), 0xff000000);
	
	CGfx::DrawQuad( V2(0, 40), V2( CGfx::s_ScreenWidth, 1),  CSkinManager::GetColor( "main_window", "title_bar_outline_color", 0xFF000000));
}
//******************************************************************************
//	
//******************************************************************************
void	CMessageViewWindow::Process()
{
	if (m_pContextMenu == NULL && m_pComposeWindow == NULL) {
		CHUHelp::Clear();
		CHUHelp::SetButton(CHUHelp::BUTTON_UP, "Scroll Up");
		CHUHelp::SetButton(CHUHelp::BUTTON_DOWN, "Scroll Down");
		CHUHelp::SetButton(CHUHelp::BUTTON_TRIANGLE, "Context Menu");
		CHUHelp::SetButton(CHUHelp::BUTTON_CROSS, "Close");
		CHUHelp::SetButton(CHUHelp::BUTTON_LTRIGGER, "Page Up");
		CHUHelp::SetButton(CHUHelp::BUTTON_RTRIGGER, "Page Down");
	}

	if ( m_pComposeWindow != NULL ) {
		if ( m_pComposeWindow->getExitValue() != -1 ) {
			//Menu has return a value, process it see eMenuItems enum
			switch(m_pComposeWindow->getExitValue()) {
				case CComposeWindow::OK:
					break;
				default:
					break;
			}
			//Remove the compose window
			SAFE_DELETE(m_pComposeWindow);

			//Return the focus to this window
			SetFocus(true);
			SetVisible(true);
		}
	}	

	if (m_pContextMenu != NULL) {
		if ( m_pContextMenu->getExitValue() != -1 ) {
			//Menu has return a value, process it see eMenuItems enum
			switch(m_pContextMenu->getExitValue()) {
				case HIDE:
					break;									   
				case REPLY:
					composeReplyFromSelected();
					SAFE_DELETE(m_pContextMenu);
					SetFocus(false);
					SetVisible(false);
					return;
				case REPLYALL:
					composeReplyFromSelected();
					SAFE_DELETE(m_pContextMenu);
					SetFocus(false);
					SetVisible(false);
					return;
				case FORWARD:
					composeForwardFromSelected();
					SAFE_DELETE(m_pContextMenu);
					SetFocus(false);
					SetVisible(false);
					return;
				case DELETE:
					m_exitValue = REMOVE; 
					break;
				case CLOSE:
					m_exitValue = OK; 
					break;
				default:
					break;
			}
			//Remove the context menu
			SAFE_DELETE(m_pContextMenu);

			//Return the focus to this window
			SetFocus( true );
		}
    }
	CWindow::Process();
}

//******************************************************************************
//
//******************************************************************************
bool	CMessageViewWindow::ProcessInput()
{
    if (HasFocus() == true) {
		if (CKeyboardNavWrapper::OpenContextMenu()) {
			// Buld the context menu
			CContextMenu::CContextMenuItemList * itemList = new CContextMenu::CContextMenuItemList();
			
			itemList->push_back( new CContextMenuItem("Hide Menu", HIDE, true));
			itemList->push_back( new CContextMenuItem("Reply", REPLY, false));
			itemList->push_back( new CContextMenuItem("Reply All", REPLYALL, false));
			itemList->push_back( new CContextMenuItem("Forward", FORWARD, false));
			itemList->push_back( new CContextMenuItem("Delete", DELETE, true));
			itemList->push_back( new CContextMenuItem("Close", CLOSE, false));
			
			// Create the context menu
			m_pContextMenu = new CContextMenu(itemList, V2(10, 10));
						
			SetFocus( false );
		}
    	
		if (CKeyboardNavWrapper::Up()) {
			m_scrollableText->up();
		}
	    
		if (CKeyboardNavWrapper::Down()) {
			m_scrollableText->down();
		}
		
		if (CKeyboardNavWrapper::PageUp()) {
			m_scrollableText->pageUp();
			
			++m_nPageRepeatCount;
		}
		
		if (CKeyboardNavWrapper::PageDown()) {
			m_scrollableText->pageDown();
			
			++m_nPageRepeatCount;
		}
		
		if (CKeyboardNavWrapper::CloseMessage()) {
			m_exitValue = OK;
		}
	}

    return false;
}

//******************************************************************************
//
//******************************************************************************
int	CMessageViewWindow::getExitValue() {
	return m_exitValue;
}

void CMessageViewWindow::composeReplyFromSelected()
{
	CString tempText("");

	CControlAddress::AddressEntryList* to = new CControlAddress::AddressEntryList();
	to->push_back(new InputBoxAddressEntry(m_pMessage->from->getEmail(), m_pMessage->from->getEmail()));

	tempText.Printf("Re: %s", (const char*)m_pMessage->subject);
	CText* subject = new CText((const char *)tempText);

	CText* body = new CText("Original Message:\n");
	body->insert("-----------------------------------\n");
	body->insert(m_pMailbox->getBody(m_pMessage).GetStr());

	m_pComposeWindow = new CComposeWindow(m_pMailbox,
											to,
											NULL,
											NULL,
											subject,
											body);
}

void CMessageViewWindow::composeReplyAllFromSelected()
{
	CString tempText("");

	CControlAddress::AddressEntryList* to = new CControlAddress::AddressEntryList();
	to->push_back(new InputBoxAddressEntry(m_pMessage->from->getEmail(), m_pMessage->from->getEmail()));

	tempText.Printf("Re: %s", (const char*)m_pMessage->subject);
	CText* subject = new CText((const char *)tempText);

	CText* body = new CText("Original Message:\n");
	body->insert("-----------------------------------\n");
	body->insert(m_pMailbox->getBody(m_pMessage).GetStr());

	ASSERT(0, "1");

	m_pComposeWindow = new CComposeWindow(m_pMailbox,
											to,
											NULL,
											NULL,
											subject,
											body);
}

void CMessageViewWindow::composeForwardFromSelected()
{
	CString tempText("");
	tempText.Printf("Fwd: %s", (const char*)m_pMessage->subject);
	CText* subject = new CText((const char *)tempText);

	CText* body = new CText("Forwarded Message:\n");
	body->insert("-----------------------------------\n");
	body->insert(m_pMailbox->getBody(m_pMessage).GetStr());

	m_pComposeWindow = new CComposeWindow(m_pMailbox,
											NULL,
											NULL,
											NULL,
											subject,
											body);
}
//*******************************  END OF FILE  ********************************
