/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CMessagesWindow.cpp

  Description :	

  Last Modified $Date: 2008/09/29 04:22:27 $

  $Revision: 1.8 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CMessagesWindow.h"
#include "CMessageViewWindow.h"
#include "CFrameWork.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CInput.h"
#include "CSkinManager.h"
#include "CGfx.h"
#include <psprtc.h>
#include "CWindow.h"
#include "CPSPoste.h"
#include "CContextMenu.h"
#include "CContextMenuItem.h"
#include "CHUHelp.h"
#include "CControlManager.h"
#include "CControlLabel.h"
#include "CControlHeader.h"
#include "CControlScrollBar.h"
#include "CConfigFile.h"
#include "CConnectionManager.h"
#include "irda_KeyboardLayouts.h"
#include "CKeyboardNavWrapper.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

/* General message list consts */
#define FIRST_MSG_Y_POSITION 65
#define MSG_Y_SPACING 21
#define ROW_HEIGHT 16
#define SUBJECT_COL 0
#define FROM_COL 1
#define DATE_COL 2

/* Column label sizes and positions */
#define SUBJECT_COL_WIDTH 235
#define SUBJECT_COL_X 24
#define FROM_COL_WIDTH 115
#define FROM_COL_X 260
#define DATE_COL_WIDTH 92
#define DATE_COL_X 376

/* The "Your mailbix is empty" label */
#define EMPTY_LABEL_WIDTH CGfx::s_ScreenWidth
#define EMPTY_LABEL_HEIGHT 16
#define EMPTY_LABEL_X 0
#define EMPTY_LABEL_Y 65

/* The progress bar */
#define PROGRESS_BAR_WIDTH CGfx::s_ScreenWidth
#define PROGRESS_BAR_HEIGHT 16
#define PROGRESS_BAR_X 0
#define PROGRESS_BAR_Y 43
#define PROGRESS_BAR_PHASE_COUNT 2

/* The scroll bar */
#define SCROLL_BAR_WIDTH 12
#define SCROLL_BAR_HEIGHT 212
#define SCROLL_BAR_X 468
#define SCROLL_BAR_Y 59

/* Context menu return value test text */
#define CONTEXT_RETURN_TEST_X 7
#define CONTEXT_RETURN_TEST_Y 47

/* Highlight bar of selected messages */
#define SELECTED_BORDER_WIDTH 2
#define SELECTED_BAR_WIDTH 468
#define SELECTED_BAR_X 0

/* Icons Saj todo: these belong in the skin manager */
#define ICON_WIDTH 16
#define ICON_HEIGHT 16
#define ICON_X 4

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//******************************************************************************
//
//	Main Window implementation
//
//******************************************************************************
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CMessagesWindow::CMessagesWindow(CMailbox* mailbox) :
	m_pCContextMenu(NULL),
	m_pComposeWindow(NULL),
	m_pMessageViewWindow(NULL),
	m_pErrorDlg(NULL),
	m_pMailbox(mailbox),
	m_pControlManager(NULL),
	m_pEmptyListLabel(NULL),	
	m_pControlHeader(NULL),
	m_pProgressBar(NULL),
	m_pScrollBar(NULL),
	m_pSelectedMessage(NULL), // These have to happen in case the list is empty
	m_selectedMsgNum(0),
	m_pFirstMessage(NULL),
	m_firstMsgNum(0),
	m_totalMarkedCount(0)
{
	SetFocus(true);

	m_pControlManager = new CControlManager();
	m_pScrollBar = new CControlScrollBar(V2(SCROLL_BAR_WIDTH, SCROLL_BAR_HEIGHT),
										 V2(SCROLL_BAR_X, SCROLL_BAR_Y),
										 DISPLAYED_MSG_COUNT);
	m_pControlManager->addControl(m_pScrollBar);

	m_pDisplayedMessages = new CMailbox::TMessageList();
	
	// Lets instantiate the message labels
	int yPosition = FIRST_MSG_Y_POSITION;
	for (int i = 0; i < DISPLAYED_MSG_COUNT; i++)
	{
		m_pMessageLabels[i][SUBJECT_COL] = new CControlLabel(
		               V2(SUBJECT_COL_WIDTH, ROW_HEIGHT),
		               V2(SUBJECT_COL_X, yPosition),
		               CSkinManager::GetColor("message_list",
		                                      "inactive_text_color",
		                                      0xFF96CBFE),
		               CString("Test1"),
		               CControlLabel::MIDDLE_ALIGN
		           );

		m_pMessageLabels[i][FROM_COL] = new CControlLabel(
		              V2(FROM_COL_WIDTH, ROW_HEIGHT), V2(FROM_COL_X, yPosition),
		              CSkinManager::GetColor("message_list",
		                                     "inactive_text_color",
		                                     0xFF96CBFE),
		              CString("Test2"),
		              CControlLabel::MIDDLE_ALIGN
		          );

		m_pMessageLabels[i][DATE_COL] = new CControlLabel(
		              V2(DATE_COL_WIDTH, ROW_HEIGHT), V2(DATE_COL_X, yPosition),
		              CSkinManager::GetColor("message_list",
		                                     "inactive_text_color",
		                                     0xFF96CBFE),
		              CString("Test3"),
					  CControlLabel::MIDDLE_ALIGN|CControlLabel::RIGHT_JUSTIFY
		          );

		yPosition += MSG_Y_SPACING;
	}

	// This label is used when the message list is empty
	m_pEmptyListLabel = new CControlLabel(
	             V2(EMPTY_LABEL_WIDTH, EMPTY_LABEL_HEIGHT),
	             V2(EMPTY_LABEL_X, EMPTY_LABEL_Y),
	             CSkinManager::GetColor("message_list",
	                                    "inactive_text_color",
	                                    0xFF96CBFE),
	             CString("Your Inbox is Empty"),
	             CControlLabel::MIDDLE_JUSTIFY|CControlLabel::MIDDLE_ALIGN
	         );

	
	m_pControlHeader = new CControlHeader();
	m_pControlHeader->setTitle("Messages");
	m_pControlHeader->setConnection(CConnectionManager::GetPSPConnectionName(CConfigFile::s_pSettings->connection_selected));
	m_pControlHeader->setAccount(CConfigFile::s_pSettings->selectedAccount->display_name);
	m_pControlManager->addControl(m_pControlHeader);

	// The progress bar
	m_pProgressBar = new CControlProgressBar(
	             V2(PROGRESS_BAR_WIDTH, PROGRESS_BAR_HEIGHT),
	             V2(PROGRESS_BAR_X, PROGRESS_BAR_Y),
	             PROGRESS_BAR_PHASE_COUNT,
	             CSkinManager::GetColor("main_window",
	                                    "selected_icon_color",
	                                    0xFF96CBFE),
	             CString("Connecting")
	         );

	m_pProgressBar->hide();
	m_pMailbox->subscribe(this);
	SortBySettings();
	BuildDisplayListFromScratch();
}

//******************************************************************************
//	
//******************************************************************************
CMessagesWindow::~CMessagesWindow()
{
	m_pMailbox->unsubscribe(this);
	
	// Delete the list
	SAFE_DELETE(m_pDisplayedMessages);
	SAFE_DELETE(m_pMessageViewWindow);
	SAFE_DELETE(m_pCContextMenu);

	for (int i = 0; i < DISPLAYED_MSG_COUNT; i++)
	{
	 	for (int j = 0; j < DISPLAYED_COL_COUNT; j++)
	 	{
			SAFE_DELETE(m_pMessageLabels[i][j]);
		}
	}

	SAFE_DELETE(m_pEmptyListLabel);
	SAFE_DELETE(m_pControlManager);
	SAFE_DELETE(m_pProgressBar);
}

//******************************************************************************
//	
//******************************************************************************
void	CMessagesWindow::Render()
{
	// Draw white background
	CGfx::DrawQuad(V2(0, 0),
	              V2( CGfx::s_ScreenWidth, CGfx::s_ScreenHeight),
	              CSkinManager::GetColor("message_list", "background_color",
	                                     0x0FFFFFFF));

	// Draw Header Control
	m_pControlHeader->setNewMessages(m_pMailbox->getUnreadMessageCount());
	m_pControlManager->RenderControl();
			 	 		
	if (m_pMailbox->getTotalMessageCount() != 0)
	{
		int yPosition = FIRST_MSG_Y_POSITION;
		int selectedYPosition = yPosition + m_selectedMsgNum * MSG_Y_SPACING;
		
		// Draw Selected Message Box under the currently selected item
		CGfx::DrawQuad(V2(0, selectedYPosition - SELECTED_BORDER_WIDTH),
		      V2(SELECTED_BAR_X + SELECTED_BAR_WIDTH, ROW_HEIGHT + 2*SELECTED_BORDER_WIDTH),
		      CSkinManager::GetColor("message_list",
		                             "selected_msg_hilight_border_color",
		                             0xFF96CBFE));

		CGfx::DrawQuad(V2(SELECTED_BAR_X + SELECTED_BORDER_WIDTH, selectedYPosition),
		      V2(SELECTED_BAR_WIDTH - 2*SELECTED_BORDER_WIDTH, ROW_HEIGHT),
		      CSkinManager::GetColor("message_list",
		                             "selected_msg_hilight_color",
		                             0xFF96CBFE));

		ARGB textColor;
		int msgNum = 0;
		
		// Display messages
		CMailbox::TMessageList::iterator it = m_pDisplayedMessages->begin();
		while( it !=  m_pDisplayedMessages->end() )
		{
		 	if ((*it)->marked)
			{
	  		   	if (yPosition != selectedYPosition)
				{
					CGfx::DrawQuad(V2(0, yPosition),
					      V2(SELECTED_BAR_WIDTH, ROW_HEIGHT),
					      CSkinManager::GetColor("message_list",
					                             "marked_msg_hilight_color",
					                             0xFF96CBFE));   
				}

				CGfx::DrawQuad(CSkinManager::GetComponent(CSkinManager::SC_ICON_MAIL_MARK)->GetTexture(),
				               V2(ICON_X, yPosition),
				               V2(ICON_WIDTH, ICON_HEIGHT),
				               0xffffffff);

				textColor = CSkinManager::GetColor("message_list",
				                                   "marked_text_color",
				                                   0xFF96CBFE);
			}
			else
			{   
				if ((*it)->unread)
				{
					CGfx::DrawQuad(CSkinManager::GetComponent(CSkinManager::SC_ICON_NEWMAIL)->GetTexture(),
					               V2(ICON_X, yPosition),
					               V2(ICON_WIDTH, ICON_HEIGHT),
					               0xffffffff);
				}
				else
				{
					CGfx::DrawQuad(CSkinManager::GetComponent(CSkinManager::SC_ICON_MAIL_OPEN)->GetTexture(),
					               V2(ICON_X, yPosition),
					               V2(ICON_WIDTH, ICON_HEIGHT),
					               0xffffffff);
				}

				textColor = CSkinManager::GetColor("message_list",
				                                   "inactive_text_color",
				                                   0xFF96CBFE);
			}
			
			if (msgNum == m_selectedMsgNum)
			{
				textColor = CSkinManager::GetColor("message_list",
				                                   "selected_text_color",
				                                   0xFF96CBFE);
				   		   
			}

			for (int i = 0; i < DISPLAYED_COL_COUNT; i++)
			{
			 	m_pMessageLabels[msgNum][i]->setColor(textColor);
				m_pMessageLabels[msgNum][i]->RenderControl();
			}

			yPosition += MSG_Y_SPACING;
			it++;
			msgNum++;
		}
	}
	else
	{
	 	m_pEmptyListLabel->RenderControl();
 	}
	
	m_pProgressBar->RenderControl();
	 	
}

//******************************************************************************
//	
//******************************************************************************
void	CMessagesWindow::Process()
{
	if ( m_pCContextMenu == NULL && m_pComposeWindow == NULL && m_pMessageViewWindow == NULL && m_pErrorDlg == NULL) {
		CHUHelp::Clear();
		CHUHelp::SetButton(CHUHelp::BUTTON_UP, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_DOWN, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_TRIANGLE, "Context Menu");
		CHUHelp::SetButton(CHUHelp::BUTTON_LTRIGGER, "Page Up");
		CHUHelp::SetButton(CHUHelp::BUTTON_RTRIGGER, "Page Down");
		CHUHelp::SetButton(CHUHelp::BUTTON_SQUARE, "Mark");
		CHUHelp::SetButton(CHUHelp::BUTTON_CROSS, "Open");
	}
	
	if ( m_pMessageViewWindow != NULL ) {
		if ( m_pMessageViewWindow->getExitValue() != -1 ) {
		   	bool deleteMsg = false; 
			//Menu has return a value, process it see eMenuItems enum
			switch(m_pMessageViewWindow->getExitValue()) {
				case CMessageViewWindow::OK:
					break;
				case CMessageViewWindow::REMOVE:
					deleteMsg = true;
					break;
				default:
					break;
			}
			//Remove the context menu
			SAFE_DELETE(m_pMessageViewWindow);

			//Return the focus to this window
			SetFocus(true);
			SetVisible(true);
			
			if (deleteMsg)
			{
				DeleteMessage();   			  
			}
		}
	}
	
	if (m_pErrorDlg != NULL)
	{
		if (m_pErrorDlg->getExitValue() != -1)
		{
			SAFE_DELETE(m_pErrorDlg);
			SetFocus(true);
		}
	}
	
	if ( m_pComposeWindow != NULL ) {
		if ( m_pComposeWindow->getExitValue() != -1 ) {
			//Menu has return a value, process it see eMenuItems enum
			switch(m_pComposeWindow->getExitValue()) {
				case CComposeWindow::OK:
					break;
				default:
					break;
			}
			//Remove the compose window
			SAFE_DELETE(m_pComposeWindow);

			//Return the focus to this window
			SetFocus(true);
			SetVisible(true);
		}
	}

	if ( m_pCContextMenu != NULL ) {
		if ( m_pCContextMenu->getExitValue() != -1 ) {
			bool contextDestroyed = false;	 	   	 
			//Menu has return a value, process it see eMenuItems enum
			switch(m_pCContextMenu->getExitValue()) {
				case HIDE:
					break;
				case OPEN:
					SAFE_DELETE(m_pCContextMenu);
					SetFocus(true);
					contextDestroyed = true;
					OpenMessage();
					break;
				case MARK:
					MarkMessage();
					break;									
				case REPLY:
					composeReplyFromSelected();
					SAFE_DELETE(m_pCContextMenu);
					SetFocus(false);
					SetVisible(false);
					contextDestroyed = true;
					break;
				case REPLYALL:
					composeReplyAllFromSelected();
					SAFE_DELETE(m_pCContextMenu);
					SetFocus(false);
					SetVisible(false);
					contextDestroyed = true;
					break;
				case FORWARD:
					composeForwardFromSelected();
					SAFE_DELETE(m_pCContextMenu);
					SetFocus(false);
					SetVisible(false);
					contextDestroyed = true;
					break;
				case CLOSE:
					CPSPoste::s_NewWindow = CWindow::WINDOW_MAIN;
					break;
				case DELETE:
					DeleteMessage();
					break;
				case DELETEMARKED:
					DeleteMarkedMessages();
					break;
				case CHECKMAIL:
					InitProgressBar();
					m_pMailbox->retrieveNewMessages();
					break;
				case SORT_DATE_ASC:
					m_pMailbox->setSort(new CMailboxSortDate);
					SortChanged();
					CConfigFile::s_pSettings->message_sort_style = "DATE_ASC";
					break;
				case SORT_DATE_DESC:
					m_pMailbox->setSort(new CMailboxDescSort(new CMailboxSortDate));
					SortChanged();
					CConfigFile::s_pSettings->message_sort_style = "DATE_DESC";
                    break;
				case SORT_SUBJECT_ASC:
					m_pMailbox->setSort(new CMailboxSortSubject);
					SortChanged();
					CConfigFile::s_pSettings->message_sort_style = "SUBJECT_ASC";
                    break;
                case SORT_SUBJECT_DESC:
					m_pMailbox->setSort(new CMailboxDescSort(new CMailboxSortSubject));
					SortChanged();
					CConfigFile::s_pSettings->message_sort_style = "SUBJECT_DESC";
                    break;
				case SORT_FROM_ASC:
					m_pMailbox->setSort(new CMailboxSortFrom);
					SortChanged();
					CConfigFile::s_pSettings->message_sort_style = "FROM_ASC";
                    break;
                case SORT_FROM_DESC:
					m_pMailbox->setSort(new CMailboxDescSort(new CMailboxSortFrom));
					SortChanged();
					CConfigFile::s_pSettings->message_sort_style = "FROM_DESC";
                    break;
				default:
					break;
			}
			if (!contextDestroyed)
			{
				//Remove the context menu
				SAFE_DELETE(m_pCContextMenu);
				//Return the focus to this window
				SetFocus(true);
			}
		}
    }
    
	CWindow::Process();
}

//******************************************************************************
//	
//******************************************************************************
bool	CMessagesWindow::ProcessInput()
{
	if ( HasFocus() == true && m_pErrorDlg == NULL ) {
	   	if ( CKeyboardNavWrapper::OpenContextMenu() ) {
			// Buld the context menu

			CContextMenu::CContextMenuItemList* itemList = new CContextMenu::CContextMenuItemList();
		
			CContextMenu::CContextMenuItemList* sortList = new CContextMenu::CContextMenuItemList();
	
			sortList->push_back(new CContextMenuItem("Date Asc", SORT_DATE_ASC, false));
			
			sortList->push_back(new CContextMenuItem("Date Desc", SORT_DATE_DESC, false));
			sortList->push_back(new CContextMenuItem("Subject Asc", SORT_SUBJECT_ASC, false));
			sortList->push_back(new CContextMenuItem("Subject Desc", SORT_SUBJECT_DESC, false));
			sortList->push_back(new CContextMenuItem("From Asc", SORT_FROM_ASC, false));
			sortList->push_back(new CContextMenuItem("From Desc", SORT_FROM_DESC, false));


			itemList->push_back( new CContextMenuItem("Hide Menu", HIDE, true));
			
			if (!m_pMailbox->isRetrieving())
				itemList->push_back( new CContextMenuItem("Check Mail", CHECKMAIL, true));


			itemList->push_back(new CContextMenuItem("< Sort", -1, true, sortList));
				
			if (m_pMailbox->getTotalMessageCount() != 0)
			{
				itemList->push_back( new CContextMenuItem("Open", OPEN, false));
				if (m_pSelectedMessage->marked)
				{
					itemList->push_back( new CContextMenuItem("Unmark", MARK, false));
				}
				else
				{
					itemList->push_back( new CContextMenuItem("Mark", MARK, false));
				}
				itemList->push_back( new CContextMenuItem("Reply", REPLY, false));
				itemList->push_back( new CContextMenuItem("Reply All", REPLYALL, false));
				itemList->push_back( new CContextMenuItem("Forward", FORWARD, false));
				
				if (m_pSelectedMessage->marked)
				{
				   itemList->push_back( new CContextMenuItem("Delete Marked", DELETEMARKED, true));
				}
				else
				{
				 	itemList->push_back( new CContextMenuItem("Delete", DELETE, true));
				}
			}
			itemList->push_back( new CContextMenuItem("Close", CLOSE, false));
			
			// Create the context menu
			m_pCContextMenu = new CContextMenu(itemList, V2(10, 10));
						
			SetFocus( false );
		}
		 
	   	if (m_pMailbox->getTotalMessageCount() == 0)
		{
		   	return false;
		} 
		
		if (CKeyboardNavWrapper::Up())
		{
			MoveUp();
		}
		else if (CKeyboardNavWrapper::Down())
		{
			MoveDown();
		}
		else if (CKeyboardNavWrapper::PageUp())
		{
			MovePageUp();
		}
		else if (CKeyboardNavWrapper::PageDown())
		{
			MovePageDown();
		}

		if (CKeyboardNavWrapper::MarkMessageToggle()) {
			MarkMessage();
		}
		
		if (CKeyboardNavWrapper::ActivateControl()) {
			OpenMessage();
		}
	}

	return false;
}

void CMessagesWindow::MoveUp()
{
	if (m_selectedMsgNum != 0) {
		m_selectedMsgNum--;
		CMailbox::TMessageList::iterator it = m_pDisplayedMessages->begin();
		while( it !=  m_pDisplayedMessages->end() )
		{
			if ((*it) == m_pSelectedMessage)
			{
				it--;
				m_pSelectedMessage = (*it);
				break;	
			}				   	
			it++;
		}
	}
	else
	{
		if (m_firstMsgNum != 0)
		{
		   	m_firstMsgNum--;
		   	int i = 0;				
		   	CMailbox::TMessageList::iterator it = m_pMailbox->m_pMessageList->begin();
		   	while(it !=  m_pMailbox->m_pMessageList->end())
			{
				if ((*it) == m_pFirstMessage)
				{
					it--;						   		  
					m_pFirstMessage = (*it);
					m_pSelectedMessage = (*it);
					m_pDisplayedMessages->clear();
					i = 1;
				}
				if (i > 0)
				{
					m_pDisplayedMessages->push_back((*it));
					i++;
				}
				if (i == (DISPLAYED_MSG_COUNT+1))
				{
					break;
				}
				it++; 						
			}								   									   
		}	   
	}
	UpdateDisplayListText();
}

void CMessagesWindow::MoveDown()
{
	if (m_selectedMsgNum != (DISPLAYED_MSG_COUNT-1)) {
	   	// If we are not at the end of the displayed message list
		if (m_selectedMsgNum != (m_pMailbox->getTotalMessageCount() - 1)) {
		 	// There are > 10 messages in the mailbox, we can move down					     				  
		 	m_selectedMsgNum++;
		 	CMailbox::TMessageList::iterator it = m_pDisplayedMessages->begin();
		 	while(it !=  m_pDisplayedMessages->end())
		 	{
			 	if ((*it) == m_pSelectedMessage)
			 	{
				 	it++;
				 	m_pSelectedMessage = (*it);
				 	break;	
			 	}				   	
			 	it++;
		 	}
		}
	}
	else
	{
	 	// We are at the end of the displayed message list
		if ((m_firstMsgNum + (DISPLAYED_MSG_COUNT+1)) <= m_pMailbox->getTotalMessageCount())
		{
		   	m_firstMsgNum++;
		   	int i = 0;
		   	CMailbox::TMessageList::iterator it = m_pMailbox->m_pMessageList->begin();
		   	while(it !=  m_pMailbox->m_pMessageList->end())
			{
				if ((*it) == m_pFirstMessage)
				{
					it++;
					m_pFirstMessage = (*it);
					m_pDisplayedMessages->clear();
					i = 1;
				}
				if (i > 0)
				{
					m_pDisplayedMessages->push_back((*it));
					i++;
				}
				if (i == (DISPLAYED_MSG_COUNT+1))
				{
					m_pSelectedMessage = (*it);
					break;
				}
				it++;
			}
		}
	}
	UpdateDisplayListText();
}

void CMessagesWindow::MovePageUp()
{
	if (m_firstMsgNum >= DISPLAYED_MSG_COUNT)
	{
		m_firstMsgNum -= DISPLAYED_MSG_COUNT;
		CMailbox::TMessageList::iterator it = m_pMailbox->m_pMessageList->begin();
		int i = 0;
		it = m_pMailbox->m_pMessageList->begin();
		while(it !=  m_pMailbox->m_pMessageList->end())
		{
			if ((*it) == m_pFirstMessage)
			{
				it -= DISPLAYED_MSG_COUNT;
				m_pFirstMessage = (*it);
				m_pSelectedMessage = (*it);
				m_pDisplayedMessages->clear();
				i = 1;
			}
			if (i > 0)
			{
				m_pDisplayedMessages->push_back((*it));
				i++;
			}
			if (i == (DISPLAYED_MSG_COUNT+1))
			{
				break;
			}
			it++;
		}
		m_selectedMsgNum = 0;
	}
	else if (m_firstMsgNum > 0 && m_firstMsgNum < DISPLAYED_MSG_COUNT)
	{
	 	m_firstMsgNum = 0;
		CMailbox::TMessageList::iterator it = m_pMailbox->m_pMessageList->begin();
		int i = 0;
		it = m_pMailbox->m_pMessageList->begin();
		while(it !=  m_pMailbox->m_pMessageList->end())
		{
			if (i == 0)
			{
				m_pFirstMessage = (*it);
				m_pSelectedMessage = (*it);
				m_pDisplayedMessages->clear();
			}
			m_pDisplayedMessages->push_back((*it));
			i++;
			if (i == DISPLAYED_MSG_COUNT)
			{
				break;
			}
			it++;
		}
		m_selectedMsgNum = 0;
	}
	UpdateDisplayListText();
}

void CMessagesWindow::MovePageDown()
{
	if ((m_firstMsgNum + DISPLAYED_MSG_COUNT) < m_pMailbox->getTotalMessageCount())
	{
		m_firstMsgNum += DISPLAYED_MSG_COUNT;
		CMailbox::TMessageList::iterator it = m_pMailbox->m_pMessageList->begin();
		int i = 0;
		if ((m_firstMsgNum + DISPLAYED_MSG_COUNT) > m_pMailbox->getTotalMessageCount())
		{
			m_firstMsgNum = m_pMailbox->getTotalMessageCount() - DISPLAYED_MSG_COUNT;
			while(it !=  m_pMailbox->m_pMessageList->end())
			{
			 	if (i == m_firstMsgNum)
				{
					m_pFirstMessage = (*it);
					m_pSelectedMessage = (*it);
					m_pDisplayedMessages->clear();
				}
				if (i >= m_firstMsgNum)
				{
					m_pDisplayedMessages->push_back((*it));
				}
				i++;
				it++;
			}
		}
		else
		{   							   									   
			it = m_pMailbox->m_pMessageList->begin();
			while(it !=  m_pMailbox->m_pMessageList->end())
			{
				if ((*it) == m_pFirstMessage)
				{
					it += DISPLAYED_MSG_COUNT;
					m_pFirstMessage = (*it);
					m_pSelectedMessage = (*it);
					m_pDisplayedMessages->clear();
					i = 1;
				}
				if (i > 0)
				{
					m_pDisplayedMessages->push_back((*it));
					i++;
				}
				if (i == (DISPLAYED_MSG_COUNT+1))
				{
					break;
				}
				it++;
			}
		}
		m_selectedMsgNum = 0;
	}
	UpdateDisplayListText();
}

void	CMessagesWindow::OpenMessage()
{
	m_pMessageViewWindow = new CMessageViewWindow(m_pSelectedMessage, m_pMailbox);
	SetFocus( false );
	SetVisible(false);
}

void	CMessagesWindow::MarkMessage()
{
	m_pSelectedMessage->marked = !m_pSelectedMessage->marked;
	if (m_pSelectedMessage->marked)
	{
		m_totalMarkedCount++;
	}
	else
	{
		m_totalMarkedCount--;
	}
}

void	CMessagesWindow::DeleteMessage()
{
	bool deletedLast = false;
	
	// Are we deleting the last message in the list?
	if (m_pSelectedMessage == (*m_pMailbox->m_pMessageList)[m_pMailbox->getTotalMessageCount()-1])
		deletedLast = true;
	
	// This will delete the message from the list and flag it for delete from
	// the filesystem on next expunge (ie: when the app quits)
	if (m_pMailbox->deleteMessage(m_pSelectedMessage))
		return;

	// Special case of deleting the last message in the list
	if (deletedLast)
	{
		// Special sub-case of deleting the only message in the list
		if (m_selectedMsgNum == 0) // Message list now empty
		{
			m_pSelectedMessage = NULL;
			m_pFirstMessage = NULL;
		}
		// Special sub-case of having less than 10 messages left
		else if (m_pMailbox->m_pMessageList->size() < DISPLAYED_MSG_COUNT)
		{
			m_selectedMsgNum--;
			m_pSelectedMessage = (*m_pMailbox->m_pMessageList)[m_selectedMsgNum];
		}
		else
		{
			m_firstMsgNum--;
			m_pFirstMessage = (*m_pMailbox->m_pMessageList)[m_firstMsgNum];
			m_pSelectedMessage = (*m_pMailbox->m_pMessageList)[m_firstMsgNum + m_selectedMsgNum];
		}
	}
	else
		m_pSelectedMessage = (*m_pMailbox->m_pMessageList)[m_firstMsgNum + m_selectedMsgNum];
	
	// Special case of deleting the first displayed message
	if (m_selectedMsgNum == 0)
		m_pFirstMessage = m_pSelectedMessage;
	
	RebuildDisplayedMessages();
}

void	CMessagesWindow::DeleteMarkedMessages()
{
	int oldSelectedIdx = m_firstMsgNum + m_selectedMsgNum;
	
	if (m_pMailbox->deleteMarkedMessages())
		return;
		
	// Let's re-adjust the first message displayed
	// Case 1: There are no messages left
	if (m_pMailbox->m_pMessageList->size() == 0)
	{
		m_firstMsgNum = 0;
		m_pFirstMessage = NULL;
	}
	
	// Case 2: There are 10 messages or less left
	else if (m_pMailbox->m_pMessageList->size() <= DISPLAYED_MSG_COUNT)
	{
		m_firstMsgNum = 0;
		m_pFirstMessage = (*m_pMailbox->m_pMessageList)[0];
	}
	
	// Case 3: There are more than 10 messages, but the first message number is
	// greater than the amount of messages left
	else if ((unsigned int)(m_firstMsgNum+DISPLAYED_MSG_COUNT) > m_pMailbox->m_pMessageList->size())
	{
		m_firstMsgNum = m_pMailbox->m_pMessageList->size() - DISPLAYED_MSG_COUNT;
		m_pFirstMessage = (*m_pMailbox->m_pMessageList)[m_firstMsgNum];
	}
	
	// Case 4: The general case
	else
	{
		m_pFirstMessage = (*m_pMailbox->m_pMessageList)[m_firstMsgNum];
	}


	// Now let's take care of the selected message
	if ((unsigned int)m_selectedMsgNum >= m_pMailbox->m_pMessageList->size())
	{
		m_selectedMsgNum = m_pMailbox->m_pMessageList->size() - 1;
		m_pSelectedMessage = (*m_pMailbox->m_pMessageList)[m_firstMsgNum+m_selectedMsgNum];
	}
	else if ((unsigned int)oldSelectedIdx >= m_pMailbox->m_pMessageList->size())
	{
		m_selectedMsgNum = m_pMailbox->m_pMessageList->size() - 1;
		m_pSelectedMessage = (*m_pMailbox->m_pMessageList)[m_firstMsgNum+m_selectedMsgNum];
	}
	else if (oldSelectedIdx > (m_firstMsgNum+DISPLAYED_MSG_COUNT-1))
	{
		m_selectedMsgNum = DISPLAYED_MSG_COUNT-1;
		m_pSelectedMessage = (*m_pMailbox->m_pMessageList)[m_firstMsgNum+m_selectedMsgNum];
	}
	else
	{
		m_selectedMsgNum = oldSelectedIdx - m_firstMsgNum;
		m_pSelectedMessage = (*m_pMailbox->m_pMessageList)[m_firstMsgNum+m_selectedMsgNum];
	}
	
	RebuildDisplayedMessages();
}

void CMessagesWindow::composeReplyAllFromSelected()
{
	CString tempText("");

	CControlAddress::AddressEntryList* to = new CControlAddress::AddressEntryList();
	to->push_back(new InputBoxAddressEntry(m_pSelectedMessage->from->getEmail(), m_pSelectedMessage->from->getEmail()));

	tempText.Printf("Re: %s", (const char*)m_pSelectedMessage->subject);
	CText* subject = new CText((const char *)tempText);

	CText* body = new CText("Original Message:\n");
	body->insert("-----------------------------------\n");
	body->insert(m_pMailbox->getBody(m_pSelectedMessage).GetStr());

	m_pComposeWindow = new CComposeWindow(m_pMailbox,
											to,
											NULL,
											NULL,
											subject,
											body);
}

void CMessagesWindow::composeReplyFromSelected()
{
	CString tempText("");

	CControlAddress::AddressEntryList* to = new CControlAddress::AddressEntryList();
	to->push_back(new InputBoxAddressEntry(m_pSelectedMessage->from->getEmail(), m_pSelectedMessage->from->getEmail()));

	tempText.Printf("Re: %s", (const char*)m_pSelectedMessage->subject);
	CText* subject = new CText((const char *)tempText);

	CText* body = new CText("Original Message:\n");
	body->insert("-----------------------------------\n");
	body->insert(m_pMailbox->getBody(m_pSelectedMessage).GetStr());

	m_pComposeWindow = new CComposeWindow(m_pMailbox,
											to,
											NULL,
											NULL,
											subject,
											body);
}

void CMessagesWindow::composeForwardFromSelected()
{
	CString tempText("");
	tempText.Printf("Fwd: %s", (const char*)m_pSelectedMessage->subject);
	CText* subject = new CText((const char *)tempText);

	CText* body = new CText("Forwarded Message:\n");
	body->insert("-----------------------------------\n");
	body->insert(m_pMailbox->getBody(m_pSelectedMessage).GetStr());

	m_pComposeWindow = new CComposeWindow(m_pMailbox,
											NULL,
											NULL,
											NULL,
											subject,
											body);
}

void	CMessagesWindow::RebuildDisplayedMessages()
{
	// We must rebuild the Display Message List
	int i = 0;
	CMailbox::TMessageList::iterator it3;
	
	if ((m_firstMsgNum + DISPLAYED_MSG_COUNT) > m_pMailbox->getTotalMessageCount())
	{
		m_firstMsgNum = 0;
		
		// Special Case: Deleteing one of last 10 messages in a large mailbox
		// Delete message and show last 10 items in mailbox
		if (m_pMailbox->getTotalMessageCount() > DISPLAYED_MSG_COUNT)
		{
			m_firstMsgNum = m_pMailbox->getTotalMessageCount() - 10;
		}

		i = 0;
		it3 = m_pMailbox->m_pMessageList->begin();
		while(it3 !=  m_pMailbox->m_pMessageList->end())
		{
		 	if (i == m_firstMsgNum)
			{
				m_pFirstMessage = (*it3);
				m_pDisplayedMessages->clear();
			}
			if (i >= m_firstMsgNum)
			{
				m_pDisplayedMessages->push_back((*it3));
			}
			
			// Make Sure the Displayed List is only 10 Messages Big
			if (i == (m_firstMsgNum + DISPLAYED_MSG_COUNT - 1))
			{
				break;
			}
			i++;
			it3++;
		}
		
		// Special Case: The message list was empty
		if (i == 0)
		{
			m_pDisplayedMessages->clear();
		}
		
		// Special Case: Deleteing one of last 10 messages in a large mailbox
		if (m_pMailbox->getTotalMessageCount() > DISPLAYED_MSG_COUNT)
		{
			if (m_selectedMsgNum != 9) m_selectedMsgNum++;
		}
	}
	else
	{
		// General Case: Standard rebuild of the Display Message List from First Message
	 	i = 0;
		it3 = m_pMailbox->m_pMessageList->begin();
		while(it3 !=  m_pMailbox->m_pMessageList->end())
		{
			if ((*it3) == m_pFirstMessage)
			{
				m_pDisplayedMessages->clear();
				i = 1;
			}
			if (i > 0)
			{
				m_pDisplayedMessages->push_back((*it3));
				i++;
			}
			if (i == (DISPLAYED_MSG_COUNT+1))
			{
				break;
			}
			it3++;
		}
	}

	UpdateDisplayListText();
	UpdateScrollBar();
}

void CMessagesWindow::BuildDisplayListFromScratch()
{
	int i = 0;
	CMailbox::TMessageList::iterator it = m_pMailbox->m_pMessageList->begin();
	
	m_pDisplayedMessages->clear();
	
	while(it !=  m_pMailbox->m_pMessageList->end() && i < DISPLAYED_MSG_COUNT) {
		if (i == 0)
		{
			m_pFirstMessage = (*it);
			m_pSelectedMessage = (*it);
			m_selectedMsgNum = 0;
			m_firstMsgNum = 0;  
		}
		m_pDisplayedMessages->push_back((*it));   
		i++;
		it++;
	}
	UpdateDisplayListText();		
	UpdateScrollBar();
}

void CMessagesWindow::UpdateDisplayListText()
{
	time_t tm = time(NULL);
	struct tm ltm;
	struct tm* msgtm;
	char msgDate[64];
	CString msgDateStr;
	int msgNum = 0;
	localtime_r(&tm, &ltm); 
	CMailbox::TMessageList::iterator it = m_pDisplayedMessages->begin();
	while( it !=  m_pDisplayedMessages->end() )
	{
		msgtm = localtime(&(*it)->date);
		if (ltm.tm_year == msgtm->tm_year && ltm.tm_mon == msgtm->tm_mon &&
		    ltm.tm_mday == msgtm->tm_mday)
		{
			strftime(msgDate, 64, "%H:%M", msgtm);
		}
		else
		{
			strftime(msgDate, 64, "%d/%m/%y %H:%M", msgtm);
		}

		msgDateStr = msgDate;   
		m_pMessageLabels[msgNum][SUBJECT_COL]->setText((*it)->subject);
		m_pMessageLabels[msgNum][FROM_COL]->setText((*it)->from->getEmail());
		m_pMessageLabels[msgNum][DATE_COL]->setText(msgDateStr);

		if (*it == m_pSelectedMessage) {
			m_pMessageLabels[msgNum][SUBJECT_COL]->scroll(true);
		} else {
			m_pMessageLabels[msgNum][SUBJECT_COL]->scroll(false);
		}
		it++;
		msgNum++;
	}	 	   
	UpdateScrollBar();
}

void CMessagesWindow::MessagesArrived()
{
	if (m_pSelectedMessage == NULL)
	{
		BuildDisplayListFromScratch();
	}
	else
	{
		int i = 0;
		CMailbox::TMessageList::iterator it = m_pMailbox->m_pMessageList->begin();
	
		while(it !=  m_pMailbox->m_pMessageList->end())
		{
		 	/* If we are at the top of the mail list and new messages arrive, 
		 	change the First Message to the new top message in the mail list */ 	 
		 	if (m_firstMsgNum == 0 && i == 0)
			{
				m_pFirstMessage	= (*it);
				break;
			}
			/* Otherwise just change First Message Number to its new numerical 
			position in the mail list */	  	 
			if ((*it) == m_pFirstMessage)
			{
				m_firstMsgNum = i;
				break;
			}
			it++;
			i++;
		}
		
		i = 0;
		it = m_pMailbox->m_pMessageList->begin();
		// Find where the Selected Message is numerically in the list and store it in i
		while(it !=  m_pMailbox->m_pMessageList->end())
		{	 
			if ((*it) == m_pSelectedMessage)
			{
				break;
			}
			it++;
			i++;
		}
		
		// Check if the Selected Message will be in the new Displayed List
		if (i >= m_firstMsgNum && i < (m_firstMsgNum + DISPLAYED_MSG_COUNT))
		{
			/* Selected Message is in the new displayed list. Find the Selected 
			Message new numerical position */
			m_selectedMsgNum = i - m_firstMsgNum;
			RebuildDisplayedMessages();	  
		}
		else
		{
			/* Selected Message is NOT in the new displayed list. Change displayed 
			list to include selected message at the top of the displayed list */
			m_firstMsgNum = i;
			m_pFirstMessage = m_pSelectedMessage;
			m_selectedMsgNum = 0;
			RebuildDisplayedMessages();
			/* If Selected Item ended up off the bottom of the list, Rebuild will
			make sure last 10 messages including Selected Message is in display list.
			However, we must manually set the Selected Message number to its new
			numerical value */
			i = 0;
			it = m_pDisplayedMessages->begin();
			// Find where the Selected Message is numerically in the displayed list and store it
			while(it !=  m_pDisplayedMessages->end())
			{	 
				if ((*it) == m_pSelectedMessage)
				{
					m_selectedMsgNum = i;
					break;
				}
				it++;
				i++;
			}						   	  
		}	
	}
}

void CMessagesWindow::SortChanged()
{
	int i = 0;
	CMailbox::TMessageList::iterator it = m_pMailbox->m_pMessageList->begin();
	
	// Find where the Selected Message is numerically in the list and store it in i
	while(it !=  m_pMailbox->m_pMessageList->end())
	{	 
		if ((*it) == m_pSelectedMessage)
		{
			break;
		}
		it++;
		i++;
	}
	
	/* Change displayed list to include selected message at the top of the 
	displayed list */
	m_firstMsgNum = i;
	m_pFirstMessage = m_pSelectedMessage;
	m_selectedMsgNum = 0;
	RebuildDisplayedMessages();
	/* If Selected Item ended up off the bottom of the list, Rebuild will
	make sure last 10 messages including Selected Message is in display list.
	However, we must manually set the Selected Message number to its new
	numerical value */
	i = 0;
	it = m_pDisplayedMessages->begin();
	// Find where the Selected Message is numerically in the displayed list and store it
	while(it !=  m_pDisplayedMessages->end())
	{	 
		if ((*it) == m_pSelectedMessage)
		{
			m_selectedMsgNum = i;
			break;
		}
		it++;
		i++;
	}						   	  
}

void CMessagesWindow::SortBySettings()
{
 	int style = SORT_DATE_ASC; 
	if (CConfigFile::s_pSettings->message_sort_style == "DATE_ASC") style = SORT_DATE_ASC;
	if (CConfigFile::s_pSettings->message_sort_style == "DATE_DESC") style = SORT_DATE_DESC;
	if (CConfigFile::s_pSettings->message_sort_style == "SUBJECT_ASC") style = SORT_SUBJECT_ASC;
	if (CConfigFile::s_pSettings->message_sort_style == "SUBJECT_DESC") style = SORT_SUBJECT_DESC;
	if (CConfigFile::s_pSettings->message_sort_style == "FROM_ASC") style = SORT_FROM_ASC;
	if (CConfigFile::s_pSettings->message_sort_style == "FROM_DESC") style = SORT_FROM_DESC;
	switch(style)
	{
		case SORT_DATE_ASC:
			m_pMailbox->setSort(new CMailboxSortDate);
			SortChanged();
			break;
		case SORT_DATE_DESC:
			m_pMailbox->setSort(new CMailboxDescSort(new CMailboxSortDate));
			SortChanged();
			break;
		case SORT_SUBJECT_ASC:
			m_pMailbox->setSort(new CMailboxSortSubject);
			SortChanged();
			break;
		case SORT_SUBJECT_DESC:
			m_pMailbox->setSort(new CMailboxDescSort(new CMailboxSortSubject));
			SortChanged();
			break;
		case SORT_FROM_ASC:
			m_pMailbox->setSort(new CMailboxSortFrom);
			SortChanged();
			break;
		case SORT_FROM_DESC:
			m_pMailbox->setSort(new CMailboxDescSort(new CMailboxSortFrom));
			SortChanged();
			break;
	}
}

void CMessagesWindow::InitProgressBar()
{
	m_pProgressBar->show();
}

void CMessagesWindow::ClearProgressBar()
{
	m_pProgressBar->reset();
	m_pProgressBar->hide();
}

void CMessagesWindow::retrievalStarted(CMailbox* aMailbox, u32 aTotal)
{
	m_pProgressBar->startNextPhase(CString("Pre-fetching"), aTotal, CControlProgressBar::FORMAT_NUMBER);
}

void CMessagesWindow::retrievedOne(CMailbox* aMailbox, u32 aIndex, u32 aTotal)
{
	m_pProgressBar->update(1);
}

void CMessagesWindow::retrievedNone(CMailbox* aMailbox)
{
	ClearProgressBar();
}

void CMessagesWindow::retrieveFailed(CMailbox* aMailbox, CString aErrorText)
{
	ClearProgressBar();
	if (m_pErrorDlg == NULL)
	{
		if (m_pCContextMenu)
			SAFE_DELETE(m_pCContextMenu);
			
		CString dlgText;
		dlgText.Printf("Message retrieve failed. %s", aErrorText.GetPtr());
		m_pErrorDlg = new CDialogBoxInput(250, V2(-1, -1), dlgText, false, false);
	}
}

void CMessagesWindow::storingStarted(CMailbox* aMailbox, u32 aTotal)
{
	m_pProgressBar->startNextPhase(CString("Retrieving & storing"), aTotal, CControlProgressBar::FORMAT_NUMBER);
}

void CMessagesWindow::storedOne(CMailbox* aMailbox, u32 aIndex, u32 aTotal)
{
	m_pProgressBar->update(1);
}

void CMessagesWindow::storingDone(CMailbox* aMailbox, u32 aTotal)
{
	MessagesArrived();
	ClearProgressBar();
}

void CMessagesWindow::storeFailed(CMailbox* aMailbox, CString aErrorText)
{
	ClearProgressBar();
	if (m_pErrorDlg == NULL)
	{
		if (m_pCContextMenu)
			SAFE_DELETE(m_pCContextMenu);
			
		CString dlgText;
		dlgText.Printf("Error storing mail. %s", aErrorText.GetPtr());
		m_pErrorDlg = new CDialogBoxInput(250, V2(-1, -1), dlgText, false, false);
	}
}

void CMessagesWindow::UpdateScrollBar() 
{
	m_pScrollBar->update(m_firstMsgNum, m_pMailbox->getTotalMessageCount());
}

//*******************************  END OF FILE  ********************************
