/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CPSprintGame.cpp

  Description :	

  Last Modified $Date: 2008/09/28 01:02:03 $

  $Revision: 1.4 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CPSprintGame.h"
#include "CFrameWork.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CInput.h"
#include "CSkinManager.h"
#include "CGfx.h"
#include <psprtc.h>
#include "CWindow.h"
#include "CPSPoste.h"
#include "CContextMenu.h"
#include "CContextMenuItem.h"
#include "CHUHelp.h"
#include "p_sprint.h"
#include "irda_KeyboardLayouts.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
const float HEADER_DIVISION(20.f);

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//******************************************************************************
//
//	Main Window implementation
//
//******************************************************************************
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CPSprintGame::CPSprintGame():
	m_usedChars(0),
	m_pointTotal(0),
	m_levelPoints(0),
	m_misses(0),
	m_pCurrentLevel(NULL),
	m_pLevels(NULL),
	m_pFallingChars(NULL),
	m_state(STATE_NEWGAME)
{
	SetFocus(true);

	// Load levels (make fake one here)
	Level* newLevel = new Level("Level 1", 10, 2000, 10, 10, 5);
	GameChar* newGameChar1 = new GameChar('a', 1);
	GameChar* newGameChar2 = new GameChar('b', 1);
	GameChar* newGameChar3 = new GameChar('c', 1);
	newLevel->addChar(newGameChar1);
	newLevel->addChar(newGameChar2);
	newLevel->addChar(newGameChar3);

	Level* level2 = new Level("Level 2", 10, 2000, 10, 10, 16);
	GameChar* newGameChar5 = new GameChar('d', 1);
	GameChar* newGameChar6 = new GameChar('e', 2);
	GameChar* newGameChar7 = new GameChar('f', 1);
	level2->addChar(newGameChar5);
	level2->addChar(newGameChar6);
	level2->addChar(newGameChar7);

	// Intialize lists
	m_pLevels = new LevelList();

	m_pLevels->push_back(newLevel);
	m_pLevels->push_back(level2);

}

//******************************************************************************
//	
//******************************************************************************
CPSprintGame::~CPSprintGame()
{

}

//******************************************************************************
//	
//******************************************************************************
void	CPSprintGame::Render()
{
	// Draw white background
	CGfx::DrawQuad( V2( 0, 0 ), V2( CGfx::s_ScreenWidth, CGfx::s_ScreenHeight), CSkinManager::GetColor( "main_window", "background_color", 0xFFFFFFFF));

	switch(m_state) {
		case STATE_PLAYING:
		{
			// Print the falling chars
			FallingCharList::iterator it = m_pFallingChars->begin();
			while (it != m_pFallingChars->end())
			{
				CString charText("");
				charText.Printf("%c", (*it)->getChar());
				CFont::GetActiveUiFont()->Print(charText, (*it)->m_position, CSkinManager::GetColor( "address_box", "header_text_color", 0xFFFFFFFF));
				it++;
			}

			// Draw white background
			CGfx::DrawQuad( V2( 0, 0 ), V2( CGfx::s_ScreenWidth, HEADER_DIVISION), CSkinManager::GetColor( "main_window", "background_color", 0xFFFFFFFF));

			CFont::GetActiveUiFont()->Print(m_pCurrentLevel->m_name, V2(5,6), CSkinManager::GetColor( "address_box", "header_text_color", 0xFFFFFFFF));
			
			CString pointTotal("");
			pointTotal.Printf("Score: %d", m_pointTotal);
			CFont::GetActiveUiFont()->Print(pointTotal, V2(100, 6), CSkinManager::GetColor( "address_box", "header_text_color", 0xFFFFFFFF));
			
			CString pointsToNextLevel("");
			pointsToNextLevel.Printf("Next Level: %d", m_pCurrentLevel->m_pointsRequired - m_levelPoints);
			CFont::GetActiveUiFont()->Print(pointsToNextLevel, V2(250, 6), CSkinManager::GetColor( "address_box", "header_text_color", 0xFFFFFFFF));
			
			CString missesLeft("");
			missesLeft.Printf("Misses Left: %d", m_pCurrentLevel->m_allowedMisses - m_misses);
			CFont::GetActiveUiFont()->Print(missesLeft, V2(350, 6), CSkinManager::GetColor( "address_box", "header_text_color", 0xFFFFFFFF));
			
			// Draw black line
			CGfx::DrawQuad( V2( 0, HEADER_DIVISION ), V2( CGfx::s_ScreenWidth, 1),  CSkinManager::GetColor( "main_window", "title_bar_outline_color", 0xFF000000));
			break;	
		}
		case STATE_GAMEOVER:
		{
			CString gameOver("Game Over!!!");
			CFont::GetActiveUiFont()->Print(gameOver, V2(CGfx::s_ScreenWidth/2 - CFont::GetActiveUiFont()->GetStringSize(gameOver).x/2, 10), CSkinManager::GetColor( "address_box", "header_text_color", 0xFFFFFFFF));
			CString youScoreGameOver("");
			youScoreGameOver.Printf("Your score was: %d", m_pointTotal);
			CFont::GetActiveUiFont()->Print(youScoreGameOver, V2(CGfx::s_ScreenWidth/2 - CFont::GetActiveUiFont()->GetStringSize(youScoreGameOver).x/2, 25), CSkinManager::GetColor( "address_box", "header_text_color", 0xFFFFFFFF));
			CString controlsGameOver("Press Cross to restart, Triangle to return to main menu");
			CFont::GetActiveUiFont()->Print(controlsGameOver, V2(CGfx::s_ScreenWidth/2 - CFont::GetActiveUiFont()->GetStringSize(controlsGameOver).x/2, 40), CSkinManager::GetColor( "address_box", "header_text_color", 0xFFFFFFFF));
			break;
		}
		case STATE_COMPLETE:
		{
			CString youWin("Congratulation you won!");
			CFont::GetActiveUiFont()->Print(youWin, V2(CGfx::s_ScreenWidth/2 - CFont::GetActiveUiFont()->GetStringSize(youWin).x/2, 10), CSkinManager::GetColor( "address_box", "header_text_color", 0xFFFFFFFF));
			CString youScoreComplete("");
			youScoreComplete.Printf("Your score was: %d", m_pointTotal);
			CFont::GetActiveUiFont()->Print(youScoreComplete, V2(CGfx::s_ScreenWidth/2 - CFont::GetActiveUiFont()->GetStringSize(youScoreComplete).x/2, 25), CSkinManager::GetColor( "address_box", "header_text_color", 0xFFFFFFFF));
			CString controlsComplete("Press Cross to restart, Triangle to return to main menu");
			CFont::GetActiveUiFont()->Print(controlsComplete, V2(CGfx::s_ScreenWidth/2 - CFont::GetActiveUiFont()->GetStringSize(controlsComplete).x/2, 40), CSkinManager::GetColor( "address_box", "header_text_color", 0xFFFFFFFF));
			break;
		}
		case STATE_NEWGAME:
		case STATE_NEXTLEVEL:
		default:
		{
			break;
	
		}
	}
}

//******************************************************************************
//	
//******************************************************************************
void	CPSprintGame::Process()
{
	FallingCharList::iterator it;
	LevelList::iterator itLevel;

	switch(m_state) {
		case STATE_NEWGAME:
		{
			m_pointTotal = 0;
			m_levelPoints = 0;
			m_misses = 0;

			//Set Timing
			m_lastTimeDrop = CFrameWork::GetTicksMs();
			m_lastTimeChar = CFrameWork::GetTicksMs();
			m_pFallingChars = new FallingCharList();
			m_pCurrentLevel = *m_pLevels->begin();

			m_state = STATE_PLAYING;
			break;
		}
		case STATE_PLAYING:
		{
			//Get timing info
			u64 thisTicks = CFrameWork::GetTicksMs();
			u64 l_curDrop = thisTicks - m_lastTimeDrop;
			u64 l_curChar = thisTicks - m_lastTimeChar;

			// Drop characters
			if (l_curDrop > m_pCurrentLevel->m_speed) {
				if (m_pFallingChars->size() > 0) {
					it = m_pFallingChars->begin();
					while (it != m_pFallingChars->end()) {
						(*it)->m_position.y += l_curDrop / m_pCurrentLevel->m_speed;
						it++;
					}

					//Check if char fell off the screen, note can only be one off screen at a time,
					//so only check the begin char
					it = m_pFallingChars->begin();
					if ((*it)->m_position.y > CGfx::s_ScreenHeight) {
						m_misses++;
						m_pFallingChars->erase(it);
					}
				}
				m_lastTimeDrop = thisTicks;
			}

			//Add new character
			if (l_curChar > m_pCurrentLevel->m_chargap) {
				//pick the character
				int n = rand() % m_pCurrentLevel->m_pChars->size(); 
				GameChar* newCharacter = m_pCurrentLevel->m_pChars->at(n);

				//pick an x coordinate and calculate y
				n = rand() % (int)CGfx::s_ScreenWidth;
				CString character("");
				character.Printf("%c", newCharacter->m_char);
				V2 position(n, HEADER_DIVISION - CFont::GetActiveUiFont()->GetStringSize(character).y);

				//make the falling char and add to the list
				FallingGameChar* newFallingChar = new FallingGameChar(newCharacter->m_char, newCharacter->m_pointValue, position);
				m_pFallingChars->push_back(newFallingChar);
				
				m_lastTimeChar = thisTicks;
			}
			if ((m_pCurrentLevel->m_allowedMisses - m_misses) <= 0) {
				m_state = STATE_GAMEOVER;
			}
			if (m_levelPoints >= m_pCurrentLevel->m_pointsRequired) {
				m_state = STATE_NEXTLEVEL;
			}
			break;
		}
		case STATE_GAMEOVER:
		{
			break;
		}
		case STATE_COMPLETE:
		{
			break;
		}
		case STATE_NEXTLEVEL:
		{
			itLevel = m_pLevels->begin();
			while (itLevel != m_pLevels->end()) {
				if (m_pCurrentLevel == *itLevel) {
					itLevel++;
					if (itLevel != m_pLevels->end()) {
						m_levelPoints = 0;
						m_misses = 0;
						m_pCurrentLevel = *itLevel;
						m_state = STATE_PLAYING;
					} else {
						m_state = STATE_COMPLETE;
					}
					break;
				}
				itLevel++;
			}
			break;
		}
		default:
		{
			break;
		}
	}
}

//******************************************************************************
//	
//******************************************************************************
bool	CPSprintGame::ProcessInput()
{
	switch(m_state) {
		case STATE_PLAYING: 
		{
			if (CInput::IsKeyboardPressed() == true ) {
   	  			char pressedKey = CInput::GetKeyboardCharKey();						   
				
				//Remove the item if it is currently falling
				FallingCharList::iterator it = m_pFallingChars->begin();
				while (it != m_pFallingChars->end()) {
					if ((*it)->getChar() == pressedKey) {
						m_pointTotal += (*it)->getPointValue();
						m_levelPoints += (*it)->getPointValue();
						m_pFallingChars->erase(it);
						break;
					}
					it++;
				}
			}	
			break;
		}
		case STATE_GAMEOVER:
		case STATE_COMPLETE:
		{
			if (CInput::IsButtonClicked(CInput::TRIANGLE)) {
				CPSPoste::s_NewWindow = WINDOW_MAIN;
			}
			if (CInput::IsButtonClicked(CInput::CROSS)) {
				m_state = STATE_NEWGAME;
			}
			break;
		}
		case STATE_NEWGAME:
		case STATE_NEXTLEVEL:
		default:
		{
			break;
		}
	}
	return false;
}
//*******************************  END OF FILE  ********************************
