/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CPSprintGame.h

  Description :	

  Last Modified 

*******************************************************************************/

#ifndef CPSprintGame_H_
#define CPSprintGame_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CWindow.h"
#include "CString.h"
#include "CGfx.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************
class Level;
class GameChar;
class FallingGameChar;
typedef std::vector<Level*> LevelList;
typedef std::vector<GameChar*> GameCharList;
typedef std::vector<FallingGameChar*> FallingCharList;

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************
class CPSprintGame : public CWindow
{

	enum eState
	{
		STATE_NEWGAME,
		STATE_PLAYING,
		STATE_NEXTLEVEL,
		STATE_GAMEOVER,
		STATE_COMPLETE
	};

	public:
		CPSprintGame();
		~CPSprintGame();

		virtual void			Render();
		virtual void			Process();
		virtual bool			ProcessInput();

	//timing vars
	private:
		u64 m_lastTimeDrop;
		u64 m_lastTimeChar;

	private:
		int m_usedChars;
		int m_pointTotal;
		int m_levelPoints;
		int m_misses;
		Level* m_pCurrentLevel;
		LevelList* m_pLevels;
		FallingCharList* m_pFallingChars;
		eState m_state;
};

class Level : public CBase
{
	public:
		Level(CString name, u32 speed, u32 charGap, int numberOfChars, int allowedMisses, int pointsRequired) :
			m_name(name),
			m_speed(speed),
			m_chargap(charGap),
			m_numberOfChars(numberOfChars),
			m_allowedMisses(allowedMisses),
			m_pointsRequired(pointsRequired),
			m_pChars(NULL)
		{
			m_pChars = new GameCharList();
		}

		void addChar(GameChar* gameChar)
		{
			m_pChars->push_back(gameChar);
		}

	public:
		CString m_name;
		u32 m_speed;
		u32 m_chargap;
		int m_numberOfChars;
		int m_allowedMisses;
		int m_pointsRequired;
		GameCharList* m_pChars;
};

class GameChar : public CBase
{
	public:

		GameChar(char charVal, int pointValue):
			m_char(charVal),
			m_pointValue(pointValue)
		{
		}

	public:
		char m_char;
		int m_pointValue;
};

class FallingGameChar : public GameChar
{
	public:
		FallingGameChar(char charVal, int pointValue, V2 position) :
			GameChar(charVal, pointValue),
			m_position(position)
		{
		}

	public:
		V2 m_position;
		char getChar() {return m_char;}
		char getPointValue() {return m_pointValue;}
};


//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CPSprintGame_H_ */
