/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CPspAnalogRepeat.cpp

  Description :	

  Last Modified $Date: 2008/09/28 01:02:03 $

  $Revision: 1.3 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CPspAnalogRepeat.h"
#include "CInput.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************
CPspAnalogRepeat::~CPspAnalogRepeat()
{
	
}

//******************************************************************************
//
//******************************************************************************
void CPspAnalogRepeat::process()
{
	// Sample the analog stick
	V2 inputSample = CInput::GetAnalogStick();
	
	// Sample the current tick
	m_sampleTick = CFrameWork::GetTicks();
	
	// Calculate the scale of the X & Y axis in %
	float inputScaleX = getScale(absFloat(inputSample.x));
	float inputScaleY = getScale(absFloat(inputSample.y));
	
	m_repeatDirections = 0;
	
	if (inputScaleX)
	{
		// Get the rate in ticks that would trigger a repeat
		u32 repeatRateX = getRepeatRate(inputScaleX);
		
		// Determine if the X axis should trigger events
		if (m_sampleTick >= (m_lastRepeatX + repeatRateX))
		{
			// X should repeat
			m_lastRepeatX = m_sampleTick;
			if (inputSample.x > 0)
				m_repeatDirections |= CPspAnalogRepeat::DIRECTION_RIGHT;
			else
				m_repeatDirections |= CPspAnalogRepeat::DIRECTION_LEFT;
		}
	}
		
	if (inputScaleY)
	{
		// Get the rate in ticks that would trigger a repeat
		u32 repeatRateY = getRepeatRate(inputScaleY);
		
		// Determine if the Y axis should trigger events
		if (m_sampleTick >= (m_lastRepeatY + repeatRateY))
		{
			// Y should repeat
			m_lastRepeatY = m_sampleTick;
			if (inputSample.y > 0)
				m_repeatDirections |= CPspAnalogRepeat::DIRECTION_DOWN;
			else
				m_repeatDirections |= CPspAnalogRepeat::DIRECTION_UP;
		}
	}
}

u32 CPspAnalogRepeat::getRepeatDirections()
{
	return m_repeatDirections;
}

float CPspAnalogRepeat::getScale(const float aAbsFloat)
{
	if (aAbsFloat < m_minInput)
		return 0.0;
	else if (aAbsFloat > m_maxInput)
		return 1.0;
	else
		return (aAbsFloat - m_minInput) / (m_inputInterval);
}
