/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CPspAnalogRepeat.h

  Description :	

  Last Modified $Date: 2008/09/29 04:22:27 $

  $Revision: 1.6 $

*******************************************************************************/

#ifndef CPSPANALOGREPEAT_H_
#define CPSPANALOGREPEAT_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CFrameWork.h"
#include <psptypes.h>
#include <pspctrl.h>
#include <time.h>
#include <psprtc.h>

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************

//******************************************************************************
//   Constants
//******************************************************************************
#define DEFAULT_REPEAT_SLOWEST_TICKS 600000L
#define DEFAULT_REPEAT_FASTEST_TICKS 60000L
#define DEFAULT_MIN_INPUT 0.15
#define DEFAULT_MAX_INPUT 0.85

//******************************************************************************
//   Class definitions
//******************************************************************************
class CPspAnalogRepeat : public CBase
{
	public: /* Types */
	
		enum eAnalogDirection
		{
			DIRECTION_UP = 1,
			DIRECTION_DOWN = 2,
			DIRECTION_LEFT = 4,
			DIRECTION_RIGHT = 8
		};
	
	public: /* Constructors & destructors */
	
		CPspAnalogRepeat(const u32 aDirections = DIRECTION_UP | DIRECTION_DOWN | DIRECTION_LEFT | DIRECTION_RIGHT,
		                 const u64 aSlowestRepeat = DEFAULT_REPEAT_SLOWEST_TICKS,
		                 const u64 aFastestRepeat = DEFAULT_REPEAT_FASTEST_TICKS,
		                 const float aMinInput = DEFAULT_MIN_INPUT,
		                 const float aMaxInput = DEFAULT_MAX_INPUT) :
			m_directions(aDirections),
			m_slowestRepeat(aSlowestRepeat),
			m_fastestRepeat(aFastestRepeat),
			m_repeatInterval((u32)(aSlowestRepeat - aFastestRepeat)),
			m_minInput(aMinInput),
			m_maxInput(aMaxInput),
			m_inputInterval(aMaxInput - aMinInput),
			m_sampleTick(0),
			m_lastRepeatX(0),
			m_lastRepeatY(0),
			m_repeatDirections(0)
		{
			reset();
		};
		~CPspAnalogRepeat();
		
	public:
	
		void process();
		void reset() { m_sampleTick = CFrameWork::GetTicks(); m_lastRepeatX = m_sampleTick; m_lastRepeatY = m_sampleTick; m_repeatDirections = 0; };
		u32 getRepeatDirections();
		
	private: /* Helper functions */
	
		float absFloat(const float aVal)
		{
			if (aVal < 0)
				return -aVal;
			return aVal;
		};
		
		u32 getRepeatRate(const float aScale)
		{
			return (u32)(m_slowestRepeat - (m_repeatInterval * aScale));
		};
		
		// aAbsFloat MUST be > 0
		float getScale(const float aAbsFloat);
		
	private:
	
		const u32 m_directions;
		const u64 m_slowestRepeat;
		const u64 m_fastestRepeat;
		const u32 m_repeatInterval;
		const float m_minInput;
		const float m_maxInput;
		const float m_inputInterval;
		
		u64 m_sampleTick;
		u64 m_lastRepeatX;
		u64 m_lastRepeatY;
		u32 m_repeatDirections;
	
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CPSPANALOGREPEAT_H_ */
