/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CPspPadRepeat.cpp

  Description :	

  Last Modified $Date: 2008/09/28 01:02:03 $

  $Revision: 1.3 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CPspPadRepeat.h"
#include "CFrameWork.h"
#include <time.h>
#include <psprtc.h>

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************
CPspPadRepeat::~CPspPadRepeat()
{
	
}

//******************************************************************************
//
//******************************************************************************
void CPspPadRepeat::process(SceCtrlData *aPadData)
{
	// If any button is newly pressed, generate an event
	u32 newButtonsDown = (~m_buttonsDown & aPadData->Buttons);
	m_buttonsDown = m_buttons & aPadData->Buttons;
	
	// If any of the buttons to watch are down, we must determine if an event
	// should be generated for this process.
	if (m_buttonsDown)
	{
		// If the sample tick is 0, means the previous call to process had no
		// buttons down
		if (m_sampleTick == 0)
			m_firstDown = true;
		else
			m_firstDown = false;
		
		m_sampleTick = CFrameWork::GetTicks();
		
		// If the buttons have just been pressed, we should generate an event
		if (m_firstDown)
		{
			sceRtcTickAddTicks(&m_nextRepeat, &m_sampleTick, m_slowRepeat);
			m_repeatButtons = m_buttonsDown;
		}
		// If the sample is larger than the next repeat, generate an event
		else if (sceRtcCompareTick(&m_sampleTick, &m_nextRepeat) >= 0)
		{
			sceRtcTickAddTicks(&m_nextRepeat, &m_sampleTick, m_fastRepeat);
			m_repeatButtons = m_buttonsDown;
		}
		else
		{
			m_repeatButtons = 0;
		}
	}
	else
	{
		m_sampleTick = 0;
		m_repeatButtons = 0;
	}
	
	m_repeatButtons |= newButtonsDown;
}

u32 CPspPadRepeat::getRepeatButtons()
{
	return m_repeatButtons;
}
