/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CPspPadRepeat.h

  Description :	

  Last Modified $Date: 2008/09/29 04:22:27 $

  $Revision: 1.2 $

*******************************************************************************/

#ifndef CPSPPADREPEAT_H_
#define CPSPPADREPEAT_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include <psptypes.h>
#include <pspctrl.h>
#include "CBase.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************

//******************************************************************************
//   Constants
//******************************************************************************
#define DEFAULT_REPEAT_SLOW_TICKS 300000
#define DEFAULT_REPEAT_FAST_TICKS 60000

//******************************************************************************
//   Class definitions
//******************************************************************************
class CPspPadRepeat : public CBase
{
	public: /* Constructors & destructors */
	
		CPspPadRepeat(const u32 aButtons,
		              const u64 aSlowRepeat = DEFAULT_REPEAT_SLOW_TICKS,
		              const u64 aFastRepeat = DEFAULT_REPEAT_FAST_TICKS) :
			m_buttons(aButtons),
			m_slowRepeat(aSlowRepeat),
			m_fastRepeat(aFastRepeat),
			m_sampleTick(0),
			m_nextRepeat(0),
			m_buttonsDown(0),
			m_repeatButtons(0),
			m_firstDown(false)
		{};
		~CPspPadRepeat();
		
	public:
	
		void process(SceCtrlData *aPadData);
		void reset() { m_sampleTick = 0; m_nextRepeat = 0; m_buttonsDown = 0; m_repeatButtons = 0; m_firstDown = 0; };
		u32 getRepeatButtons();
		
	private:
	
		const u32 m_buttons;
		const u64 m_slowRepeat;
		const u64 m_fastRepeat;
		
		u64 m_sampleTick;
		u64 m_nextRepeat;
		u32 m_buttonsDown;
		u32 m_repeatButtons;
		
		bool m_firstDown;
	
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CPSPPADREPEAT_H_ */
