/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CRandomBin.cpp

  Description :	

  Last Modified $Date: 2008/09/27 22:08:56 $

  $Revision: 1.1 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CRandomBin.h"
#include "CFileSystem.h"
#include "CString.h"
#include "CTypes.h"

#include <pspctrl.h>
#include <pspdebug.h>
#include <pspiofilemgr.h>
#include <pspiofilemgr_stat.h>

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const CString s_szRandomBinFile("Data/random.bin");

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
bool CRandomBin::Open()
{
	if (ExistsAndAppearValid())
	{
		return true;
	}

	return CreateRandomBinFile();
}

//******************************************************************************
//	
//******************************************************************************
bool CRandomBin::ExistsAndAppearValid()
{
	if (CFileSystem::FileExists(s_szRandomBinFile))
	{
		SceIoStat fileStat;
		sceIoGetstat(s_szRandomBinFile, &fileStat);
		
		// 120 = 60 * sizeof(u16)
		if (fileStat.st_size == 120)
		{
			return true;
		}
	}
	
	return false;
}

//******************************************************************************
//	
//******************************************************************************
bool CRandomBin::CreateRandomBinFile()
{
	u16 buf[60];
	u32 prevClock = clock();
	u32 newClock = prevClock;
	u16 value = 0;
	u32 length = sizeof(buf) / sizeof(u16);
	
	// This color is the light grey color taken from the Default Skin.xml file
	pspDebugScreenSetBackColor(0xff4d4d4d);
	pspDebugScreenClear();
	pspDebugScreenPrintf("For security purposes, you must generate a random seed file.\n");
	pspDebugScreenPrintf("This only needs to happen once. Press any button to proceed.\n");
	for (u32 i = 0; i < length; i++)
	{
		pspDebugScreenSetXY(0, 3);
		pspDebugScreenPrintf("%02d of %02d button presses left.\n", (int)(length - i), (int)length);
		pspDebugScreenPrintf("Last value: %04x\n", value);
		WaitForButtonPress();
	    newClock  = clock();
	    //LUDO: we take only the 16 less significant bits
	    value = ((newClock - prevClock) & 0xffff);
	    prevClock = newClock;
	    buf[i] = value;
	    WaitForButtonClear();
	}
	
	pspDebugScreenSetBackColor(0xff000000);
	pspDebugScreenClear();
	
	return WriteBufToFile(buf, length);
}

//******************************************************************************
//	
//******************************************************************************
void CRandomBin::WaitForButtonPress()
{
	SceCtrlData c;
	
	do {
		sceCtrlReadBufferPositive(&c, 1);
		c.Buttons &= 0xffff;
	} while (c.Buttons == 0);
}

//******************************************************************************
//	
//******************************************************************************
void CRandomBin::WaitForButtonClear()
{
	SceCtrlData c;

	do {
		sceCtrlPeekBufferPositive(&c, 1);
	} while (c.Buttons != 0); 
}

//******************************************************************************
//	
//******************************************************************************
bool CRandomBin::WriteBufToFile(u16* buf, u32 length)
{
	CFile* file = CFileSystem::Open(s_szRandomBinFile, "w");
	if (file)
	{
		u8* byteBuf = (u8*)buf;
		u32 byteLength = length*sizeof(u16);
		u32 bytesLeft = byteLength;
		u32 bytesWrote = 0;
		
		while (bytesLeft != 0)
		{
			file->Write(byteBuf+(byteLength-bytesLeft), bytesLeft, &bytesWrote);
			bytesLeft -= bytesWrote;
			
			if (bytesWrote == 0)
			{
				CFileSystem::Close(file);
				return false;
			}
		}
		
		CFileSystem::Close(file);
		return true;
	}
	
	return false;
}

//*******************************  END OF FILE  ********************************
