/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CSMTPSender.h

  Description :	

*******************************************************************************/

#ifndef CSMTPSENDER_H_
#define CSMTPSENDER_H_

#include "CMailSender.h"
#include "CString.h"
#include "CConfigFile.h"
#include "VMimeTypes.h"
#include "vmime/utility/progressListener.hpp"

class CSMTPSender : public CMailSender//, public vmime::utility::progressListener
{
	public: // Constructors & Destructors
		CSMTPSender(CAccount* aAccount,
		               SceUID aMailboxMbx, SceUID aSenderMbx, vmime::ref<CCertificateManager> aCertManager);
		virtual ~CSMTPSender();
		
	public: // Inherited from CMailSender
		virtual void send(CMailbox::TSendMsgList* msgList);
		virtual void certDeclined();
		virtual void certAccepted();
		
	/*public: // Inherited from vmime::utility::progressListener
		const bool cancel() const;
		void start(const int predictedTotal);
		void progress(const int current, const int currentTotal);
		void stop(const int total);*/
		
	private: // New methods		
		void connect();
		void disconnect();
		
		void successfulMessageSend();
		void failed(CMailbox::TSendMsgList* msgList, CString aErrorText);
		void disconnected();
		
	private: // Data members
		CAccount m_account;
		SceUID m_mailbox_mbx;
		SceUID m_sender_mbx;
		
		/* List of messages that have been successfully sent */
		CMailbox::TSendMsgList* m_pSentList;

		/* Total number of messages to send */
		u32 m_total;

		/* Total number of messages sent */
		u32 m_sent;

		/* The list of messages to be sent if the pending cert is accepted */
		CMailbox::TSendMsgList* m_listAwaitingCert;

		VSessionRef m_sess_ref;
		vmime::ref<vmime::net::transport> m_tr_ref;
};

#endif /*CSMTPSENDER_H_*/
