/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CScrollableText.h

  Description :	

*******************************************************************************/

#ifndef CSCROLLABLETEXT_H_
#define CSCROLLABLETEXT_H_

#include <CText.h>
#include <CTextIterator.h>
#include <CTextLineBuffer.h>

#include "CVector.h"
#include "CTypes.h"

#define YES 1
#define NO 0
#define UNKNOWN -1

class CScrollableText : public CBase
{
	public: // Constructors & destructors
		CScrollableText(CText* aText, const V2 & size, const V2 & pos, ARGB aColor, bool aEditable = false);
		virtual ~CScrollableText();
		
	public: // CTextLineBuffer wrappers
		void insert(const std::string aStr)
		{
			if (m_editable)
				m_lineBuf->insert(aStr);
		};
		void insert(const ct_char_t* aStr)
		{
			if (m_editable)
				m_lineBuf->insert(aStr);
		};
		void insert(const ct_char_t aChar)
		{
			if (m_editable)
				m_lineBuf->insert(aChar);
		};
		void del(size_t aCount = 1)
		{
			if (m_editable)
				m_lineBuf->del(aCount);
		};
		void backspace(size_t aCount = 1)
		{
			if (m_editable)
				m_lineBuf->backspace(aCount);
		};
		void incCursor(size_t aCount = 1)
		{
			if (m_editable)
				m_lineBuf->incCursor(aCount);
		};
		void decCursor(size_t aCount = 1)
		{
			if (m_editable)
				m_lineBuf->decCursor(aCount);
		};
		void upCursor()
		{
			if (m_editable)
				m_lineBuf->upCursor();
		};
		void downCursor()
		{
			if (m_editable)
				m_lineBuf->downCursor();
		};
		void home()
		{
			if (m_editable)
				m_lineBuf->home();
		};
		void end()
		{
			if (m_editable)
				m_lineBuf->end();
		};
		void pageUp()
		{
			m_lineBuf->pageUp();
		};
		void pageDown()
		{
			m_lineBuf->pageDown();
		};
		
	public: // New methods
		void render(bool aShowCursor = true);
		void renderCursor();
		void down();
		void up();
		
		/* Dumps debug info to psp_trace output */
		void dumpDebug();
		
	private:
		// Information about the CText
		CText* m_text;
		CTextLineBuffer* m_lineBuf;
		const V2 m_size;
		const V2 m_pos;
		ARGB m_color;
		bool m_editable;
		
};

#endif /*CSCROLLABLETEXT_H_*/
