/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CSettingsWindow.cpp

  Description :	

  Last Modified $Date: 2008/10/01 03:04:29 $

  $Revision: 1.21 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CSettingsWindow.h"
#include "CFrameWork.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CInput.h"
#include "CSkinManager.h"
#include "CGfx.h"
#include "CWindow.h"
#include "CPSPoste.h"
#include "CHUHelp.h"
#include "CConfigFile.h"
#include "CConnectionManager.h"
#include "CControlLabel.h"
#include "CConfigFile.h"
#include "irda_KeyboardLayouts.h"
#include "CKeyboardNavWrapper.h"
#include "CControlGroupBox.h"
#include "CAccountTemplatesFile.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************


//******************************************************************************
//******************************************************************************
//
//	Main Window implementation
//
//******************************************************************************
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CSettingsWindow::CSettingsWindow() :
	m_pAddDialog(NULL),
	m_pDeleteDialog(NULL),
	m_pMsgDialog(NULL),
	m_pQuickAddDialog(NULL),
	m_pRenameDialog(NULL),
	m_pContextMenu(NULL),
	m_pAccountsTab(NULL),
	m_pAccountTemplate(NULL),
	m_inQuickAdd(false)
{
	m_pTempSettings = new CSettings(CConfigFile::s_pSettings);

	buildConnectionTab();
	buildAccountsTab();
	buildInputTab();

	SetFocus(true);
}

//******************************************************************************
//	
//******************************************************************************
CSettingsWindow::~CSettingsWindow()
{
	// Note that the CTabbedWindow class, CTabbedWindowItem and CControlManager
	// will handle the deletion of all control items when the window is destroyed
	SAFE_DELETE(m_pContextMenu);
	SAFE_DELETE(m_pAccountTemplate);
	SAFE_DELETE(m_pTempSettings);
	
	// As per the above comment, these will get deleted automatically by the
	// component they're registered with.
	m_pAccountsTab = NULL;
	m_pInputControlManager = NULL;
}

//******************************************************************************
//	
//******************************************************************************
void	CSettingsWindow::Render()
{
	// Draw white background
	CGfx::DrawQuad( V2(0, 0), V2(CGfx::s_ScreenWidth, CGfx::s_ScreenHeight), ARGB(255, 255, 255, 255));

	CTabbedWindow::Render();
}

//******************************************************************************
//	
//******************************************************************************
void	CSettingsWindow::Process()
{
	if (m_pContextMenu == NULL && m_pAddDialog == NULL && m_pDeleteDialog == NULL && m_pMsgDialog == NULL) {
		CHUHelp::Clear();
		CHUHelp::SetButton(CHUHelp::BUTTON_UP, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_DOWN, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_RIGHT, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_LEFT, "Navigate");
		CHUHelp::SetButton(CHUHelp::BUTTON_TRIANGLE, "Context Menu");
		CHUHelp::SetButton(CHUHelp::BUTTON_LTRIGGER, "Tab Left");
		CHUHelp::SetButton(CHUHelp::BUTTON_RTRIGGER, "Tab Right");
		CHUHelp::SetButton(CHUHelp::BUTTON_CROSS, "Activate Control");
	}

	if (m_pContextMenu != NULL) {
		if ( m_pContextMenu->getExitValue() != -1 ) {
			//Menu has return a value, process it see eMenuItems enum
			switch(m_pContextMenu->getExitValue()) {
				case HIDE:
					break;
				case CANCEL:
					CPSPoste::s_NewWindow = CWindow::WINDOW_MAIN;
					break;
				case SAVEANDCLOSE:
					Save();
					CPSPoste::Get()->ResetMailbox();
					CPSPoste::s_NewWindow = CWindow::WINDOW_MAIN;
					break;									
				default:
					u32 exitValue = m_pContextMenu->getExitValue();
					//Remove the context menu
					SAFE_DELETE(m_pContextMenu);
					//Return the focus to this window
					SetFocus(true);
					startQuickAdd(exitValue);
					return;
			}
			//Remove the context menu
			SAFE_DELETE(m_pContextMenu);
			//Return the focus to this window
			SetFocus(true);
		}
    }

	if (m_pAddDialog != NULL) {
		if (m_pAddDialog->getExitValue() != -1) {
			switch(m_pAddDialog->getExitValue()) {
				case CDialogBoxInput::OK:
					if (m_inQuickAdd) {
						CString displayName = m_pAddDialog->getReturnText();
						SAFE_DELETE(m_pAddDialog);
						addDisplayNameForQuickAdd(displayName);
						SetFocus(false);
						return;
					} else {
						if (!createNewAccount(m_pAddDialog->getReturnText()))
						{
							SAFE_DELETE(m_pAddDialog);
							m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "The display name entered is already in use", false, false);
							SetFocus(false);
							return;
						}
					}
            		break;
				case CDialogBoxInput::CANCEL:
					if (m_inQuickAdd) {
						stopQuickAdd();	
					}
					break;
				default:
					break;
			}
			//Remove the dialog box
			SAFE_DELETE(m_pAddDialog);
			SetFocus(true);
		}
	}

	if (m_pDeleteDialog != NULL) {
		if (m_pDeleteDialog->getExitValue() != -1) {
			switch(m_pDeleteDialog->getExitValue()) {
				case CDialogBoxInput::OK:
					deleteSelectedAccount();
            		break;
				case CDialogBoxInput::CANCEL:
					break;
				default:
					break;
			}
			//Remove the dialog box
			SAFE_DELETE(m_pDeleteDialog);
			SetFocus(true);
		}
	}

	if (m_pMsgDialog != NULL) {
		if (m_pMsgDialog->getExitValue() != -1) {
			//Remove the dialog box
			SAFE_DELETE(m_pMsgDialog);
			SetFocus(true);
		}
	}

	if (m_pQuickAddDialog != NULL) {
		if (m_pQuickAddDialog->getExitValue() != -1) {
			switch(m_pQuickAddDialog->getExitValue()) {
				case CDialogBoxInput::OK:
					continueQuickAdd(m_pQuickAddDialog->getReturnText());
					return;
            		break;
				case CDialogBoxInput::CANCEL:
					stopQuickAdd();
					break;
				default:
					break;
			}
			//Remove the dialog box
			SAFE_DELETE(m_pQuickAddDialog);
			SetFocus(true);
		}	
	}
	
	if (m_pRenameDialog != NULL) {
		if (m_pRenameDialog->getExitValue() != -1) {
			switch(m_pRenameDialog->getExitValue()) {
				case CDialogBoxInput::OK:
					if (!renameSelectedAccount(m_pRenameDialog->getReturnText()))
					{
						SAFE_DELETE(m_pRenameDialog);
						m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "The display name entered is already in use", false, false);
						SetFocus(false);
						return;
					}
            		break;
				case CDialogBoxInput::CANCEL:
					break;
				default:
					break;
			}
			//Remove the dialog box
			SAFE_DELETE(m_pRenameDialog);
			SetFocus(true);
		}
	}

	if (m_pAddButton->getClicked()) {
		m_pAddDialog = new CDialogBoxInput(250, V2(-1, -1), "Please enter the display name for the account you are creating.", true, true);
		SetFocus(false);
	}
	
	if (m_pDeleteButton->getClicked()) {
		if (m_pTempSettings->accountList->size() <= 1) {
			m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "You must have at least one account.", false, false);
		} else {
			m_pDeleteDialog = new CDialogBoxInput(250, V2(-1, -1), "Are you sure you want to delete the selected account?", false, true);
		}
		SetFocus(false);
	}

	if (m_pRenameButton->getClicked()) {
		m_pRenameDialog = new CDialogBoxInput(250, V2(-1, -1), "Please enter the new display name for the account.", true, true, m_pTempSettings->selectedAccount->display_name);
		SetFocus(false);
	}

	CTabbedWindow::Process();
}

//******************************************************************************
//	
//******************************************************************************
bool CSettingsWindow::ProcessInput()
{
	if (HasFocus() == true && m_pMsgDialog == NULL && m_pQuickAddDialog == NULL) {
		if (CTabbedWindow::ProcessInput()) {
			if (CKeyboardNavWrapper::OpenContextMenu()) {
				createContextMenu();
			}
		}
	}
	return false;
}

//******************************************************************************
//	Take the values from the input boxes and place them in the global settings
//******************************************************************************
void CSettingsWindow::Save()
{
	/* Grab data from connection tab */
	m_pTempSettings->connection_selected = m_pConnectionSelected->getSelectedValue();
	m_pTempSettings->connection_internet_timeout = atoi(m_pConnectionInternetTimeout->getText()->toString().c_str());
	m_pTempSettings->connection_socket_timeout = atoi(m_pConnectionSocketTimeout->getText()->toString().c_str());
	
	/* Grab data from account tab (for currently selected */
	m_pTempSettings->selectedAccount->email_address = m_pEmailAddress->getText()->toString().c_str();
	m_pTempSettings->selectedAccount->reply_to = m_pReplyTo->getText()->toString().c_str();
	m_pTempSettings->selectedAccount->incoming_server = m_pIncomingServer->getText()->toString().c_str();
	m_pTempSettings->selectedAccount->incoming_server_port = atoi(m_pIncomingServerPort->getText()->toString().c_str());
	m_pTempSettings->selectedAccount->incoming_server_username = m_pIncomingServerUsername->getText()->toString().c_str();
	m_pTempSettings->selectedAccount->incoming_server_password = m_pIncomingServerPassword->getText()->toString().c_str();
	m_pTempSettings->selectedAccount->incoming_server_tls = m_pIncomingServerTLS->isChecked();
	
	m_pTempSettings->selectedAccount->outgoing_server = m_pOutgoingServer->getText()->toString().c_str();
	m_pTempSettings->selectedAccount->outgoing_server_port = atoi(m_pOutgoingServerPort->getText()->toString().c_str());
	m_pTempSettings->selectedAccount->outgoing_server_username = m_pOutgoingServerUsername->getText()->toString().c_str();
	m_pTempSettings->selectedAccount->outgoing_server_password = m_pOutgoingServerPassword->getText()->toString().c_str();
	m_pTempSettings->selectedAccount->outgoing_server_tls = m_pOutgoingServerTLS->isChecked();
	
	/* Grab data from options tab */
	m_pTempSettings->keyboard = m_pKeyboardSelected->getSelectedText();
	m_pTempSettings->analogNav = m_pAnalogNavCheckBox->isChecked();
	CInput::SetKeyboardFromName(m_pTempSettings->keyboard, m_pTempSettings->analogNav);

	CSettings* p_temp = CConfigFile::s_pSettings;
	CConfigFile::s_pSettings = m_pTempSettings;
	m_pTempSettings = NULL; // Avoids deleting the new settings!
	SAFE_DELETE(p_temp);

	CConfigFile::Save();
}


/*
 * Build the connection tab
 */
void CSettingsWindow::buildConnectionTab()
{
	CString tempText("");
    CControlManager* p_controlManager;

	CControlSelect::CControlSelectItemList * itemList = new CControlSelect::CControlSelectItemList();	
	CControlSelectItem *currentSelectedConnection;

	//Build the connection list from the connection manager
	std::vector<MPSPConnection> connectionList;
	CConnectionManager::GetPSPConnections(connectionList);
	std::vector<MPSPConnection>::iterator it = connectionList.begin();
	while (it != connectionList.end()) {
		if (m_pTempSettings->connection_selected == (*it).id) {
			currentSelectedConnection = new CControlSelectItem((*it).name, (*it).id);
			itemList->push_back(currentSelectedConnection);
		} else {
			itemList->push_back(new CControlSelectItem((*it).name, (*it).id));
		}
		it++;
	}
	m_pConnectionSelected = new CControlSelect(V2(300, 20), V2(115, 30), 6, itemList, currentSelectedConnection);

	tempText.Printf("%d", m_pTempSettings->connection_internet_timeout);
	m_pConnectionInternetTimeout = new CControlTextBox(V2(300, 20), V2(115, 55),  new CText(tempText.GetPtr()) );
	tempText.Printf("%d", m_pTempSettings->connection_socket_timeout);
	m_pConnectionSocketTimeout = new CControlTextBox (V2(300, 20), V2(115, 80),  new CText(tempText.GetPtr()) );
		
	m_pConnectionSelected->m_pDown = m_pConnectionInternetTimeout;
	m_pConnectionInternetTimeout->m_pUp = m_pConnectionSelected;
	m_pConnectionInternetTimeout->m_pDown = m_pConnectionSocketTimeout;
	m_pConnectionSocketTimeout->m_pUp = m_pConnectionInternetTimeout;
	
	// Build control manager and tab
	p_controlManager = new CControlManager();

	p_controlManager->addControl(new CControlLabel(V2(105, 20), V2(5, 35), 0xFF000000, CString("WiFi Connection:")));
	p_controlManager->addControl(m_pConnectionSelected);
	p_controlManager->addControl(new CControlLabel(V2(105, 20), V2(5, 60), 0xFF000000, CString("Internet Timeout:")));
	p_controlManager->addControl(m_pConnectionInternetTimeout);
	p_controlManager->addControl(new CControlLabel(V2(105, 20), V2(5, 85), 0xFF000000, CString("Socket Timeout:")));
	p_controlManager->addControl(m_pConnectionSocketTimeout);
	p_controlManager->selectControl(m_pConnectionSelected);
	CTabbedWindow::addTab(new CTabbedWindowItem("Connection", p_controlManager));
}

/*
 * Build the accounts tab
 */
void CSettingsWindow::buildAccountsTab()
{
	CString tempText("");
    CControlManager* p_controlManager;

	// Selected Account
	CControlSelect::CControlSelectItemList* accountItemList = new CControlSelect::CControlSelectItemList();
	CControlSelectItem *currentSelectedAccount;
	// Build list of account items
	// Use a for loop instead of iterator so we can ID the selection items
	for (u32 i=0; i < m_pTempSettings->accountList->size(); i++)
	{
		if (m_pTempSettings->selectedAccount == m_pTempSettings->accountList->at(i)) {
			currentSelectedAccount = new CControlSelectItem(m_pTempSettings->accountList->at(i)->display_name, i);
			accountItemList->push_back(currentSelectedAccount);
		} else {
			accountItemList->push_back(new CControlSelectItem(m_pTempSettings->accountList->at(i)->display_name, i));
		}
	}
	m_pAccountSelected = new CControlSelect(V2(190,20), V2(120, 27), 6, accountItemList, currentSelectedAccount);
	m_pAccountSelected->subscribe(this);

	m_pAddButton = new CControlButton(V2(50, 20), V2(315, 27), CString("Add"));
	m_pDeleteButton = new CControlButton(V2(50, 20), V2(370, 27), CString("Delete"));
	m_pRenameButton = new CControlButton(V2(50, 20), V2(425, 27), CString("Rename")); 
	
	// Account Details
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->email_address);
	m_pEmailAddress = new CControlTextBox(V2(353, 20), V2(116, 67), new CText(tempText.GetPtr()));
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->reply_to);
	m_pReplyTo = new CControlTextBox(V2(353, 20), V2(116, 92), new CText(tempText.GetPtr()));
	m_pAccountDetailsGroupBox = new CControlGroupBox(V2(470, 66), V2(5, 52), "Account Details");
	m_pEmailAddress->addSelectionListener(m_pAccountDetailsGroupBox);
	m_pReplyTo->addSelectionListener(m_pAccountDetailsGroupBox);
	
	// Outgoing Sever
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->outgoing_server);
	m_pOutgoingServer = new CControlTextBox(V2(246, 20), V2(79, 138), new CText(tempText.GetPtr()));
	tempText.Printf("%d", m_pTempSettings->selectedAccount->outgoing_server_port);
	m_pOutgoingServerPort = new CControlTextBox(V2(40, 20), V2(369, 138), new CText(tempText.GetPtr()));
	m_pOutgoingServerTLS = new CControlCheckBox(V2(20, 20), V2(449, 138), m_pTempSettings->selectedAccount->outgoing_server_tls);
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->outgoing_server_username);
	m_pOutgoingServerUsername = new CControlTextBox(V2(165, 20), V2(79, 163), new CText(tempText.GetPtr()));
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->outgoing_server_password);
	m_pOutgoingServerPassword = new CControlTextBox(V2(154, 20), V2(316, 163), new CText(tempText.GetPtr()));
	m_pOutgoingServerPassword->setPassword(true);
	m_pOutgoingServerGroupBox = new CControlGroupBox(V2(470, 66), V2(5, 123), "Outgoing Server");
	m_pOutgoingServer->addSelectionListener(m_pOutgoingServerGroupBox);
	m_pOutgoingServerPort->addSelectionListener(m_pOutgoingServerGroupBox);
	m_pOutgoingServerUsername->addSelectionListener(m_pOutgoingServerGroupBox);
	m_pOutgoingServerPassword->addSelectionListener(m_pOutgoingServerGroupBox);
	
	// Incoming Sever
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->incoming_server);
	m_pIncomingServer = new CControlTextBox(V2(246, 20), V2(79, 209), new CText(tempText.GetPtr()));
	tempText.Printf("%d", m_pTempSettings->selectedAccount->incoming_server_port);
	m_pIncomingServerPort = new CControlTextBox(V2(40, 20), V2(369, 209), new CText(tempText.GetPtr()));
	m_pIncomingServerTLS = new CControlCheckBox(V2(20, 20), V2(449, 209), m_pTempSettings->selectedAccount->incoming_server_tls);
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->incoming_server_username);
	m_pIncomingServerUsername = new CControlTextBox(V2(165, 20), V2(79, 234), new CText(tempText.GetPtr()));
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->incoming_server_password);
	m_pIncomingServerPassword = new CControlTextBox(V2(154, 20), V2(316, 234), new CText(tempText.GetPtr()));
	m_pIncomingServerPassword->setPassword(true);
	m_pIncomingServerGroupBox = new CControlGroupBox(V2(470, 66), V2(5, 194), "Incoming Server");
	m_pIncomingServer->addSelectionListener(m_pIncomingServerGroupBox);
	m_pIncomingServerPort->addSelectionListener(m_pIncomingServerGroupBox);
	m_pIncomingServerUsername->addSelectionListener(m_pIncomingServerGroupBox);
	m_pIncomingServerPassword->addSelectionListener(m_pIncomingServerGroupBox);

	p_controlManager = new CControlManager();
	m_pAccountSelected->m_pUp = m_pIncomingServerUsername;
	m_pAccountSelected->m_pDown = m_pEmailAddress;
	m_pAccountSelected->m_pRight = m_pAddButton;
	m_pAddButton->m_pLeft = m_pAccountSelected;
	m_pAddButton->m_pRight = m_pDeleteButton;
	m_pAddButton->m_pDown = m_pEmailAddress;
	m_pDeleteButton->m_pRight = m_pRenameButton;
	m_pDeleteButton->m_pLeft = m_pAddButton;
	m_pDeleteButton->m_pDown = m_pEmailAddress;
	m_pRenameButton->m_pLeft = m_pDeleteButton;
	m_pRenameButton->m_pDown = m_pEmailAddress;
	m_pEmailAddress->m_pUp = m_pAccountSelected;
	m_pEmailAddress->m_pDown = m_pReplyTo;
	m_pReplyTo->m_pUp = m_pEmailAddress;
	m_pReplyTo->m_pDown = m_pOutgoingServer;
	m_pOutgoingServer->m_pUp = m_pReplyTo;
	m_pOutgoingServer->m_pDown = m_pOutgoingServerUsername;
	m_pOutgoingServer->m_pRight = m_pOutgoingServerPort;
	m_pOutgoingServerPort->m_pUp = m_pReplyTo;
	m_pOutgoingServerPort->m_pDown = m_pOutgoingServerPassword;
	m_pOutgoingServerPort->m_pLeft = m_pOutgoingServer;
	m_pOutgoingServerPort->m_pRight = m_pOutgoingServerTLS;
	m_pOutgoingServerTLS->m_pUp = m_pReplyTo;
	m_pOutgoingServerTLS->m_pDown = m_pOutgoingServerPassword;
	m_pOutgoingServerTLS->m_pLeft = m_pOutgoingServerPort;
	m_pOutgoingServerUsername->m_pUp = m_pOutgoingServer;
	m_pOutgoingServerUsername->m_pDown = m_pIncomingServer;
	m_pOutgoingServerUsername->m_pRight = m_pOutgoingServerPassword;
	m_pOutgoingServerPassword->m_pUp = m_pOutgoingServerPort;
	m_pOutgoingServerPassword->m_pDown = m_pIncomingServerPort;
	m_pOutgoingServerPassword->m_pLeft = m_pOutgoingServerUsername;
	m_pIncomingServer->m_pUp = m_pOutgoingServerUsername;
	m_pIncomingServer->m_pDown = m_pIncomingServerUsername;
	m_pIncomingServer->m_pRight = m_pIncomingServerPort;
	m_pIncomingServerPort->m_pUp = m_pOutgoingServerPassword;
	m_pIncomingServerPort->m_pDown = m_pIncomingServerPassword;
	m_pIncomingServerPort->m_pLeft = m_pIncomingServer;
	m_pIncomingServerPort->m_pRight = m_pIncomingServerTLS;
	m_pIncomingServerTLS->m_pUp = m_pOutgoingServerPassword;
	m_pIncomingServerTLS->m_pDown = m_pIncomingServerPassword;
	m_pIncomingServerTLS->m_pLeft = m_pIncomingServerPort;
	m_pIncomingServerUsername->m_pUp = m_pIncomingServer;
	m_pIncomingServerUsername->m_pDown = m_pAccountSelected;
	m_pIncomingServerUsername->m_pRight = m_pIncomingServerPassword;
	m_pIncomingServerPassword->m_pUp = m_pIncomingServerPort;
	m_pIncomingServerPassword->m_pDown = m_pAddButton;
	m_pIncomingServerPassword->m_pLeft = m_pIncomingServerUsername;
	
	
	// Selected Account
	p_controlManager->addControl(new CControlLabel(V2(110, 20), V2(5, 32), 0xFF000000, CString("Selected Account:")));
	p_controlManager->addControl(m_pAccountSelected);
	p_controlManager->addControl(m_pAddButton);
	p_controlManager->addControl(m_pDeleteButton);
	p_controlManager->addControl(m_pRenameButton);
	
	// Account Details
	p_controlManager->addControl(new CControlLabel(V2(100, 20), V2(11, 72), 0xFF000000, CString("Email Address:")));
	p_controlManager->addControl(m_pEmailAddress);
	p_controlManager->addControl(new CControlLabel(V2(100, 20), V2(11, 97), 0xFF000000, CString("Reply To:")));
	p_controlManager->addControl(m_pReplyTo);
	p_controlManager->addControl(m_pAccountDetailsGroupBox);
	
	// Outgoing Server	
	p_controlManager->addControl(new CControlLabel(V2(60, 20), V2(11, 143), 0xFF000000, CString("Address:")));
	p_controlManager->addControl(m_pOutgoingServer);
	p_controlManager->addControl(new CControlLabel(V2(25, 20), V2(335, 143), 0xFF000000, CString("Port:")));
	p_controlManager->addControl(m_pOutgoingServerPort);
	p_controlManager->addControl(new CControlLabel(V2(30, 20), V2(417, 143), 0xFF000000, CString("TLS:")));
	p_controlManager->addControl(m_pOutgoingServerTLS);
	p_controlManager->addControl(new CControlLabel(V2(66, 20), V2(11, 168), 0xFF000000, CString("Username:")));
	p_controlManager->addControl(m_pOutgoingServerUsername);
	p_controlManager->addControl(new CControlLabel(V2(62, 20), V2(252, 168), 0xFF000000, CString("Password:")));
	p_controlManager->addControl(m_pOutgoingServerPassword);
	p_controlManager->addControl(m_pOutgoingServerGroupBox);
	
	// Incoming Server 
	p_controlManager->addControl(new CControlLabel(V2(60, 20), V2(11, 214), 0xFF000000, CString("Address:")));
	p_controlManager->addControl(m_pIncomingServer);
	p_controlManager->addControl(new CControlLabel(V2(25, 20), V2(335, 214), 0xFF000000, CString("Port:")));
	p_controlManager->addControl(m_pIncomingServerPort);
	p_controlManager->addControl(new CControlLabel(V2(30, 20), V2(417, 214), 0xFF000000, CString("TLS:")));
	p_controlManager->addControl(m_pIncomingServerTLS);
	p_controlManager->addControl(new CControlLabel(V2(66, 20), V2(11, 239), 0xFF000000, CString("Username:")));
	p_controlManager->addControl(m_pIncomingServerUsername);
	p_controlManager->addControl(new CControlLabel(V2(62, 20), V2(252, 239), 0xFF000000, CString("Password:")));
	p_controlManager->addControl(m_pIncomingServerPassword);
	p_controlManager->addControl(m_pIncomingServerGroupBox);
	
	p_controlManager->selectControl(m_pAccountSelected);
	m_pAccountsTab = new CTabbedWindowItem("Accounts", p_controlManager);
	CTabbedWindow::addTab(m_pAccountsTab);
}

/*
 * Build the options tab
 */
void CSettingsWindow::buildInputTab()
{
	CString tempText("");

	CControlSelect::CControlSelectItemList * keyboardItemList = new CControlSelect::CControlSelectItemList();	
	CControlSelectItem *currentSelectedKeyboard;
	CString keyboard("");
	
	for (int i = 0; i < 6; i++)
	{
		if (i == 0) keyboard = "p-sprint";
		if (i == 1) keyboard = "IR Keyboard (pspirkeyb)";
		if (i == 2) keyboard = "GUI Keyboard";
		if (i == 3) keyboard = "GUI Keyboard Analog1";
		if (i == 4) keyboard = "Danzeff";
		if (i == 5) keyboard = "IR Keyboard (Stowaway Legacy)";
	 	if (keyboard == m_pTempSettings->keyboard) 
	 	{
  		 	currentSelectedKeyboard = new CControlSelectItem(keyboard, i); 
		 	keyboardItemList->push_back(currentSelectedKeyboard);
		}
		else 
		{
		 	keyboardItemList->push_back(new CControlSelectItem(keyboard, i));		 	 
		}
	}
	
	m_pKeyboardSelected = new CControlSelect(V2(300, 20), V2(110, 30), 5, keyboardItemList, currentSelectedKeyboard);
	m_pKeyboardSelected->subscribe(this);
	m_pAnalogNavCheckBox = new CControlCheckBox(V2(20, 20), V2(110, 55), CConfigFile::s_pSettings->analogNav);

	m_pKeyboardSelected->m_pDown = m_pAnalogNavCheckBox;
	m_pAnalogNavCheckBox->m_pUp = m_pKeyboardSelected;

	m_pInputControlManager = new CControlManager();
	m_pInputControlManager->addControl(new CControlLabel(V2(100, 20), V2(5, 35), 0xFF000000, CString("Input Method:")));
	m_pInputControlManager->addControl(m_pKeyboardSelected);
	m_pInputControlManager->addControl(new CControlLabel(V2(100, 20), V2(5, 60), 0xFF000000, CString("Analog Nav:")));
	m_pInputControlManager->addControl(m_pAnalogNavCheckBox);
	m_pInputControlManager->selectControl(m_pKeyboardSelected);
	CTabbedWindow::addTab(new CTabbedWindowItem("Input", m_pInputControlManager));
	setAnalogNavUiState();
}

/*
 * Create the account with the given display name
 *
 * Returns true if account creation is successful, false otherwise
 * (display name already in use)
 */
bool CSettingsWindow::createNewAccount(CString displayName)
{
	// Ensure that the display name doesn't already exist
	CSettings::CAccountList::iterator it = m_pTempSettings->accountList->begin();
	while (it != m_pTempSettings->accountList->end()) {
		if (displayName.Equals((*it)->display_name)) {
			return false;
		}
		it++;
	}

	CAccount* p_newAccount = new CAccount();
	p_newAccount->display_name = displayName;
	m_pTempSettings->accountList->push_back(p_newAccount);
	m_pTempSettings->selectedAccount = p_newAccount;

	rebuildSelectedAccount(p_newAccount);
	setAccountFields();

	return true;
}

/*
 * Delete the selected account
 */
void CSettingsWindow::deleteSelectedAccount()
{
	CSettings::CAccountList::iterator it = m_pTempSettings->accountList->begin();

	/* Ensure there are items to delete */
	if (m_pTempSettings->accountList->size() <= 1)
	{
		return;
	}

	/* Delete the item and rebuild the list */
	while (it != m_pTempSettings->accountList->end()) {
		if (*it == m_pTempSettings->selectedAccount) {
			m_pTempSettings->accountList->erase(it);
			rebuildSelectedAccount(NULL);
			break;
		}
		it++;
	}

	// Update the selected account to the first one in the list (somewhat cheating)
	m_pTempSettings->selectedAccount = *m_pTempSettings->accountList->begin();
		
	setAccountFields();
}

void CSettingsWindow::setAccountFields()
{
	CString tempText("");
	
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->email_address);
	m_pEmailAddress->setText(new CText(tempText.GetPtr()));
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->reply_to);
	m_pReplyTo->setText(new CText(tempText.GetPtr()));
	
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->incoming_server);
	m_pIncomingServer->setText(new CText(tempText.GetPtr()));
	tempText.Printf("%d", m_pTempSettings->selectedAccount->incoming_server_port);
	m_pIncomingServerPort->setText(new CText(tempText.GetPtr()));
	m_pIncomingServerTLS->setCheck(m_pTempSettings->selectedAccount->incoming_server_tls);
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->incoming_server_username);
	m_pIncomingServerUsername->setText(new CText(tempText.GetPtr()));
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->incoming_server_password);
	m_pIncomingServerPassword->setText(new CText(tempText.GetPtr()));
	
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->outgoing_server);
	m_pOutgoingServer->setText(new CText(tempText.GetPtr()));
	tempText.Printf("%d", m_pTempSettings->selectedAccount->outgoing_server_port);
	m_pOutgoingServerPort->setText(new CText(tempText.GetPtr()));
	m_pOutgoingServerTLS->setCheck(m_pTempSettings->selectedAccount->outgoing_server_tls);
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->outgoing_server_username);
	m_pOutgoingServerUsername->setText(new CText(tempText.GetPtr()));
	tempText.Printf("%s", (const char *)m_pTempSettings->selectedAccount->outgoing_server_password);
	m_pOutgoingServerPassword->setText(new CText(tempText.GetPtr()));
}

void CSettingsWindow::onChange(CControlSelect* control)
{
	if (control == m_pAccountSelected)
	{
		// Grab the current selected items
		m_pTempSettings->selectedAccount->email_address = m_pEmailAddress->getText()->toString().c_str();
		m_pTempSettings->selectedAccount->reply_to = m_pReplyTo->getText()->toString().c_str();
		
		m_pTempSettings->selectedAccount->incoming_server = m_pIncomingServer->getText()->toString().c_str();
		m_pTempSettings->selectedAccount->incoming_server_port = atoi(m_pIncomingServerPort->getText()->toString().c_str());
		m_pTempSettings->selectedAccount->incoming_server_username = m_pIncomingServerUsername->getText()->toString().c_str();
		m_pTempSettings->selectedAccount->incoming_server_password = m_pIncomingServerPassword->getText()->toString().c_str();
		m_pTempSettings->selectedAccount->incoming_server_tls = m_pIncomingServerTLS->isChecked();
		
		m_pTempSettings->selectedAccount->outgoing_server = m_pOutgoingServer->getText()->toString().c_str();
		m_pTempSettings->selectedAccount->outgoing_server_port = atoi(m_pOutgoingServerPort->getText()->toString().c_str());
		m_pTempSettings->selectedAccount->outgoing_server_username = m_pOutgoingServerUsername->getText()->toString().c_str();
		m_pTempSettings->selectedAccount->outgoing_server_password = m_pOutgoingServerPassword->getText()->toString().c_str();
		m_pTempSettings->selectedAccount->outgoing_server_tls = m_pOutgoingServerTLS->isChecked();
	
		// Update the selected account field
		std::vector<CAccount*>::iterator it = m_pTempSettings->accountList->begin();
		while (it != m_pTempSettings->accountList->end())
		{
			if (m_pAccountSelected->getSelectedText().Equals((*it)->display_name))
			{
				m_pTempSettings->selectedAccount = *it;
			}
			it++;
		}
	
		// Update the text boxes with the newly selected account information
		setAccountFields();
	}
	else if (control == m_pKeyboardSelected)
	{
		setAnalogNavUiState();
	}
}

void CSettingsWindow::rebuildSelectedAccount(CAccount *selectedAccount)
{
	CControlSelect::CControlSelectItemList* accountItemList = new CControlSelect::CControlSelectItemList();
	CControlSelectItem *currentSelectedAccount = NULL;

	for (u32 i=0; i < m_pTempSettings->accountList->size(); i++)
	{
		if (m_pTempSettings->accountList->at(i) == selectedAccount) {
			currentSelectedAccount = new CControlSelectItem(m_pTempSettings->accountList->at(i)->display_name, i);
			accountItemList->push_back(currentSelectedAccount);
		} else {
			accountItemList->push_back(new CControlSelectItem(m_pTempSettings->accountList->at(i)->display_name, i));
		}
	}
	m_pAccountSelected->rebuildList(accountItemList, currentSelectedAccount);
}

//******************************************************************************
//	
//******************************************************************************
void CSettingsWindow::createContextMenu()
{
	
	// Buld the context menu
	CContextMenu::CContextMenuItemList * itemList = new CContextMenu::CContextMenuItemList();
	
	itemList->push_back( new CContextMenuItem("Hide Menu", HIDE, true));
	
	// If the accounts tab is active, allow quick create
	if (m_pSelectedTab == m_pAccountsTab && CAccountTemplatesFile::s_pAccountTemplates->size() > 0) {
		CContextMenu::CContextMenuItemList* templateList = new CContextMenu::CContextMenuItemList();
		
		u32 counter = MENU_SIZE;
		  
		CAccountTemplatesFile::CAccountTemplateList::iterator it = CAccountTemplatesFile::s_pAccountTemplates->begin();
		while (it != CAccountTemplatesFile::s_pAccountTemplates->end())
		{
			templateList->push_back(new CContextMenuItem((*it)->template_name, counter, false));
			++it;
			++counter;
		}
		itemList->push_back(new CContextMenuItem("< Quick Account Add", -1, true, templateList));
	}
	
	itemList->push_back( new CContextMenuItem("Cancel", CANCEL, false));
	
	if (CPSPoste::Get()->IsAccountSwitchSafe())
	{
		itemList->push_back( new CContextMenuItem("Save and Close", SAVEANDCLOSE, false));
	}

	// Create the context menu
	m_pContextMenu = new CContextMenu(itemList, V2(10, 10));
				
	SetFocus( false );
}

void CSettingsWindow::startQuickAdd(u32 templateNumber)
{
	m_pAccountTemplate = new CAccountTemplate(CAccountTemplatesFile::s_pAccountTemplates->at(templateNumber - MENU_SIZE));
	m_quickAdd_it = m_pAccountTemplate->p_paramList->begin();
	m_inQuickAdd = true;
	// Launch the add dialog to prompt for the display name
	m_pAddDialog = new CDialogBoxInput(250, V2(-1, -1), "Please enter the display name for the account you are creating.", true, true);
	SetFocus(false);
	
	return;	
}

void CSettingsWindow::addDisplayNameForQuickAdd(CString paramValue)
{
	m_pAccountTemplate->display_name = paramValue;	
	
	m_pQuickAddDialog = new CDialogBoxInput(250, V2(-1, -1), (*m_quickAdd_it)->prompt, true, true, (*m_quickAdd_it)->value, true);
	SetFocus(false);
}

void CSettingsWindow::continueQuickAdd(CString paramValue)
{
	(*m_quickAdd_it)->value = paramValue;
	++m_quickAdd_it;
	if (m_quickAdd_it == m_pAccountTemplate->p_paramList->end()) {
		SAFE_DELETE(m_pQuickAddDialog);
		finishQuickAdd();
	} else {
		SAFE_DELETE(m_pQuickAddDialog);
		m_pQuickAddDialog = new CDialogBoxInput(250, V2(-1, -1), (*m_quickAdd_it)->prompt, true, true, (*m_quickAdd_it)->value, false);
		SetFocus(false);
	}
}

void CSettingsWindow::finishQuickAdd() 
{
	if (!createNewAccount(m_pAccountTemplate->display_name))
	{
		m_pMsgDialog = new CDialogBoxInput(250, V2(-1, -1), "The display name entered is already in use", false, false);
		SetFocus(false);
	} else {
		// Do the loading
		m_pTempSettings->selectedAccount->email_address = m_pAccountTemplate->replaceWithParams(m_pAccountTemplate->email_address);
		m_pTempSettings->selectedAccount->reply_to = m_pAccountTemplate->replaceWithParams(m_pAccountTemplate->reply_to);
		m_pTempSettings->selectedAccount->incoming_server = m_pAccountTemplate->replaceWithParams(m_pAccountTemplate->incoming_server);
		m_pTempSettings->selectedAccount->incoming_server_port = atoi(m_pAccountTemplate->replaceWithParams(m_pAccountTemplate->incoming_server_port));
		m_pTempSettings->selectedAccount->incoming_server_tls = m_pAccountTemplate->replaceWithParams(m_pAccountTemplate->incoming_server_tls).Equals("true") ? true : false;
		m_pTempSettings->selectedAccount->incoming_server_username = m_pAccountTemplate->replaceWithParams(m_pAccountTemplate->incoming_server_username);
		m_pTempSettings->selectedAccount->incoming_server_password = m_pAccountTemplate->replaceWithParams(m_pAccountTemplate->incoming_server_password);
		m_pTempSettings->selectedAccount->outgoing_server = m_pAccountTemplate->replaceWithParams(m_pAccountTemplate->outgoing_server);
		m_pTempSettings->selectedAccount->outgoing_server_port = atoi(m_pAccountTemplate->replaceWithParams(m_pAccountTemplate->outgoing_server_port));
		m_pTempSettings->selectedAccount->outgoing_server_tls = m_pAccountTemplate->replaceWithParams(m_pAccountTemplate->outgoing_server_tls).Equals("true") ? true : false;
		m_pTempSettings->selectedAccount->outgoing_server_username = m_pAccountTemplate->replaceWithParams(m_pAccountTemplate->outgoing_server_username);
		m_pTempSettings->selectedAccount->outgoing_server_password = m_pAccountTemplate->replaceWithParams(m_pAccountTemplate->outgoing_server_password);

		//m_pOutgoingServerUsername->setText(new CText(tempText.GetPtr()));
		//tempText.Printf("%s", (const char *));
			
		setAccountFields();
		SetFocus(true);	
	}
	stopQuickAdd();
}

void CSettingsWindow::stopQuickAdd()
{
	SAFE_DELETE(m_pAccountTemplate);
	m_inQuickAdd = false;	
}

/*
 * Create the account with the given display name
 *
 * Returns true if account creation is successful, false otherwise
 * (display name already in use)
 */
bool CSettingsWindow::renameSelectedAccount(CString name)
{
	// Ensure that the display name doesn't already exist
	CSettings::CAccountList::iterator it = m_pTempSettings->accountList->begin();
	while (it != m_pTempSettings->accountList->end()) {
		if ((*it) != m_pTempSettings->selectedAccount && name.Equals((*it)->display_name)) {
			return false;
		}
		it++;
	}

	CPSPoste::Get()->RenameMailbox(m_pTempSettings->selectedAccount->display_name, name);

	m_pTempSettings->selectedAccount->display_name = name;

	rebuildSelectedAccount(m_pTempSettings->selectedAccount);
	setAccountFields();

	return true;
}

void CSettingsWindow::setAnalogNavUiState()
{
	if ((m_pKeyboardSelected->getSelectedText().Equals("IR Keyboard")) || (m_pKeyboardSelected->getSelectedText().Equals("pspirkeyb")))
	{
		// The IR Keyboards have their own nav keyboard and don't allow analog nav
		m_pInputControlManager->disableControl(m_pAnalogNavCheckBox);
		m_pTempSettings->analogNav = m_pAnalogNavCheckBox->isChecked();
		m_pAnalogNavCheckBox->unCheck();
	}
	else
	{
		// Save the state if coming from a keyboard that allows analog nav
		if (m_pAnalogNavCheckBox->isEnabled())
			m_pTempSettings->analogNav = m_pAnalogNavCheckBox->isChecked();
			
		m_pInputControlManager->enableControl(m_pAnalogNavCheckBox);
		m_pAnalogNavCheckBox->setCheck(m_pTempSettings->analogNav);
	}
}

//*******************************  END OF FILE  ********************************
