/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CSettingsWindow.h

  Description :	

  Last Modified $Date: 2007/02/05 01:28:40 $

  $Revision: 1.6 $

*******************************************************************************/

#ifndef CSettingsWindow_H_
#define CSettingsWindow_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CTabbedWindow.h"
#include "CFileSystem.h"
#include "CString.h"
#include "CGfx.h"
#include "CControlSelect.h"
#include "CControlTextBox.h"
#include "CContextMenu.h"
#include "CControlManager.h"
#include "CControlLabel.h"
#include "CControlButton.h"
#include "CControlCheckBox.h"
#include "CDialogBoxInput.h"
#include "CConfigFile.h"
#include "CControlGroupBox.h"
#include "CAccountTemplatesFile.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************
class CSettingsWindow : public CTabbedWindow, public MControlSelectEventListener
{
	public:
		CSettingsWindow();
		~CSettingsWindow();

	public: // Inherited from CTabbedWindow
		virtual void Render();
		virtual void Process();
		virtual bool ProcessInput();

	public: // Inherited from MControlSelectEventListener
		void onChange(CControlSelect* control);

	public:
		/* Connection Tab Controls */
		CControlSelect* m_pConnectionSelected;
		CControlTextBox* m_pConnectionInternetTimeout;
		CControlTextBox* m_pConnectionSocketTimeout;

		/* Accounts Tab Controls */
		CControlSelect* m_pAccountSelected;
		CControlButton* m_pAddButton;
		CControlButton* m_pDeleteButton;
		CControlButton* m_pRenameButton;
		CControlTextBox* m_pEmailAddress;
		CControlTextBox* m_pIncomingServer;
		CControlTextBox* m_pIncomingServerPort;
		CControlTextBox* m_pIncomingServerUsername;
		CControlTextBox* m_pIncomingServerPassword;
		CControlCheckBox* m_pIncomingServerTLS;
		CControlTextBox* m_pOutgoingServer;
		CControlTextBox* m_pOutgoingServerPort;
		CControlTextBox* m_pOutgoingServerUsername;
		CControlTextBox* m_pOutgoingServerPassword;
		CControlCheckBox* m_pOutgoingServerTLS;
		CControlTextBox* m_pReplyTo;

		CControlGroupBox* m_pAccountDetailsGroupBox;
		CControlGroupBox* m_pIncomingServerGroupBox;
		CControlGroupBox* m_pOutgoingServerGroupBox;
		
		/* Input Tab Controls */
		CControlSelect* m_pKeyboardSelected;
		CControlCheckBox* m_pAnalogNavCheckBox;

		/* Dialog Boxes */
		CDialogBoxInput* m_pAddDialog;
		CDialogBoxInput* m_pDeleteDialog;
		CDialogBoxInput* m_pMsgDialog;
		CDialogBoxInput* m_pQuickAddDialog;
		CDialogBoxInput* m_pRenameDialog;
		
		/* Context Menu */
		CContextMenu* m_pContextMenu;

		CTabbedWindowItem* m_pAccountsTab;
		CControlManager* m_pInputControlManager; // Keep a reference to enable/disable the analog nav checkbox

	/* These values used for the quick add state machine */
	private:
		CAccountTemplate::CAccountTemplateParamList::iterator m_quickAdd_it;
		CAccountTemplate* m_pAccountTemplate;
		bool m_inQuickAdd;
		
	private:
		/* Saves the inputted settings to xml*/
		void Save();

		/* Build the connection tab controls and manager */
		void buildConnectionTab();

		/* Build the accounts tab controls and manager */
		void buildAccountsTab();

		/* Build the options tab controls and manager */
		void buildInputTab();

		/* Create a new account with given display name */
		bool createNewAccount(CString displayName);

		/* Delete the selected account */
		void deleteSelectedAccount();

		/* Sets the account fields to the selected connection's properties */
		void setAccountFields();

		void createContextMenu();

		void startQuickAdd(u32 templateNumber);

		void finishQuickAdd();
		
		void addDisplayNameForQuickAdd(CString paramValue);
		
		void continueQuickAdd(CString promptValue);
		
		void stopQuickAdd();
		
		bool renameSelectedAccount(CString name);
		
		/* Rebuilds the selected account select control */
		void rebuildSelectedAccount(CAccount* selectedAccount);
		
		void setAnalogNavUiState();

		CSettings* m_pTempSettings;

		enum eMenuItems {
			HIDE,
			CANCEL,
			SAVEANDCLOSE,
			MENU_SIZE
		};	
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CSettingsWindow_H_ */
