/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CSkinManager.h

  Description :	

  Last Modified $Date: 2008/09/29 04:22:27 $

  $Revision: 1.4 $

*******************************************************************************/

#ifndef CSKINMANAGER_H_
#define CSKINMANAGER_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CString.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************
class CTexture;
class TiXmlDocument;

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************

//******************************************************************************
//******************************************************************************
//
//	Skin texture class definition
//
//******************************************************************************
//******************************************************************************
class CSkinTexture : public CBase
{
	public:

		CSkinTexture();
		~CSkinTexture();

		void				Load( const CString & texture_name );

		CTexture *			GetTexture() const;

	private:

		CTexture *			m_pTexture;
};


//******************************************************************************
//******************************************************************************
//
//	Skin manager class definition
//
//******************************************************************************
//******************************************************************************
class CSkinManager
{
	public:

		enum eSkinComponent
		{
			SC_FONT,

			SC_BACKGROUND,

			SC_ICON_MESSAGES,
			SC_ICON_COMPOSE,
			SC_ICON_CONTACTS,
			SC_ICON_SETTINGS,
			SC_ICON_GAME,
			SC_ICON_EXIT,
			SC_ICON_NEWMAIL,

			SC_ICON_MAIL_OPEN,
			SC_ICON_MAIL_FORWARD,
			SC_ICON_MAIL_REPLY,
			SC_ICON_MAIL_REPLYALL,
			SC_ICON_MAIL_MARK,

			SC_BUTTON_CROSS,
			SC_BUTTON_CIRCLE,
			SC_BUTTON_SQUARE,
			SC_BUTTON_TRIANGLE,
			SC_BUTTON_LTRIGGER,
			SC_BUTTON_RTRIGGER,
			SC_BUTTON_START,
			SC_BUTTON_SELECT,
			SC_BUTTON_UP,
			SC_BUTTON_DOWN,
			SC_BUTTON_RIGHT,
			SC_BUTTON_LEFT,

			SC_BATTERY_00,
			SC_BATTERY_01,
			SC_BATTERY_02,
			SC_BATTERY_03,

			MAX_SKIN_COMPONENTS
		};

	public:

		static void				Open();
		static void				Close();

		static void				SetCurrentSkin( const CString & skin_name );

		static CSkinTexture *	GetComponent( eSkinComponent component );

		static V2				GetV2( const CString & section, const CString & sub_section, const V2 & default_val );
		static ARGB				GetColor( const CString & section, const CString & sub_section, const ARGB & default_val );
		static float			GetFloat( const CString & section, const CString & sub_section, const float & default_val );

		static void				OptimizeTextures();
		static void				UploadFontTexture();
		
		static CString			GetUiFontPath();
		static CString			GetTextFontPath();

	protected:

		static void				LoadConfig();

	public:

		static const CString	s_szSkinFolder;

	private:

		static bool				s_bConfigLoaded;
		static TiXmlDocument *	s_pSkinConfigFile;
		static const CString	s_szComponentNames[ MAX_SKIN_COMPONENTS ];
		static CSkinTexture *	s_pComponents[ MAX_SKIN_COMPONENTS ];
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CSKINMANAGER_H_ */
