/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CSmtp.cpp

  Description :	

*******************************************************************************/

#include "CSmtp.h"

#include <time.h>

#include <vmime/vmime.hpp>
#include <vmime/platforms/psp/pspHandler.hpp>

#include "CLog.h"
#include "CMailbox.h"

int CSmtp::Test(CSendMsg* msg)
{
	try
	{
		CAccount* m_account = CConfigFile::s_pSettings->selectedAccount;

		// Build the URL
		CString smtpUrl;
		if (m_account->incoming_server.Substring(0, 7).IEquals("smtp://"))
		{
			smtpUrl = m_account->incoming_server;
		}
		else
		{
			smtpUrl = "smtp://";
			smtpUrl += m_account->outgoing_server;
			if (m_account->outgoing_server_port != 0)
			{
				smtpUrl += ":";
				smtpUrl += m_account->outgoing_server_port;
			}
		}
		
		vmime::platform::setHandler <vmime::platforms::psp::pspHandler>();
		
		// Build the message
		vmime::messageBuilder mb;
		mb.setSubject(vmime::text(msg->subject));
		mb.setExpeditor(vmime::mailbox(CConfigFile::s_pSettings->selectedAccount->email_address));
		mb.setRecipients(msg->to);
		mb.setBlindCopyRecipients(msg->bcc);
		mb.setCopyRecipients(msg->cc);
		mb.getTextPart()->setText(vmime::create<vmime::stringContentHandler>(msg->body));
		
		// Message construction
		vmime::ref<vmime::message> msg = mb.construct();

		// Append a reply to field (no message builder method to do this
		vmime::headerFieldFactory* hfFactory = vmime::headerFieldFactory::getInstance();
		vmime::ref<vmime::header> hdr = msg->getHeader();
		vmime::ref<vmime::headerField> replyToField =
			hfFactory->create(vmime::fields::REPLY_TO);
		replyToField->setValue(vmime::text(CConfigFile::s_pSettings->selectedAccount->reply_to));
		hdr->appendField(replyToField);

		// Create a new session
		vmime::utility::url url(smtpUrl);
		url.setPort(vmime::port_t(CConfigFile::s_pSettings->selectedAccount->outgoing_server_port));
		vmime::ref <vmime::net::session> sess = vmime::create <vmime::net::session>();
		
		// Create an instance of the transport service
		vmime::ref <vmime::net::transport> tr = sess->getTransport(url);
		
		// Set authentication
		/*sess->getProperties()[tr->getInfos().getPropertyPrefix() + "auth.username"] = 
			CConfigFile::s_pSettings->selectedAccount->username;
		sess->getProperties()[tr->getInfos().getPropertyPrefix() + "auth.password"] =
			CConfigFile::s_pSettings->selectedAccount->password;
		*/

		// Connect it
		tr->connect();
		
		// Send the message
		tr->send(msg);

		// We have finished using the service
		tr->disconnect();
	}
	catch (vmime::exception& e)
	{
		ASSERT(0, "VMIME Exception");
		CLog::Error(e.what());
		return 1;
	}
	catch (std::exception& e)
	{
		ASSERT(0, "STD Exception");
		CLog::Error(e.what());
		return 1;
	}
	
	return 0;
}
