/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CString.h
  
  Description :
  
  Last Modified $Date: 2008/09/29 04:22:27 $
  
  $Revision: 1.5 $

*******************************************************************************/

#ifndef CSTRING_H_
#define CSTRING_H_

#include "CBase.h"

#include <CAssert.h>
#include <psptypes.h>

#include <string>
#include <string.h>
#include <stdarg.h>

class CString : public CBase
{
	public: // Constructors

		// Empty string
		CString() : m_str() {};

		// Initialize to aString
		CString(const CString& aString) : m_str(aString.GetStr()) {};

		// Initialize to aString
		CString(const std::string aString) : m_str(aString) {};

		// Initialize to aString
		CString(const char* aString)
		{
			SafeAssign(aString);
		}

		// Initialize to aString1 + aString2
		CString(const char* aString1, const char* aString2)
		{
			SafeAssign(aString1);
				
			if (aString2 != NULL)
				m_str.append(aString2);
		}
		
		CString(const char* aString, u32 len)
		{
			SafeAssign(aString);
		}

		CString(s32 val)
		{
			*this = val;
		}
		

	public: // Overloaded operators

		// Overloaded casts
		operator const std::string () const
		{
			return m_str;
		}
		
		operator const char* () const
		{
			return m_str.c_str();
		}

		// Overloaded =
		CString& operator= (const CString& aString)
		{
			m_str.assign(aString.GetStr());
			return *this;
		}
		
		CString& operator= (const std::string& aString)
		{
			m_str.assign(aString);
			return *this;
		}
		
		CString& operator= (const char* aString)
		{
			SafeAssign(aString);
			return *this;
		}
		
		CString& operator= (s32 aVal)
		{
			Printf("%d", aVal);
			return *this;
		}

		// Overloaded +=
		CString& operator+= (const CString& aString)
		{
			m_str.append(aString.GetStr());
			return *this;
		}
				
		CString& operator+= (const std::string& aString)
		{
			m_str.append(aString);
			return *this;
		}
		
		CString& operator+= (const char* aString)
		{
			if (aString != NULL)
				m_str.append(aString);
			return *this;
		}
		
		CString& operator+= (const s32 aInt)
		{
			char intStr[12]; // 10 Chars for up to 2^32, 1 char for sign, 1 for \0
			snprintf(intStr, 12, "%d", (int)aInt);
			
			m_str.append(intStr);
			return *this;
		}
		
		CString& operator+= (const u32 aInt)
		{
			char intStr[12]; // 10 Chars for up to 2^32, 1 char for sign, 1 for \0
			snprintf(intStr, 12, "%u", (int)aInt);
			
			m_str.append(intStr);
			return *this;
		}

		// Overloaded +
		CString operator+ (const CString& aString) const
		{
			CString copy(m_str);
			copy.Append(aString);
			return copy;
		}
				
		CString operator+ (const std::string& aString) const
		{
			CString copy(m_str);
			copy.Append(aString);
			return copy;
		}
		
		CString operator+ (const char* aString) const
		{
			if (aString == NULL)
				return CString(m_str);
				
			CString copy(m_str);
			copy.Append(aString);
			return copy;
		}
		
		CString operator+ (const s32 aInt) const
		{
			char intStr[12]; // 10 Chars for up to 2^32, 1 char for sign, 1 for \0
			snprintf(intStr, 12, "%d", (int)aInt);
			
			CString copy(m_str);
			copy.Append(intStr);
			return copy;
		}
		
		CString operator+ (const u32 aInt) const
		{
			char intStr[12]; // 10 Chars for up to 2^32, 1 char for sign, 1 for \0
			snprintf(intStr, 12, "%u", (int)aInt);
			
			CString copy(m_str);
			copy.Append(intStr);
			return copy;
		}

		// Overloaded ==
		bool operator== (const CString& aString) const
		{
			return (m_str == aString.GetStr());
		}
				
		bool operator== (const std::string& aString) const
		{
			return (m_str == aString);
		}
				
		bool operator== (const char* aString) const
		{
			if (aString == NULL)
				return m_str == "";
			return (m_str == aString);
		}

		// Overloaded !=
		bool operator!= (const CString& aString) const
		{
			return (m_str != aString.GetPtr());
		}
				
		bool operator!= (const std::string& aString) const
		{
			return (m_str != aString);
		}
				
		bool operator!= (const char* aString) const
		{
			if (aString == NULL)
				return (m_str != "");
			return (m_str != aString);
		}

		// Overloaded >
		bool operator> (const CString& aString) const
		{
			return (strcmpi(m_str.c_str(), aString.GetPtr()) > 0);
		}
		
		bool operator> (const std::string aString) const
		{
			return (strcmpi(m_str.c_str(), aString.c_str()) > 0);
		}
		
		bool operator> (const char* aString) const
		{
			return (strcmpi(m_str.c_str(), aString) > 0);
		}

		// Overloaded <
		bool operator< (const CString& aString) const
		{
			return (strcmpi(m_str.c_str(), aString.GetPtr()) < 0);
		}
				
		bool operator< (const std::string aString) const
		{
			return (strcmpi(m_str.c_str(), aString.c_str()) < 0);
		}
				
		bool operator< (const char* aString) const
		{
			return (strcmpi(m_str.c_str(), aString) < 0);
		}


	public: // Convenience methods
		
		CString& Reverse()
		{
			const u32 len(Length());
			const char* str(GetPtr());
			char dest[Length()];

			for (u32 i = 0; i < (len/2); ++i)
			{
				const char tmp(str[i]);

				dest[i] = str[len - 1 - i];
				dest[len - 1 - i] = tmp;
			}

			m_str = dest;
			return *this;
		}
		
		const char* GetPtr() const
		{
			return m_str.c_str();
		}
		
		const std::string GetStr() const
		{
			return m_str;
		}
		
		bool Equals(const char* aString) const
		{
			return (strcmp(m_str.c_str(), aString) == 0);
		}
				
		bool IEquals(const char* aString) const
		{
			return (strcmpi(m_str.c_str(), aString) == 0);
		}

		void ToUpper()
		{
			strupr(const_cast<char*>(m_str.c_str()));
		}
		
		void ToLower()
		{
			strlwr(const_cast<char*>(m_str.c_str()));
		}
		
		s32 Length() const
		{
			return m_str.length();
		}
				
		bool IsEmpty() const
		{
			return m_str.empty();
		}
				
		// The Find API is the only one to have changed from the old CString. It
		// is now a little more standard and less prone to "evil" use.
		int Find(const CString& aString, int pos = 0) const
		{
			return m_str.find(aString.GetStr(), pos);
		}
		
		int Find(const std::string aString, int pos = 0) const
		{
			return m_str.find(aString, pos);
		}
		
		int Find(const char* aString, int pos = 0) const
		{
			return m_str.find(aString, pos);
		}
				
		int Find(char c, int pos = 0) const
		{
			return m_str.find(c, pos);
		}
		
		int RFind(const CString& aString, int pos = -1) const
		{
			if (pos < 0)
				pos = m_str.size() - 1;
			return m_str.rfind(aString.GetStr(), pos);
		}
		
		int RFind(const std::string aString, int pos = -1) const
		{
			if (pos < 0)
				pos = m_str.size() - 1;
			return m_str.rfind(aString, pos);
		}
		
		int RFind(const char* aString, int pos = -1) const
		{
			if (pos < 0)
				pos = m_str.size() - 1;
			return m_str.rfind(aString, pos);
		}
				
		int RFind(char c, int pos = -1) const
		{
			if (pos < 0)
				pos = m_str.size() - 1;
			return m_str.rfind(c, pos);
		}
		
		CString Substring(int start, int size) const
		{
			return CString(m_str.substr(start, size));
		}
		
		CString Substring(int start) const
		{
			return CString(m_str.substr(start, m_str.size() - start));
		}
		
		CString& Append(const CString& aString)
		{
			m_str.append(aString.GetStr());
			return *this;
		}
				
		CString& Append(const std::string aString)
		{
			m_str.append(aString);
			return *this;
		}
		
		CString& Append(const char* aString)
		{
			m_str.append(aString);
			return *this;
		}
		
		CString& Prepend(const CString& aString)
		{
			m_str.insert(0, aString.GetStr());
			return *this;
		}
				
		CString& Prepend(const std::string aString)
		{
			m_str.insert(0, aString);
			return *this;
		}
		
		CString& Prepend(const char* aString)
		{
			m_str.insert(0, aString);
			return *this;
		}

		void Clear()
		{
			m_str.clear();
		}

		CString& Printf(const char* aFormat, ...)
		{
			char dest[256];
			va_list ap;
			va_start(ap, aFormat);
			vsnprintf(dest, 256, aFormat, ap);
			va_end(ap);
			
			m_str = dest;
			return *this;
		}
		
	private:
	
		void SafeAssign(const char* aString)
		{
			if (aString == NULL)
				m_str.assign("");
			else
				m_str.assign(aString);
		}

	private:

		std::string m_str;
};

#endif /* CSTRING_H_ */
