/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) T Swann, Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CTextureManager.h

  Description :	

  Last Modified $Date: 2008/09/29 04:22:27 $

  $Revision: 1.4 $

*******************************************************************************/

#ifndef CTEXTUREMANAGER_H_
#define CTEXTUREMANAGER_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"
#include "CString.h"
#include "CGfx.h"

#include <ft2build.h>
#include FT_FREETYPE_H
#include FT_GLYPH_H

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************
class CFile;
class CFont;
class CCachedTexture;
class CCachedFont;

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************

//*********************************************************************************
//	
//*********************************************************************************
class CTexture : public CBase
{
	friend class CTextureManager;
	friend class CGfx;
	
	public:
	
		typedef enum _Format {
			rgba32, mono8
		} Format;
	
	public:

		CTexture(Format format = rgba32);
		virtual ~CTexture();

		bool Init(u32 width, u32 height);
		void Save(const CString & file_name);
		void Upload();
		void Swizzle();
		void Optimize();
		u8* GetPtr() const { return (m_pVRAM != NULL ? m_pVRAM : m_pBuffer); }

	protected:

		u32 m_nWidth; /* Width of the source */
		u32 m_nHeight; /* Height of the source */
		u32 m_nCanvasWidth; /* Width of the resulting texture */
		u32 m_nCanvasHeight; /* Height of the resulting texture */
		u8* m_pBuffer; /* The aligned pointer */
		u8* m_pOriginalAddress; /* The address allocated on init */
		u8* m_pVRAM; /* NULL until Upload is called */
		
		u32 m_pixelFormat; /* Dependant on the Format */
		u32 m_bpp; /* Bytes per pixel, dependant on the Format */
		u32 m_pixelComponent; /* Always GU_TCC_RGBA for now */
		u32 m_texEnv; /* Always GU_TEX_MODULATE for now */
		u32 m_texMin; /* Always GU_LINEAR for now */
		u32 m_texMag; /* Always GU_LINEAR for now */
		bool m_swizzled; /* Toggled by calls to Swizzle */
		
};

//*********************************************************************************
//	
//*********************************************************************************
class CTextureManager
{
	public:

		static CTexture* Create( const CString & filename, bool warn = false, CTexture* destTexture = NULL );
		static CCachedTexture* CreateCached( const CString & filename );
		static CFont* CreateFont( const CString & filename, u32 size, CFont* destFont = NULL );
		static CCachedFont* CreateCachedFont( const CString & filename, u32 size );

		static void screenshot(const char* filename);

	private:
        static CTexture* ReadBMP( CFile * const p_file, CTexture* destTexture = NULL );
		static CTexture* ReadPNG( CFile * const p_file, CTexture* destTexture = NULL );
		static CFont* ReadTTF( const CString & filename, u32 size, CFont* destFont = NULL );
		static CFont* RenderTTF( FT_Library& library, FT_Face& face, u32 fontSize, CFont* destFont = NULL );
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CTEXTUREMANAGER_H_ */
