/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) T Swann, Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CWindow.cpp

  Description :	

  Last Modified $Date: 2006/10/09 16:55:45 $

  $Revision: 1.1 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CWindow.h"
#include "CGfx.h"
#include "CFrameWork.h"
#include "CFont.h"
#include "CSkinManager.h"
#include "CTextureManager.h"
#include "CRenderable.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************
static std::list< CWindow* > s_RenderList;

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
void	CWindow::Open()
{
	s_RenderList.clear();
	CRenderable::Register( CRenderable::RO_WINDOWS, RenderAll );
}

//******************************************************************************
//	
//******************************************************************************
void	CWindow::Close()
{
	s_RenderList.clear();
	CRenderable::UnRegister( CRenderable::RO_WINDOWS, RenderAll );
}

//******************************************************************************
//	
//******************************************************************************
void	CWindow::RenderAll()
{
	for ( std::list< CWindow * >::reverse_iterator it = s_RenderList.rbegin(); it != s_RenderList.rend(); ++it )
	{
		if ( ( *it )->IsVisible() == true )
		{
			( *it )->Render();
		}
	}
}

//******************************************************************************
//	
//******************************************************************************
CWindow::CWindow() : m_bInFocus( false ),
	m_bVisible( true )
{
	s_RenderList.push_back( this );
}

//******************************************************************************
//	
//******************************************************************************
CWindow::~CWindow()
{
	s_RenderList.remove( this );
}

//******************************************************************************
//	
//******************************************************************************
void	CWindow::Render()
{

}

//******************************************************************************
//	
//******************************************************************************
void	CWindow::Process()
{

}

//******************************************************************************
//	
//******************************************************************************
bool	CWindow::ProcessInput()
{
	return false;
}

//******************************************************************************
//	
//******************************************************************************
bool	CWindow::HasFocus() const
{
	return m_bInFocus;
}

//******************************************************************************
//	
//******************************************************************************
void	CWindow::SetFocus( bool focus )
{
	m_bInFocus = focus;

	if ( focus == true )
	{
		CProcess::MoveToFrontOfList();

		s_RenderList.remove( this );
		s_RenderList.push_front( this );
	}
	else
	{
		CProcess::MoveToBackOfList();

		s_RenderList.remove( this );
		s_RenderList.push_back( this );
	}
}

//******************************************************************************
//	
//******************************************************************************
bool	CWindow::IsVisible() const
{
	return m_bVisible;
}

//******************************************************************************
//	
//******************************************************************************
void	CWindow::SetVisible( bool visible )
{
	m_bVisible = visible;
}



//*******************************  END OF FILE  ********************************
