/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CKeyboardDanzeff.cpp

  Description :	

  Last Modified $Date: 2006/11/22 02:37:40 $

  $Revision: 1.4 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CKeyboardDanzeff.h"
#include "CGfx.h"
#include "CInput.h"
#include <danzeff.h>

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************
CKeyboardDanzeff::CKeyboardDanzeff() : CKeyboard(),
	m_location(CKeyboardDanzeff::LOCATION_BOTTOM_RIGHT)
{
	danzeff_load();
	moveToLocation();
}

//******************************************************************************
//
//******************************************************************************
CKeyboardDanzeff::~CKeyboardDanzeff()
{
	danzeff_free();
}

//******************************************************************************
//
//******************************************************************************
bool CKeyboardDanzeff::doRender()
{
	if (danzeff_isinitialized())
	{
		danzeff_render();
		return true;
	}
	
	return false;
}

//******************************************************************************
//
//******************************************************************************
bool CKeyboardDanzeff::doProcess(SceCtrlData *aPadData)
{
	m_charKey = 0;
	m_controlKey = 0;
	unsigned int danzeffChar = danzeff_readInput(*aPadData);
	
	// Note that we don't use the DANZEFF_START button event to determine if the
	// keyboard should be disabled since that will happen on "button down" as
	// opposed to button released. Having it happen on button down means the HUH
	// will display briefly when the start button is pressed to disable the
	// keyboard.
	if (CInput::IsButtonReleased(CInput::START))
	{
		m_shouldDisable = true;
	}
	else if (danzeffChar == DANZEFF_SELECT)
	{
		m_location = (m_location + 1) % LOCATION_MAX;
		moveToLocation();
	}
	else
	{
		switch (danzeffChar)
		{
			case 0:
				break;
			case DANZEFF_LEFT:
				m_controlKey = CKeyboard::LEFT;
				break;
			case DANZEFF_RIGHT:
				m_controlKey = CKeyboard::RIGHT;
				break;
			case DANZEFF_UP:
				m_controlKey = CKeyboard::UP;
				break;
			case DANZEFF_DOWN:
				m_controlKey = CKeyboard::DOWN;
				break;
			case 8:
				m_controlKey = CKeyboard::BACKSPACE;
				break;
			case '\n':
				m_controlKey = CKeyboard::ENTER;
				break;
			default:
				m_charKey = danzeffChar;
				break;
		}
	}
	
	return (m_charKey || m_controlKey);
}

void CKeyboardDanzeff::moveToLocation()
{
	switch (m_location)
	{
		case CKeyboardDanzeff::LOCATION_BOTTOM_RIGHT:
			danzeff_moveTo(CGfx::s_ScreenWidth - DANZEFF_WIDTH, CGfx::s_ScreenHeight - DANZEFF_HEIGHT);
			break;
		case CKeyboardDanzeff::LOCATION_TOP_RIGHT:
			danzeff_moveTo(CGfx::s_ScreenWidth - DANZEFF_WIDTH, 0);
			break;
	}
}
