/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CKeyboardGUI.h

  Description :	

  Last Modified $Date: 2006/11/18 06:46:45 $

  $Revision: 1.4 $

*******************************************************************************/

#ifndef CKEYBOARDGUI_H_
#define CKEYBOARDGUI_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CKeyboard.h"
#include "CPspPadRepeat.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************

//******************************************************************************
//   Constants
//******************************************************************************
#define GUI_KB_CHARS_WIDE 10
#define GUI_KB_CHARS_HIGH 5

//******************************************************************************
//   Class definitions
//******************************************************************************
/* The CKeyboard class defines the interface PSPoste is to use when wanting to
 * poll for key presses when in text entry mode. See the CKeyboardNav class for
 * the class responsible for navigating the interface.
 */
class CKeyboardGUI : public CKeyboard
{
	public: /* Constructors & Destructors */
		CKeyboardGUI() : CKeyboard(),
			m_position(V2(378, 200)),
			m_selectPosition(V2(378, 200)),
			m_selected(0),
			m_shift(false),
			m_editMode(true),
			m_pspPadRepeat(PSP_CTRL_LEFT | PSP_CTRL_RIGHT | PSP_CTRL_UP | PSP_CTRL_DOWN | PSP_CTRL_SQUARE | PSP_CTRL_CIRCLE | PSP_CTRL_CROSS)
		{};
	
	protected: /* Overriding methods */
	
		virtual bool doRender();
		virtual bool doProcess(SceCtrlData *aPadData);
		virtual void reset();
		
	protected: /* Select cursor movement */
	
		void selectUp();
		void selectUpNoWrap();
		void selectDown();
		void selectDownNoWrap();
		void selectLeft();
		void selectLeftNoWrap();
		void selectRight();
		void selectRightNoWrap();
		void select();
		
	private: /* Helper functions */
	
		virtual void guiProcess();
		void editModeProcess();
		void cursorModeProcess();
		u16 getSelectedKey()
		{
			if (m_shift)
				return s_upperCharTable[m_selected / GUI_KB_CHARS_WIDE][m_selected % GUI_KB_CHARS_WIDE];
			return s_lowerCharTable[m_selected / GUI_KB_CHARS_WIDE][m_selected % GUI_KB_CHARS_WIDE];
		};
		
		u16 getKeyAt(unsigned int i, unsigned int j)
		{
			if (m_shift)
			{
				return s_upperCharTable[i][j];
			}
			else
			{				  	  	 
				return s_lowerCharTable[i][j];
			}
		};
		
	private: /* Control data */
	
		V2 m_position;
		V2 m_selectPosition;
		int m_selected;
		bool m_shift;
		bool m_editMode;
		
	protected:
		
		CPspPadRepeat m_pspPadRepeat;
		
	private: /* Static character tables */
	
		static u16 s_lowerCharTable[GUI_KB_CHARS_HIGH][GUI_KB_CHARS_WIDE];
		static u16 s_upperCharTable[GUI_KB_CHARS_HIGH][GUI_KB_CHARS_WIDE];

};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CKEYBOARDGUI_H_ */
