/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CKeyboardIrda.cpp

  Description :	

  Last Modified $Date: 2006/11/16 03:44:59 $

  $Revision: 1.2 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CKeyboardIrda.h"
#include "CKeyboardNavWrapper.h"
#include "irda_KeyboardLayouts.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************
bool CKeyboardIrda::doProcess(SceCtrlData *aPadData)
{
	int len;
	
	if (m_syncRequired)
	{
		// Let's look for the packet start
		len = sceIoRead(m_irdafd, m_irdaPacket, 1);
		if (len == 1 && m_irdaPacket[0] == 0xff)
		{
			len = sceIoRead(m_irdafd, &m_irdaPacket[1], 1);
			if (len == 1 && m_irdaPacket[1] == 0xc0)
			{
				// Alright, we're synced up
				m_bufLeft = IRDA_PACKET_SIZE - 2;
				m_syncRequired = false;
			}
		}
		
		len = 0;
	}
	else
	{
		len = sceIoRead(m_irdafd, &m_irdaPacket[IRDA_PACKET_SIZE - m_bufLeft], m_bufLeft);
	}
	
	if (len == 0)
	{
		processCode(0);
	}
	else
	{
		m_bufLeft -= len;
		if (m_bufLeft == 0)
		{
			if (m_irdaPacket[0] != 0xff || m_irdaPacket[1] != 0xc0)
			{
				// Problem: The irdaPacket is 6 bytes long, to get an entire
				// packet to process, we sometimes need more than one read. We
				// only process the packet when we have exactly 6 bytes. If one
				// read is missed or something weird happens, the start of the
				// packet may no longer be what it is suppose to be (0xffc0);
				// this value may be offset by some bytes. By setting the
				// syncRequired flag, everytime this method is called, we simply
				// wait for the next packet start to begin reading like normal
				// again.
				m_syncRequired = true;
				processCode(0);
			}
			else
			{
				processCode(thinkoutside(m_irdaPacket));
				m_bufLeft = IRDA_PACKET_SIZE;
			}
		}
		else
			processCode(0);
	}
	
	// Last chance; a keyChar may be generated by the repeat timer
	if (m_charKey == 0 && m_controlKey == 0)
		processCode(thinkoutside_repeat());
		
	// We do this to avoid the display from turning off, it basically mimics
	// pressing a PSP button
	if (m_charKey || m_controlKey)
		scePowerTick(0);
		
	if (CKeyboardNavWrapper::DeactivateControl())
	{
		m_shouldDisable = true;
		return false;
	}
	return (m_charKey || m_controlKey);
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardIrda::processCode(int code)
{
	if (code == 0)
	{
		m_charKey = 0;
		m_controlKey = 0;
	}
	else
	{
		if (code >= ' ' && code <= '~')
		{
			m_charKey = code;
		}
		else
		{
			// BEN TODO: Figure out home/end values
			switch (code)
			{
				case KB_LEFT:
					m_controlKey = CKeyboard::LEFT;
					break;
				case KB_RIGHT:
					m_controlKey = CKeyboard::RIGHT;
					break;
				case KB_UP:
					m_controlKey = CKeyboard::UP;
					break;
				case KB_DOWN:
					m_controlKey = CKeyboard::DOWN;
					break;
				case KB_BACKSPACE:
					m_controlKey = CKeyboard::BACKSPACE;
					break;
				case KB_TAB:
					m_controlKey = CKeyboard::TAB;
					break;
				case KB_ENTER:
					m_controlKey = CKeyboard::ENTER;
					break;
				case KB_PGUP:
					m_controlKey = CKeyboard::PAGE_UP;
					break;
				case KB_PGDN:
					m_controlKey = CKeyboard::PAGE_DOWN;
					break;
				case KB_ESC:
					m_controlKey = CKeyboard::ESC;
					break;
				case KB_DELETE:
					m_controlKey = CKeyboard::DELETE;
					break;
				default:
					m_controlKey = 0;
					m_charKey = 0;
					break;
					
			}
		}
	}
}
