/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CKeyboardPspirkeyb.cpp

  Description :	

  Last Modified $Date: 2008/09/24 01:54:31 $

  $Revision: 1.1 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CKeyboardPspirkeyb.h"
#include "CKeyboardNavWrapper.h"
#include "CLog.h"

#include <pspirkeyb.h>

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
struct Vt100EscapeEntry
{
	const char* sequence;
	enum CKeyboard::eKeyboardControlKey controlKey;
};

Vt100EscapeEntry vt100EscapeTable[] =
{
    { "\e[A", CKeyboard::UP },
    { "\e[B", CKeyboard::DOWN },
    { "\e[C", CKeyboard::RIGHT },
    { "\e[D", CKeyboard::LEFT },
    { "\e[[2~", CKeyboard::INSERT },
    { "\e[[3~", CKeyboard::DELETE },
    { "\e[OF", CKeyboard::END },
    { "\e[OH~", CKeyboard::HOME },
    { "\e[[5~", CKeyboard::PAGE_UP },
    { "\e[[6~", CKeyboard::PAGE_DOWN },
    { "\e[[Z", CKeyboard::TAB }
};

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************
bool CKeyboardPspirkeyb::doProcess(SceCtrlData *aPadData)
{
	int len;
	unsigned char buffer[255];
	
	if (pspIrKeybReadinput(buffer, &len) >= 0)
	{
		if (len == 1)
		{
			processCode(buffer[0]);
		}
		else if (len > 1)
		{
			processEscapeCode(buffer, len);
		}
		else
		{
			processCode(0);
		}
	}
	else
	{
		processCode(0);
	}
		
	if (CKeyboardNavWrapper::DeactivateControl())
	{
		m_shouldDisable = true;
		return false;
	}
	return (m_charKey || m_controlKey);
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardPspirkeyb::processCode(unsigned char code)
{
	if (code == 0)
	{
		/* Clear both control and char keys */
		m_charKey = 0;
		m_controlKey = 0;
	}
	else if (code >= ' ' && code <= '~')
	{
		/* This is an ASCII char */
		m_charKey = code;
	}
	else
	{
		/* Could be a non-escaped control code */
		switch (code)
		{
			case 0x08:
				m_controlKey = CKeyboard::BACKSPACE;
				break;
			case 0x09:
				m_controlKey = CKeyboard::TAB;
				break;
			case 0x0D:
				m_controlKey = CKeyboard::ENTER;
				break;
			case 0x1B:
				m_controlKey = CKeyboard::ESC;
				break;
			default:
				m_controlKey = 0;
				m_charKey = 0;
				break;
		}
	}
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardPspirkeyb::processEscapeCode(unsigned char* buffer, int len)
{
	u32 tableCount = sizeof(vt100EscapeTable) / sizeof(Vt100EscapeEntry);
	
	/* Ensure buffer is NULL terminated so we can us strcmp */
	buffer[len] = 0;
	
	for (u32 i = 0; i < tableCount; i++)
	{
		if (strcmp((char*)buffer, vt100EscapeTable[i].sequence) == 0)
		{
			m_controlKey = vt100EscapeTable[i].controlKey;
			return;
		}
	}
	
	/* No matching escape sequence found */
	m_controlKey = 0;
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardPspirkeyb::Initialize()
{
	int err = pspIrKeybInit("Data/pspirkeyb/pspirkeyb.ini", 0);
	if (err == PSP_IRKBD_RESULT_OK)
	{
		pspIrKeybOutputMode(PSP_IRKBD_OUTPUT_MODE_VT100);
	}
	else
	{
		// Non-fatel error, let's just log it
		CLog::Error("Error initializing pspirkeyb: %d\n", err);
	}
}
