/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CKeyboardPspirkeyb.h

  Description :	

  Last Modified $Date: 2008/09/24 01:54:31 $

  $Revision: 1.1 $

*******************************************************************************/

#ifndef CKEYBOARDPSPIRKEYB_H_
#define CKEYBOARDPSPIRKEYB_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CKeyboard.h"
#include <pspirkeyb.h>

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************
/* The CKeyboard class defines the interface PSPoste is to use when wanting to
 * poll for key presses when in text entry mode. See the CKeyboardNav class for
 * the class responsible for navigating the interface.
 */
class CKeyboardPspirkeyb : public CKeyboard
{
	public: /* Constructors & Destructors */
	
		CKeyboardPspirkeyb() : CKeyboard(),
			m_allowHUH(true)
		{
			m_inputEnabled = true; // We always want to process the irda keyboard
			Initialize();
		};
		
		~CKeyboardPspirkeyb()
		{
			pspIrKeybFinish();
		}
	
	protected: /* Overriding methods */
	
		virtual void enableInput() { m_allowHUH = false; };
		virtual void disableInput() { m_allowHUH = true; m_shouldDisable = false; };
		virtual void enableRender() {};
		virtual void disableRender() {};
		virtual bool doProcess(SceCtrlData *aPadData);
		virtual bool allowHUH() { return m_allowHUH; };
		
	private: /* Helper methods */
	
		void Initialize();
		void processCode(unsigned char code);
		void processEscapeCode(unsigned char* buffer, int len);
		
	private: /* Control data */
		
		bool m_allowHUH;
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CKEYBOARDPSPIRKEYB_H_ */
