/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CKeyboardPsprint.cpp

  Description :	

  Last Modified $Date: 2006/11/16 03:44:59 $

  $Revision: 1.2 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CKeyboardPsprint.h"
#include "CKeyboardNavWrapper.h"
#include "CInput.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************
bool CKeyboardPsprint::doRender()
{
	return false;
}

//******************************************************************************
//
//******************************************************************************
bool CKeyboardPsprint::doProcess(SceCtrlData *aPadData)
{
	// Clear the keys produced by the last call to Process
	m_charKey = 0;
	m_controlKey = 0;
	
	m_pspPadRepeat.process(aPadData);
	
	if (CInput::IsButtonClicked(CInput::RTRIGGER))	
		m_editMode = !m_editMode;
	
	if (m_editMode)
		editModeProcess();
	else
		cursorModeProcess();
		
	if (CKeyboardNavWrapper::DeactivateControl())
	{
		m_shouldDisable = true;
		return false;
	}
	
	return (m_charKey || m_controlKey);
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardPsprint::editModeProcess()
{
	p_spReadKey(&m_psprintKey);
	
	switch (m_psprintKey.keychar)
	{
		case 8:
			m_controlKey = CKeyboard::BACKSPACE;
			break;
		case 27:
			m_controlKey = CKeyboard::ESC;
			break;
		case 13:
			m_controlKey = CKeyboard::ENTER;
			break;
		case 37:
			m_controlKey = CKeyboard::LEFT;
			break;
		case 38:
			m_controlKey = CKeyboard::UP;
			break;
		case 39:
			m_controlKey = CKeyboard::RIGHT;
			break;
		case 40:
			m_controlKey = CKeyboard::DOWN;
			break;
		case 46:
			m_controlKey = CKeyboard::DELETE;
			break;
		case 36:
			m_controlKey = CKeyboard::HOME;
			break;
		case 35:
			m_controlKey = CKeyboard::END;
			break;
		case 33:
			m_controlKey = CKeyboard::PAGE_UP;
			break;
		case 34:
			m_controlKey = CKeyboard::PAGE_DOWN;
			break;
		default:
			m_charKey = m_psprintKey.keychar;
			break;
	}
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardPsprint::cursorModeProcess()
{
	u32 repeatButtons = m_pspPadRepeat.getRepeatButtons();
	
	if (repeatButtons & PSP_CTRL_UP)
	{
		m_controlKey = CKeyboard::UP;
	}
	else if (repeatButtons & PSP_CTRL_DOWN)
	{
		m_controlKey = CKeyboard::DOWN;
	}
	else if (repeatButtons & PSP_CTRL_LEFT)
	{
		m_controlKey = CKeyboard::LEFT;
	}
	else if (repeatButtons & PSP_CTRL_RIGHT)
	{
		m_controlKey = CKeyboard::RIGHT;
	}
	else if (repeatButtons & PSP_CTRL_CIRCLE)
	{
		m_controlKey = CKeyboard::BACKSPACE;
	}
	else if (repeatButtons & PSP_CTRL_SQUARE)
	{
		m_charKey = 32; // Space
	}
}

//******************************************************************************
//
//******************************************************************************
void CKeyboardPsprint::reset()
{
	m_editMode = true;
	m_pspPadRepeat.reset();
	memset(&m_psprintKey, 0, sizeof(m_psprintKey));
}
