/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	main_fw2x.c

  Description :	Main file for release AND psplink mode targetting 2.x firmwares.

  Last Modified $Date: 2008/09/27 22:08:42 $

  $Revision: 1.5 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include <psputility_netmodules.h>
#include <psputility_modules.h>
#include <pspthreadman.h>
#include "main_globals.h"
#include "pspsdk.h"
#include "CFrameWork.h"
#include "CConnectionManager.h"
#include "CLog.h"

PSP_MODULE_INFO("PSPoste", PSP_MODULE_USER, 0, 7);
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER);

PSP_MAIN_THREAD_STACK_SIZE_KB(MAIN_THREAD_STACK_SIZE);
PSP_HEAP_SIZE_KB(PSPOSTE_HEAP_SIZE_KB);

//******************************************************************************
// 
//******************************************************************************
int main(int argc, char* argv[])
{
	int err;
	SceUID thid; // Thread id of user thread
	SceUID connManThid; // Thread id of connection manager
	
	// Initialise the debug output so we can catch any early DebugScreenPrintfs
	pspDebugScreenInit();

	// Open the log file now to catch any early log messages
	CLog::Open();
	
	// Failing to load the IrDa module isn't fatal
	if ((err = sceUtilityLoadModule(PSP_MODULE_IRDA)) < 0)
	{
		CLog::Error("Error, could not load PSP_MODULE_IRDA: 0x%x\n", err);
	}
	
	if ((err = sceUtilityLoadNetModule(PSP_NET_MODULE_COMMON)) < 0)
	{
		CLog::Error("Failed to load PSP_NET_MODULE_COMMON: 0x%x\n", err);
	}
	else if ((err = sceUtilityLoadNetModule(PSP_NET_MODULE_INET)) < 0)
	{
		CLog::Error("Error, could not load PSP_NET_MODULE_INET: 0x%x\n", err);
	}
	else
	{
		// Start the connection manager thread
		connManThid = sceKernelCreateThread("Conn Man Thread",
		                                    CConnectionManager::ConnManThreadCB,
		                                    0x11, CONN_MAN_THREAD_STACK_SIZE,
		                                    PSP_THREAD_ATTR_USER, NULL);
	
		if (connManThid & 0x80000000)
		{
			CLog::Error("Failed to create the connection manager thread: 0x%x\n",
			            connManThid);
		}
		else
		{
			sceKernelStartThread(connManThid, 0, NULL);
		}
		
		// This path is used to set the working directory in the user thread, it's
		// also used in this function to reboot the app
		strncpy((char*)&s_ebootPath, argv[0], 255);
		
		// 0x11 is the priority, 0x40000 is the stack size and corresponds to the
		// main thread stack size as determined from the crt0.c _start function
		// Note that 0x2D000 is the max free memory with PSPLink running
		thid = sceKernelCreateThread("User Mode Thread", user_main, 0x11,
		                             USER_THREAD_STACK_SIZE,
		                             PSP_THREAD_ATTR_USER, NULL);
	
		if (thid & 0x80000000)
		{
			CLog::Error("Failed to create the user thread: 0x%x\n", thid);
		}
		else
		{
			// Start thread with no parameters
			sceKernelStartThread(thid, 0, NULL);
		}
		
		// Wait for the user thread to terminate
		sceKernelWaitThreadEnd(thid, NULL);
		
		// Kill the conn man then clean up
		CConnectionManager::RequestKill();
		sceKernelWaitThreadEnd(connManThid, NULL);
	}
	
	CLog::Close();
	sceKernelExitGame();
	return 0;
}
