#!/bin/sh
# Modeled off Dan Peori's toolchain.sh

########################
## MAIN CONFIGURATION ##
########################
LIBICONV="libiconv-1.12"
GETTEXT="gettext-0.17" # For libintl
LIBGPGERROR="libgpg-error-1.6"
LIBGCRYPT="libgcrypt-1.4.3"
GNUTLS="gnutls-2.4.2"
LIBGSASL="libgsasl-0.2.28"
BUILDDIR="/tmp/pspostesetup"

BUILD_ZLIB=1
BUILD_JPEG=1
BUILD_LIBPNG=1
BUILD_FREETYPE=1

BUILD_LIBICONV=1
BUILD_GETTEXT=1
BUILD_LIBGPGERROR=1
BUILD_LIBGCRYPT=1
BUILD_GNUTLS=1
BUILD_LIBGSASL=1
BUILD_DANZEFF=1
BUILD_VMIME=1
BUILD_CTEXT=1
BUILD_LIBPSPDBG=1
BUILD_PSPIRKEYB=1

VMIME_CO_REV="@473" # empty for "latest"
CTEXT_CO_DATE="Sat Jan  6 20:45:00 EST 2007" # empty for "latest"
LIBPSPDBG_CO_DATE="Tue Aug 26 20:00:00 EDT 2008" # empty for "latest
LIBPSPIRKEYB_CO_DATE="Tue Sep 23 21:40:00 EDT 2008" # empty for "latest
DANZEFF_CO_REV="@1240" # empty for "latest", must start with @
DANZEFF_INCLUDE_CO_REV="1028" # empty for "latest"

###########################
## SOFTWARE DEPENDENCIES ##
###########################

## Check for make.
if test "`gmake -v 2> /dev/null`" ; then
	MAKE="gmake"
else
	if test "`make -v 2> /dev/null`" ; then
		MAKE="make"
	else
		echo "ERROR: Please make sure you have GNU 'make' installed."
		exit
	fi
fi

## Check for patch.
if test "`gpatch -v 2> /dev/null`" ; then
	PATCH="gpatch -p1"
else
	if test "`patch -v 2> /dev/null`" ; then
		PATCH="patch -p1"
	else
		echo "ERROR: Please make sure you have 'patch' installed."
		exit
	fi
fi

## Check for wget.
if test "`wget -V 2> /dev/null`" ; then
	WGET="wget -c --passive-ftp"
else
	echo "ERROR: Please make sure you have 'wget' installed."
	exit
fi

## Check for cvs.
if test "`cvs -v 2> /dev/null`" ; then
	CVS="cvs"
else
	echo "ERROR: Please make sure you have 'cvs' installed."
	exit
fi

## Check for autoconf and automake
if ! test "`automake --version 2> /dev/null`" ; then
	echo "ERROR: Please make sure you have 'automake' installed."
	exit
fi

if ! test "`autoconf --version 2> /dev/null`" ; then
	echo "ERROR: Please make sure you have 'autoconf' installed."
	exit
fi

## Check for scons
if ! test "`scons --version 2> /dev/null`" ; then
	echo "ERROR: Please make sure you have 'scons' installed."
	exit
fi

## Check for sed
if ! test "`sed --version 2> /dev/null`" ; then
    echo "ERROR: Please make sure you have 'sed' installed."
    exit
fi

## Check for psp-config
if ! test "`psp-config -p 2> /dev/null`" ; then
	echo "ERROR: Please make sure you have 'psp-config' installed."
	exit
fi

## Check for svn.
if ! test "`svn --version 2> /dev/null`" ; then
	echo "ERROR: Please make sure you have 'svn' installed."
	exit
fi

echo "Dependencies passed, proceeding"

################################
## DOWNLOAD, UNPACK AND PATCH ##
################################

SRCDIR="`pwd`"

## Download the zlib source
if test "$BUILD_ZLIB" = "1" ; then
	svn co svn://svn.ps2dev.org/psp/trunk/zlib
fi

## Download the jpeg source
if test "$BUILD_JPEG" = "1" ; then
	svn co svn://svn.ps2dev.org/psp/trunk/jpeg
fi

## Download the libpng source
if test "$BUILD_LIBPNG" = "1" ; then
	svn co svn://svn.ps2dev.org/psp/trunk/libpng
fi

## Download the freetype source
if test "$BUILD_FREETYPE" = "1" ; then
	svn co svn://svn.ps2dev.org/psp/trunk/freetype
fi

## Download the iconv source.
if test "$BUILD_LIBICONV" = "1" ; then
	if test ! -f "$LIBICONV.tar.gz" ; then
		$WGET ftp://ftp.gnu.org/pub/gnu/libiconv/$LIBICONV.tar.gz || { echo "ERROR DOWNLOADING LIBICONV"; exit; }
	fi
fi

## Download the gsasl source.
if test "$BUILD_LIBGSASL" = "1" ; then
	if test ! -f "$LIBGSASL.tar.gz" ; then
		$WGET ftp://alpha.gnu.org/pub/gnu/gsasl/$LIBGSASL.tar.gz || { echo "ERROR DOWNLOADING LIBGSASL"; exit; }
	fi
fi

## Download the gettext source.
if test "$BUILD_GETTEXT" = "1" ; then
	if test ! -f "$GETTEXT.tar.gz" ; then
		$WGET ftp://ftp.gnu.org/pub/gnu/gettext/$GETTEXT.tar.gz || { echo "ERROR DOWNLOADING GETTEXT"; exit; }
	fi
fi

## Download the libgpg-error source.
if test "$BUILD_LIBGPGERROR" = "1" ; then
	if test ! -f "$LIBGPGERROR.tar.gz" ; then
		$WGET ftp://ftp.gnupg.org/gcrypt/libgpg-error/$LIBGPGERROR.tar.gz || { echo "ERROR DOWNLOADING LIBGPGERROR"; exit; }
	fi
fi

## Download the libgcrypt source.
if test "$BUILD_LIBGCRYPT" = "1" ; then
	if test ! -f "$LIBGCRYPT.tar.gz" ; then
		$WGET ftp://ftp.gnupg.org/gcrypt/libgcrypt/$LIBGCRYPT.tar.gz || { echo "ERROR DOWNLOADING LIBGCRYPT"; exit; }
	fi
fi

## Download the gnutls source.
if test "$BUILD_GNUTLS" = "1" ; then
	if test ! -f "$GNUTLS.tar.bz2" ; then
		$WGET ftp://ftp.gnupg.org/gcrypt/gnutls/$GNUTLS.tar.bz2 || { echo "ERROR DOWNLOADING GNUTLS"; exit; }
	fi
fi

## Download the vmime source.
if test "$BUILD_VMIME" = "1" -a ! -d vmime ; then
	svn co https://vmime.svn.sourceforge.net/svnroot/vmime/trunk/vmime$VMIME_CO_REV || { echo "ERROR DOWNLOADING VMIME"; exit; }
fi

## Download the CText source.
if test "$BUILD_CTEXT" = "1" -a ! -d CText ; then
	if test "$CTEXT_CO_DATE" = "" ; then
		$CVS -d:pserver:anonymous@psposte.cvs.sourceforge.net:/cvsroot/psposte co CText || { echo "ERROR DOWNLOADING CTEXT"; exit; }
	else
		$CVS -d:pserver:anonymous@psposte.cvs.sourceforge.net:/cvsroot/psposte co -D "$CTEXT_CO_DATE" CText || { echo "ERROR DOWNLOADING CTEXT"; exit; }
	fi
fi

## Download the libpspdbg source
if test "$BUILD_LIBPSPDBG" = "1" -a ! -d libpspdbg ; then
	if test "$LIBPSPDBG_CO_DATE" = "" ; then
		$CVS -d:pserver:anonymous@psposte.cvs.sourceforge.net:/cvsroot/psposte co libpspdbg || { echo "ERROR DOWNLOADING LIBPSPDBG"; exit; }
	else
		$CVS -d:pserver:anonymous@psposte.cvs.sourceforge.net:/cvsroot/psposte co -D "$LIBPSPDBG_CO_DATE" libpspdbg || { echo "ERROR DOWNLOADING LIBPSPDBG"; exit; }
	fi
fi

## Download the pspirkeyb source
if test "$BUILD_PSPIRKEYB" = "1" -a ! -d pspirkeyb ; then
	if test "$LIBPSPIRKEYB_CO_DATE" = "" ; then
		$CVS -d:pserver:anonymous@psposte.cvs.sourceforge.net:/cvsroot/psposte co pspirkeyb || { echo "ERROR DOWNLOADING PSPIRKEYB"; exit; }
	else
		$CVS -d:pserver:anonymous@psposte.cvs.sourceforge.net:/cvsroot/psposte co -D "$LIBPSPIRKEYB_CO_DATE" pspirkeyb || { echo "ERROR DOWNLOADING PSPIRKEYB"; exit; }
	fi
fi

## Download the Danzeff source.
if test "$BUILD_DANZEFF" = "1" -a ! -d danzeff ; then
	svn co https://svn.sourceforge.net/svnroot/pspradio/trunk/PSP/SharedLib/danzeff$DANZEFF_CO_REV danzeff
	svn cat https://svn.sourceforge.net/svnroot/pspradio/trunk/PSP/include.mak -r $DANZEFF_INCLUDE_CO_REV > danzeff/include.mak
fi

## Create and enter the temp directory.
mkdir -p "$BUILDDIR"; cd "$BUILDDIR"

## Unpack and patch the iconv source.
if test "$BUILD_LIBICONV" = "1" ; then
	rm -Rf $LIBICONV; gzip -cd "$SRCDIR/$LIBICONV.tar.gz" | tar xvf -
	cd $LIBICONV; cat "$SRCDIR/$LIBICONV.patch" | $PATCH || { echo "ERROR PATCHING LIBICONV"; exit; }
	chmod u+x "$BUILDDIR/$LIBICONV/configure.sh"
	cd ..
fi

## Unpack and patch the gettext source.
if test "$BUILD_GETTEXT" = "1" ; then
	rm -Rf $GETTEXT; gzip -cd "$SRCDIR/$GETTEXT.tar.gz" | tar xvf -
	cd $GETTEXT; cat "$SRCDIR/$GETTEXT.patch" | $PATCH || { echo "ERROR PATCHING GETTEXT"; exit; }
	chmod u+x "$BUILDDIR/$GETTEXT/configure.sh"
	cd ..
fi

## Unpack and patch the libgpg-error source.
if test "$BUILD_LIBGPGERROR" = "1" ; then
	rm -Rf $LIBGPGERROR; gzip -cd "$SRCDIR/$LIBGPGERROR.tar.gz" | tar xvf -
	cd $LIBGPGERROR; cat "$SRCDIR/$LIBGPGERROR.patch" | $PATCH || { echo "ERROR PATCHING LIBGPGERROR"; exit; }
	chmod u+x "$BUILDDIR/$LIBGPGERROR/configure.sh"
	cd ..
fi

## Unpack and patch the libgcrypt source.
if test "$BUILD_LIBGCRYPT" = "1" ; then
	rm -Rf $LIBGCRYPT; gzip -cd "$SRCDIR/$LIBGCRYPT.tar.gz" | tar xvf -
	cd $LIBGCRYPT; cat "$SRCDIR/$LIBGCRYPT.patch" | $PATCH || { echo "ERROR PATCHING LIBGCRYPT"; exit; }
	chmod u+x "$BUILDDIR/$LIBGCRYPT/configure.sh"
	cd ..
fi

## Unpack and patch the gnutls source.
if test "$BUILD_GNUTLS" = "1" ; then
	rm -Rf $GNUTLS; bzip2 -cd "$SRCDIR/$GNUTLS.tar.bz2" | tar xvf -
	cd $GNUTLS; cat "$SRCDIR/$GNUTLS.patch" | $PATCH || { echo "ERROR PATCHING GNUTLS"; exit; }
	chmod u+x "$BUILDDIR/$GNUTLS/configure.sh"
	cd ..
fi

## Patch the danzeff source.
if test "$BUILD_DANZEFF" = "1" ; then
	cp -a "$SRCDIR/danzeff" .
	cd danzeff; cat "$SRCDIR/danzeff.patch" | $PATCH || { echo "ERROR PATCHING DANZEFF"; exit; }
	cd ..
fi

## Unpack and patch the gsasl source.
if test "$BUILD_LIBGSASL" = "1" ; then
	rm -Rf $LIBGSASL; gzip -cd "$SRCDIR/$LIBGSASL.tar.gz" | tar xvf -
	cd $LIBGSASL; cat "$SRCDIR/$LIBGSASL.patch" | $PATCH || { echo "ERROR PATCHING LIBGSASL"; exit; }
	chmod u+x "$BUILDDIR/$LIBGSASL/configure.sh"
	cd ..
fi

## Enter the source directory to build the pre-req libs that don't require patching
cd $SRCDIR

############################
## BUILD AND INSTALL ZLIB ##
############################
if test "$BUILD_ZLIB" = "1" ; then
	cd zlib
	$MAKE clean; $MAKE || { echo "ERROR BUILDING ZLIB"; exit; }
	$MAKE install || { echo "ERROR INSTALLING ZLIB"; exit; }
	$MAKE clean
	cd ..
fi

############################
## BUILD AND INSTALL JPEG ##
############################
if test "$BUILD_JPEG" = "1" ; then
	cd jpeg
	$MAKE clean; $MAKE || { echo "ERROR BUILDING JPEG"; exit; }
	$MAKE install || { echo "ERROR INSTALLING JPEG"; exit; }
	$MAKE clean
	cd ..
fi

##############################
## BUILD AND INSTALL LIBPNG ##
##############################
if test "$BUILD_LIBPNG" = "1" ; then
	cd libpng
	$MAKE clean; $MAKE || { echo "ERROR BUILDING LIBPNG"; exit; }
	$MAKE install || { echo "ERROR INSTALLING LIBPNG"; exit; }
	$MAKE clean
	cd ..
fi

################################
## BUILD AND INSTALL FREETYPE ##
################################
if test "$BUILD_FREETYPE" = "1" ; then
	cd freetype
	sh autogen.sh || { echo "ERROR RUNNING AUTOGEN FOR FREETYPE"; exit; }
	LDFLAGS="-L$(psp-config --pspsdk-path)/lib -lc -lpspuser" ./configure --host psp --prefix=$(psp-config --psp-prefix) || { echo "ERROR CONFIGURING FREETYPE"; exit; }
	$MAKE clean; $MAKE || { echo "ERROR BUILDING FREETYPE"; exit; }
	$MAKE install; $MAKE || { echo "ERROR INSTALLING FREETYPE"; exit; }
	$MAKE clean
	cd ..
fi

## Go back to the build dir to build the patched libs
cd $BUILDDIR

################################
## BUILD AND INSTALL LIBICONV ##
################################
if test "$BUILD_LIBICONV" = "1" ; then
	cd $LIBICONV
	mkdir build-psp; cd build-psp
	../configure.sh
	$MAKE clean; $MAKE || { echo "ERROR BUILDING LIBICONV"; exit; }
	$MAKE install || { echo "ERROR INSTALLING LIBICONV"; exit; }
	$MAKE clean
	cd ..; cd ..
fi

#########################################
## BUILD AND INSTALL GETTEXT (libintl) ##
#########################################
if test "$BUILD_GETTEXT" = "1" ; then
	cd $GETTEXT
	mkdir build-psp; cd build-psp
	../configure.sh
	cd gettext-runtime/intl
	$MAKE clean; $MAKE || { echo "ERROR BUILDING GETTEXT"; exit; }
	$MAKE install || { echo "ERROR INSTALLING GETTEXT"; exit; }
	$MAKE clean
	cd ..; cd ..; cd ..; cd ..
fi

###################################
## BUILD AND INSTALL LIBGPGERROR ##
###################################
if test "$BUILD_LIBGPGERROR" = "1" ; then
	cd $LIBGPGERROR
	mkdir build-psp; cd build-psp
	../configure.sh
	$MAKE clean; $MAKE || { echo "ERROR BUILDING LIBGPGERROR"; exit; }
	$MAKE install || { echo "ERROR INSTALLING LIBGPGERROR"; exit; }
	$MAKE clean
	cd ..; cd ..
fi

#################################
## BUILD AND INSTALL LIBGCRYPT ##
#################################
if test "$BUILD_LIBGCRYPT" = "1" ; then
	cd $LIBGCRYPT
	mkdir build-psp; cd build-psp
	../configure.sh
	$MAKE clean; $MAKE || { echo "ERROR BUILDING LIBGCRYPT"; exit; }
	$MAKE install || { echo "ERROR INSTALLING LIBGCRYPT"; exit; }
	$MAKE clean
	cd ..; cd ..
fi

##############################
## BUILD AND INSTALL GNUTLS ##
##############################
if test "$BUILD_GNUTLS" = "1" ; then
	cd $GNUTLS
	mkdir build-psp; cd build-psp
	../configure.sh
	$MAKE clean; $MAKE || { echo "ERROR BUILDING GNUTLS"; exit; }
	$MAKE install || { echo "ERROR INSTALLING GNUTLS"; exit; }
	$MAKE clean
	cd ..; cd ..
fi

################################
## BUILD AND INSTALL LIBGSASL ##
################################
if test "$BUILD_LIBGSASL" = "1" ; then
	cd $LIBGSASL
	mkdir build-psp; cd build-psp
	../configure.sh
	$MAKE clean; $MAKE || { echo "ERROR BUILDING LIBGSASL"; exit; }
	$MAKE install || { echo "ERROR INSTALLING LIBGSASL"; exit; }
	$MAKE clean
	cd ..; cd ..
fi

###############################
## BUILD AND INSTALL DANZEFF ##
###############################
if test "$BUILD_DANZEFF" = "1" ; then
    cd danzeff
	make -f Makefile.gu || { echo "ERROR BUILDING DANZEFF"; exit; }
	cp libdanzeffgu.a `psp-config -P`/lib && cp danzeff.h `psp-config -P`/include || { echo "ERROR INSTALLING DANZEFF"; exit; }
fi

#####################################
## PATCH, BUILD, AND INSTALL VMIME ##
#####################################
if test "$BUILD_VMIME" = "1" ; then
	export PKG_CONFIG_PATH=$PKG_CONFIG_PATH:`psp-config -P`/lib/pkgconfig
	cd $SRCDIR/vmime; cat "$SRCDIR/vmime.patch" | $PATCH || { echo "ERROR PATCHING VMIME"; exit; }
	# NOTE1: If you have automake 1.10, you'll need to change the "bootstrap" file to set amvers="" on line 23... this is because the version compare is stupid
	# NOTE2: If you get the following error: "configure.in:75: required file `autotools/config.rpath' not found", just copy config.sub to config.rpath
	scons prefix=`psp-config --psp-prefix` with_platforms="psp posix" with_messaging_protocols="pop3 smtp maildir" with_sasl=yes with_tls=yes autotools || { echo "ERROR RUNNING SCONS"; exit; }
	chmod u+x configure.sh

	# Because the autotools are generated by scons, we must "patch" the
	# config.sub. Note that we use sed to do a search and replace 'cause the
	# generated config.sub depends on the version of autotools and a bunch of
	# environment stuff.
	sed -i 's/^\tmsdos)$/\tpsp)\n\t\tbasic_machine=mipsallegrexel-psp\n\t\tos=-elf\n\t\t;;\n\tmsdos)/' autotools/config.sub || { echo "ERROR COPYING VMIME CONFIG.SUB"; exit; }

	mkdir build-psp; cd build-psp
	../configure.sh
	$MAKE clean; $MAKE || { echo "ERROR BUILDING VMIME"; exit; }
	$MAKE install || { echo "ERROR INSTALLING VMIME"; exit; }
	$MAKE clean
	cd ..; cd ..
fi

#############################
## BUILD AND INSTALL CTEXT ##
#############################
if test "$BUILD_CTEXT" = "1" ; then
	cd $SRCDIR/CText
	$MAKE -f Makefile.psp clean; $MAKE -f Makefile.psp || { echo "ERROR BUILDING CTEXT"; exit; }
	$MAKE -f Makefile.psp install || { echo "ERROR INSTALLING CTEXT"; exit; }
	$MAKE -f Makefile.psp clean
	cd ..
fi

#################################
## BUILD AND INSTALL LIBPSPDBG ##
#################################
if test "$BUILD_LIBPSPDBG" = "1" ; then
	cd $SRCDIR/libpspdbg
	$MAKE clean; $MAKE || { echo "ERROR BUILDING LIBPSPDBG"; exit; }
	$MAKE install || { echo "ERROR INSTALLING LIBPSPDBG"; exit; }
	$MAKE clean
	cd ..
fi

#################################
## BUILD AND INSTALL PSPIRKEYB ##
#################################
if test "$BUILD_PSPIRKEYB" = "1" ; then
	cd $SRCDIR/pspirkeyb
	$MAKE clean; $MAKE || { echo "ERROR BUILDING PSPIRKEYB"; exit; }
	$MAKE install || { echo "ERROR INSTALLING PSPIRKEYB"; exit; }
	$MAKE clean
	cd ..
fi

echo "Success!"
