/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	user_main.c

  Description :	

  Last Modified $Date: 2008/09/10 03:14:08 $

  $Revision: 1.4 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "main_globals.h"
#include "CFrameWork.h"
#include "CPSPoste.h"
#include "CFileSystem.h"
#include <pspdbg.h> // For psp_trace_init/kill

//******************************************************************************
//   Global Variables
//******************************************************************************
char s_ebootPath[256];

//******************************************************************************
// 
//******************************************************************************
int	user_main(SceSize argc, void* argp)
{
	/* This code must run in user mode. */
	int ret;
	
	// Set the working folder
	CString	drive, dir;
	CFileSystem::SplitPath( CString( s_ebootPath ), &drive, &dir, NULL, NULL );
	drive += dir;
	CFileSystem::SetRoot( drive );
	
	// Open the trace log in safe mode; the log is opened/closed for every trace
	psp_trace_init("psposte_trace.log", 0, 1, 0);
	
	// Open the framework
	if ( CFrameWork::Open() == true )
	{
		// We install a catch-all exception handler to alert the user of any unhandled vmime exceptions
		try {
			// Create the application
			CPSPoste::Create();
		} catch (std::exception& e) {
			CLog::Error(e.what());
			ASSERT(0, "Exception occurred, see err.log for details.");
			return UNHANDLED_CXX_EXCEPTION_RETURN_CODE;
		}

		// The main loop
		while ((ret = CFrameWork::Process()) == 0)
		{
		}
	}
	else
	{
		ASSERT(0, "CFrameWork::Open()");
	}
	
	psp_trace_kill();

	return ret;
}
